@@ -21,7 +21,30 @@
 #ifndef XSTRING_H
 #define XSTRING_H
 
-#include <string.h>
+/* Declare string and memory handling routines.  Take care that an ANSI
+   string.h and pre-ANSI memory.h might conflict, and that memory.h and
+   strings.h conflict on some systems.  */
+
+#if STDC_HEADERS || HAVE_STRING_H
+# include <string.h>
+# if !STDC_HEADERS && HAVE_MEMORY_H
+#  include <memory.h>
+# endif
+#else
+# include <strings.h>
+# ifndef strchr
+#  define strchr index
+# endif
+# ifndef strrchr
+#  define strrchr rindex
+# endif
+# ifndef memcpy
+#  define memcpy(Dst, Src, Num) bcopy (Src, Dst, Num)
+# endif
+# ifndef memcmp
+#  define memcmp(Src1, Src2, Num) bcmp (Src1, Src2, Num)
+# endif
+#endif
 
 static inline int xstrcmp(const char *s1,const char *s2)
 {
