@@ -18,6 +18,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: ArgV.cc,v 1.7 1999/10/04 17:11:01 lav Exp $ */
+
 #include <config.h>
 
 #include "getopt.h"
@@ -55,9 +57,7 @@ char *ArgV::getnext()
 {
    if(++ind>=c)
    {
-      ind=c-1;
-      if(ind<0)
-	 ind=0;
+      ind=c; // so that getcurr will return 0
       return 0;
    }
    return v[ind];
@@ -69,7 +69,7 @@ void ArgV::back()
       ind--;
 }
 
-char *ArgV::Combine(int start)
+char *ArgV::Combine(int start) const
 {
    int	 i;
    char  *res;
@@ -110,7 +110,41 @@ void ArgV::setarg(int n,const char *s)
       Append(s);
    else if(n<count() && n>=0)
    {
-      free(v[n]);
+      xfree(v[n]);
       v[n]=xstrdup(s);
    }
 }
+
+void ArgV::delarg(int n)
+{
+   if(n<count() && n>=0)
+   {
+      if(n<ind)
+	 ind--;
+      xfree(v[n]);
+      // copy with trailing null pointer
+      memmove(v+n,v+n+1,(count()-n)*sizeof(*v));
+      c--;
+   }
+}
+void ArgV::insarg(int n,const char *s)
+{
+   if(n==count())
+      Append(s);
+   else
+   {
+      v=(char**)xrealloc(v,sizeof(*v)*(c+2));
+      // copy with trailing null pointer
+      memmove(v+n+1,v+n,(count()-n+1)*sizeof(*v));
+      c++;
+      v[n]=xstrdup(s);
+   }
+}
+
+int ArgV::getopt_long(const char *opts,const struct option *lopts,int *lind)
+{
+   optind=ind;
+   int r=::getopt_long(c,v,opts,lopts,lind);
+   ind=optind;
+   return r;
+}
