@@ -18,11 +18,11 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: ArgV.h,v 1.8 1999/08/16 11:37:11 lav Exp $ */
+
 #ifndef ARGV_H
 #define ARGV_H
 
-#include "getopt.h"
-
 class ArgV
 {
    char **v;
@@ -40,27 +40,33 @@ public:
    ArgV(int new_c,const char * const *new_v) { Init(new_c,new_v); }
    ~ArgV() { Empty(); }
 
-   char *Combine(int start_index=0);
+   char *Combine(int start_index=0) const;
+   char *CombineQuoted(int start_index=0) const;
 
+   int getopt_long(const char *opts,const struct option *lopts,int *lind);
    int getopt(const char *opts)
-      { optind=ind; int r=::getopt(c,v,opts); ind=optind; return r; }
-   int getopt_long(const char *opts,const struct option *lopts,int *lind)
-      { optind=ind; int r=::getopt_long(c,v,opts,lopts,lind); ind=optind; return r; }
+      {
+	 return getopt_long(opts,0,0);
+      }
 
    void rewind();
    char *getnext();
 
-   char *getarg(int n)
+   char *getarg(int n) const
       {
 	 if(n>=c)
 	    return 0;
 	 return v[n];
       }
-   char *getcurr() { return getarg(ind); }
+   char *getcurr() const { return ind<c?getarg(ind):0; }
+   int getindex() const { return ind; }
    void setarg(int n,const char *s);
-   char *a0() { return getarg(0); }
+   void delarg(int n);
+   void insarg(int n,const char *s);
+   char *a0() const { return getarg(0); }
    void back();
-   int count() { return c; }
+   int count() const { return c; }
+   char **GetV() const { return v; }
 };
 
 #endif//ARGV_H
