@@ -18,6 +18,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: CatJob.cc,v 1.10 1999/05/31 12:34:48 lav Exp $ */
+
 #include <config.h>
 #include <stdio.h>
 #include <errno.h>
@@ -26,15 +28,12 @@
 #include "xmalloc.h"
 
 #include "CatJob.h"
-
-bool has_wildcard(char *s)
-{
-   return strchr(s,'*') || strchr(s,'?');
-}
+#include "ArgV.h"
+#include "Filter.h"
 
 int   CatJob::Done()
 {
-   return args==0 && curr==0 && filter_wait==0 && global==0;
+   return curr==0 && filter_wait==0 && global==0;
 }
 
 void  CatJob::ShowRunStatus(StatusLine *s)
@@ -54,13 +53,9 @@ void  CatJob::ShowRunStatus(StatusLine *s)
 
 void  CatJob::PrintStatus(int verbose)
 {
-   SessionJob::PrintStatus(verbose);
-   if(Done())
-   {
-      XferJob::PrintStatus(verbose);
+   if(!session)
       return;
-   }
-   if(curr==0)
+   if(!Done() && curr==0)
    {
       putchar('\t');
       puts(_("Waiting for filter to terminate"));
@@ -76,67 +71,20 @@ int   CatJob::Do()
    int m=STALL;
    int res;
 
-   while(args)
-   {
-      // globbing
-      if(!rg)
-      {
-	 for(;;)
-	 {
-	    char *a=args->getnext();
-	    if(!a)
-	       break;
-	    if(has_wildcard(a))
-	    {
-	       rg=new RemoteGlob(session,a);
-	       rg->RestrictPath();
-	       m=MOVED;
-	       break;
-	    }
-	    args_globbed->Append(a);
-	 }
-      }
-      if(!rg)
-      {
-	 delete args;
-	 args=0;
-	 args_globbed->rewind();
-	 NextFile();
-	 m=MOVED;
-      }
-      else
-      {
-	 if(!rg->Done())
-	    return m;
-
-	 char **files,**i;
-	 m=MOVED;
-
-	 files=rg->GetResult();
-	 if(rg->Error())
-	    eprintf("rglob: %s - %s\n",rg->pattern,session->StrError(rg->ErrorCode()));
-	 else if(!files)
-	    eprintf(_("%s: no files found\n"),rg->pattern);
-	 else
-	 {
-	    for(i=files; *i; i++)
-	       args_globbed->Append(*i);
-	 }
-	 delete rg;
-	 rg=0;
-      }
-   }
-
    if(filter_wait)
    {
-      if(filter_wait->Done())
-      {
-	 delete filter_wait;
-	 filter_wait=0;
-   	 m=MOVED;
-      }
-      else
+      if(!filter_wait->Done())
 	 return m;
+      delete filter_wait;
+      filter_wait=0;
+      m=MOVED;
+   }
+
+   if(curr==0)
+   {
+      NextFile();
+      if(curr)
+	 m=MOVED;
    }
 
    if(curr==0)
@@ -163,10 +111,10 @@ int   CatJob::Do()
       {
 	 offset=0;
 	 m=MOVED;
-	 session->Open(curr,Ftp::RETRIEVE,offset);
+	 session->Open(curr,FA::RETRIEVE,offset);
       }
       res=TryRead(session);
-      if(res<0 && res!=Ftp::DO_AGAIN)
+      if(res<0 && res!=FA::DO_AGAIN)
       {
 	 NextFile();
 	 return MOVED;
@@ -179,7 +127,7 @@ int   CatJob::Do()
    if(res<0)
    {
       NextFile();
-      if(local==global)
+      if(local==global) // global filter failed, cannot do anything.
       {
 	 while(curr)
 	 {
@@ -197,9 +145,6 @@ int   CatJob::Do()
 
 void CatJob::NextFile()
 {
-   if(!args_globbed)
-      return;
-
    if(curr)
    {
       if(filter_wait)
@@ -212,7 +157,13 @@ void CatJob::NextFile()
       local=0;
    }
 
-   XferJob::NextFile(args_globbed->getnext());
+   if(!args)
+   {
+      XferJob::NextFile(0);
+      return;
+   }
+
+   XferJob::NextFile(args->getnext());
    if(!curr)
       return;
    if(for_each)
@@ -235,10 +186,6 @@ CatJob::~CatJob()
       delete filter_wait;
    if(args)
       delete args;
-   if(args_globbed)
-      delete args_globbed;
-   if(rg)
-      delete rg;
 };
 
 int CatJob::AcceptSig(int sig)
@@ -263,23 +210,35 @@ int CatJob::AcceptSig(int sig)
    return MOVED;
 }
 
-CatJob::CatJob(FileAccess *new_session,FDStream *new_global,ArgV *args) : XferJob(new_session)
+void CatJob::Init()
 {
-   global=new_global;
+   global=0;
    filter_wait=0;
    local=0;
    for_each=0;
 
-   this->args=args;
-   rg=0;
-   args_globbed=new ArgV(args->a0());
-   op=args_globbed->a0();
+   args=0;
+   op="cat";
+
+   print_run_status=false;
+}
+
+CatJob::CatJob(FileAccess *new_session,FDStream *new_global,ArgV *new_args)
+   : XferJob(new_session)
+{
+   Init();
+
+   global=new_global;
+
+   args=new_args;
+   args->rewind();
+   op=args->a0();
 
    if(!strcmp(op,"more") || !strcmp(op,"zmore"))
    {
       if(!global)
       {
-	 char *pager=getenv("PAGER");
+	 const char *pager=getenv("PAGER");
 	 if(pager==NULL)
 	    pager="more";
 	 global=new OutputFilter(pager);
@@ -298,3 +257,16 @@ CatJob::CatJob(FileAccess *new_session,FDStream *new_global,ArgV *args) : XferJo
 
    print_run_status=!global->usesfd(1);
 }
+
+CatJob::CatJob(FDStream *g,char *data,int data_len) : XferJob(0)
+{
+   Init();
+   global=g;
+   if(!global)
+      global=new FDStream(1,"<stdout>");
+   buffer=data;
+   in_buffer=data_len;
+   got_eof=true;
+   local=global;
+   curr="<memory>";
+}
