@@ -18,31 +18,30 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: CatJob.h,v 1.4 1999/05/31 12:34:48 lav Exp $ */
+
 #ifndef CATJOB_H
 #define CATJOB_H
 
-#include "Job.h"
-#include "ProcWait.h"
-#include "Filter.h"
-#include "ArgV.h"
-#include "xmalloc.h"
 #include "XferJob.h"
-#include "rglob.h"
+
+class ArgV;
+class FDStream;
 
 class CatJob : public XferJob
 {
+   void Init();
+
 protected:
    FDStream *global;
    FDStream *local;
    FDStream *filter_wait;
 
    ArgV	 *args;
-   ArgV	 *args_globbed;
-   RemoteGlob *rg;
 
    void	 NextFile();
 
-   char	 *for_each;
+   const char *for_each;
 
 public:
    int	 Do();
@@ -54,6 +53,7 @@ public:
    void	 PrintStatus(int);
 
    CatJob(FileAccess *s,FDStream *global,ArgV *args);
+   CatJob(FDStream *global,char *buffer,int size);
    virtual ~CatJob();
 };
 
