@@ -31,8 +31,8 @@ class CharReader : public SMTask
    int	 Do();
 
 public:
-   static const NOCHAR=-2;
-   static const EOFCHAR=-1;
+   static const int NOCHAR=-2;
+   static const int EOFCHAR=-1;
 
    int	 GetChar() { return ch; };
 
