@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-1997 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-1999 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,6 +18,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: CmdExec.cc,v 1.39.2.1 2000/03/09 14:17:19 lav Exp $ */
+
 #include <config.h>
 #include <errno.h>
 #include <unistd.h>
@@ -28,29 +30,46 @@
 #include "xstring.h"
 #include "SignalHook.h"
 #include "alias.h"
-extern "C" {
-#include <readline/readline.h>
-}
+#include "misc.h"
+#include "ResMgr.h"
+#include "module.h"
+
+#define RL_PROMPT_START_IGNORE	'\001'
+#define RL_PROMPT_END_IGNORE	'\002'
+
+#define super SessionJob
 
 static ResDecl
-   res_long_running	   ("cmd:long-running","30",ResMgr::UNumberValidate,0),
+   res_long_running	   ("cmd:long-running",	"30",ResMgr::UNumberValidate,0),
    res_remote_completion   ("cmd:remote-completion","on",ResMgr::BoolValidate,0),
-   res_prompt		   ("cmd:prompt","lftp> ",0,0),
-   res_default_ls	   ("cmd:ls-default","",0,0),
-   res_csh_history	   ("cmd:csh-history","off",ResMgr::BoolValidate,0);
+   res_prompt		   ("cmd:prompt",	"lftp> ",0,0),
+   res_default_ls	   ("cmd:ls-default",	"",0,0),
+   res_csh_history	   ("cmd:csh-history",	"off",ResMgr::BoolValidate,0),
+   res_verify_path	   ("cmd:verify-path",	"yes",ResMgr::BoolValidate,0),
+   res_verify_host	   ("cmd:verify-host",	"yes",ResMgr::BoolValidate,0),
+   res_at_exit		   ("cmd:at-exit",	"",   0,0),
+   res_fail_exit	   ("cmd:fail-exit",	"no", ResMgr::BoolValidate,0),
+   res_verbose		   ("cmd:verbose",	"no", ResMgr::BoolValidate,0);
 
 CmdExec	 *CmdExec::cwd_owner=0;
+CmdExec	 *CmdExec::chain=0;
+
+void  CmdExec::SetCWD(const char *c)
+{
+   xfree(cwd);
+   cwd=xstrdup(c);
+   if(cwd_owner==this)
+      cwd_owner=0;
+}
 
 void  CmdExec::SaveCWD()
 {
+   cwd=xgetcwd();
    if(cwd==0)
-      cwd=(char*)xmalloc(1024);
-   if(getcwd(cwd,1024)==0)
    {
       // A bad case, but we can do nothing ( xgettext:c-format )
-      eprintf(_("Warning: getcwd() failed\n"));
-      free(cwd);
-      cwd=0;
+      eprintf(_("Warning: getcwd() failed: %s\n"),strerror(errno));
+      cwd_owner=this;
    }
    else
    {
@@ -58,12 +77,27 @@ void  CmdExec::SaveCWD()
 	 cwd_owner=this;
    }
 }
-void  CmdExec::RestoreCWD()
+int  CmdExec::RestoreCWD()
 {
-   if(cwd==0 || cwd_owner==this)
-      return;
+   if(cwd_owner==this)
+      return 0;
+   if(cwd==0)
+      goto fail;
    if(chdir(cwd)==0)
+   {
       cwd_owner=this;
+      return 0;
+   }
+   else
+   {
+      eprintf(_("Warning: chdir(%s) failed: %s\n"),cwd,strerror(errno));
+   fail:
+      // can't run further commands in wrong directory
+      while(!Done())
+	 RemoveFeeder();
+      exit_code=1;
+      return -1;
+   }
 }
 
 void CmdExec::FeedCmd(const char *c)
@@ -79,7 +113,6 @@ void CmdExec::FeedCmd(const char *c)
    memmove(cmd_buf,next_cmd,len);
    cmd_buf=next_cmd=(char*)xrealloc(cmd_buf,len+strlen(c)+1);
    strcpy(next_cmd+len,c);
-//    printf("after feed: next_cmd=`%s'\n",next_cmd);
 };
 
 void CmdExec::PrependCmd(const char *c)
@@ -109,8 +142,7 @@ int CmdExec::find_cmd(const char *cmd_name,const struct cmd_rec **ret)
 {
    int part=0;
    const cmd_rec *c;
-
-   for(c=cmd_table; c->name; c++)
+   for(c=dyn_cmd_table?dyn_cmd_table:static_cmd_table; c->name; c++)
    {
       if(!strcmp(c->name,cmd_name))
       {
@@ -123,6 +155,7 @@ int CmdExec::find_cmd(const char *cmd_name,const struct cmd_rec **ret)
 	 *ret=c;
       }
    }
+
    if(part!=1)
       *ret=0;
    return part;
@@ -133,6 +166,12 @@ void  CmdExec::exec_parsed_command()
    switch(condition)
    {
    case(COND_ANY):
+      if(exit_code!=0 && bool(res_fail_exit.Query(0)))
+      {
+	 while(!Done())
+	    RemoveFeeder();
+	 return;
+      }
       break;
    case(COND_AND):
       if(exit_code!=0)
@@ -151,6 +190,10 @@ void  CmdExec::exec_parsed_command()
    SignalHook::ResetCount(SIGHUP);
    SignalHook::ResetCount(SIGTSTP);
 
+   bool did_default=false;
+
+restart:
+
    const struct cmd_rec *c;
    const char *cmd_name=args->getarg(0);
    int part=find_cmd(cmd_name,&c);
@@ -160,17 +203,39 @@ void  CmdExec::exec_parsed_command()
       eprintf(_("Ambiguous command `%s'.\n"),cmd_name);
    else
    {
-      RestoreCWD();
+      if(RestoreCWD()==-1)
+	 return;
 
       args->setarg(0,c->name); // in case it was abbreviated
 
       if(cmd==0)
 	 cmd=args->Combine();
 
-      Job *(CmdExec::*func)()=c->func;
-      waiting=(this->*func)();
+      if(c->creator==0)
+      {
+	 if(did_default)
+	 {
+	    eprintf("Module for command `%s' did not register the command.\n",cmd_name);
+	    exit_code=1;
+	    return;
+	 }
+	 waiting=default_cmd();
+	 did_default=true;
+      }
+      else
+      {
+	 waiting=c->creator(this);
+      }
       if(waiting==this) // builtin
+      {
+	 if(builtin==BUILTIN_EXEC_RESTART)
+	 {
+	    waiting=0;
+	    builtin=BUILTIN_NONE;
+	    goto restart;
+	 }
 	 return;
+      }
       if(waiting)
       {
 	 waiting->parent=this;
@@ -181,7 +246,8 @@ void  CmdExec::exec_parsed_command()
 	    waiting->cmdline=cmd;
 	    cmd=0;
       	 }
-	 waiting->Fg();
+	 if(fg && !background)
+	    waiting->Fg();
       }
       if(background)
       {
@@ -199,6 +265,7 @@ void  CmdExec::exec_parsed_command()
 		     waiting->cmdline?waiting->cmdline:"?");
 		  waiting->PrintStatus(1);
 	       }
+	       last_bg=waiting->jobno;
 	       waiting=0;
 	    }
 	 }
@@ -268,11 +335,12 @@ int CmdExec::Do()
       {
       case(BUILTIN_CD):
 	 res=session->Done();
-	 if(res==Ftp::OK)
+	 if(res==FA::OK)
 	 {
 	    // done
-	    status_line->Show("");
-	    if(interactive)
+	    if(status_line)
+	       status_line->Clear();
+	    if(interactive || verbose)
 	    {
 	       const char *cwd=session->GetCwd();
 	       eprintf(_("cd ok, cwd=%s\n"),cwd?cwd:"~");
@@ -281,16 +349,19 @@ int CmdExec::Do()
 	    session->Close();
 	    exit_code=0;
 	    waiting=0;
+	    builtin=BUILTIN_NONE;
 	    beep_if_long();
 	    return MOVED;
 	 }
 	 if(res<0)
 	 {
 	    // error
-	    status_line->Show("");
+	    if(status_line)
+	       status_line->Clear();
 	    eprintf("%s: %s\n",args->getarg(0),session->StrError(res));
 	    session->Close();
 	    waiting=0;
+	    builtin=BUILTIN_NONE;
 	    beep_if_long();
 	    exit_code=1;
 	    return MOVED;
@@ -299,11 +370,13 @@ int CmdExec::Do()
 
       case(BUILTIN_OPEN):
 	 res=session->Done();
-	 if(res==Ftp::OK)
+	 if(res==FA::OK)
 	 {
-	    status_line->Show("");
+	    if(status_line)
+	       status_line->Clear();
 	    session->Close();
 	    waiting=0;
+	    builtin=BUILTIN_NONE;
 	    beep_if_long();
 	    exit_code=0;
 
@@ -311,30 +384,104 @@ int CmdExec::Do()
 	 }
 	 if(res<0)
 	 {
-	    status_line->Show("");
+	    if(status_line)
+	       status_line->Clear();
 	    eprintf("%s: %s\n",args->getarg(0),session->StrError(res));
 	    session->Close();
 	    waiting=0;
+	    builtin=BUILTIN_NONE;
 	    beep_if_long();
 	    exit_code=1;
 	    return MOVED;
 	 }
 	 break;
+      case(BUILTIN_GLOB):
+	 if(glob->Error())
+	 {
+	    if(status_line)
+	       status_line->Clear();
+	    eprintf("%s: %s\n",args->getarg(0),glob->ErrorText());
+	 }
+	 else if(glob->Done())
+	 {
+	    const char *const* glob_res=glob->GetResult();
+	    if(glob_res)
+	    {
+	       while(*glob_res)
+	       {
+		  args_glob->Append(*glob_res);
+		  glob_res++;
+	       }
+	    }
+	 }
+	 if(glob->Done() || glob->Error())
+	 {
+	    delete glob;
+	    glob=0;
+	    const char *pat=args->getnext();
+	    if(!pat)
+	    {
+	       // it was last argument
+	       delete args;
+	       args=args_glob;
+	       args_glob=0;
+	       waiting=0;
+	       builtin=BUILTIN_NONE;
+	       if(status_line)
+		  status_line->Clear();
+	       exit_code=prev_exit_code;
+	       exec_parsed_command();
+	       return MOVED;
+	    }
+	    glob=session->MakeGlob(pat);
+	    if(!glob)
+	       glob=new NoGlob(pat);
+	    m=MOVED;
+	 }
+	 break;
+
+      case(BUILTIN_NONE):
+      case(BUILTIN_EXEC_RESTART):
+	 abort(); // can't happen
       }
       if(interactive)
       {
 	 if(SignalHook::GetCount(SIGINT))
 	 {
-	    status_line->WriteLine(_("Interrupt"));
+	    if(status_line)
+	       status_line->WriteLine(_("Interrupt"));
 	    return AcceptSig(SIGINT);
 	 }
+	 if(SignalHook::GetCount(SIGTSTP))
+	 {
+	    if(builtin==BUILTIN_CD || builtin==BUILTIN_OPEN)
+	    {
+	       if(status_line)
+		  status_line->Clear();
+	       if(builtin==BUILTIN_CD)
+	       {
+		  // accept the path
+		  session->Chdir(session->GetFile(),false);
+	       }
+	       session->Close();
+	       exit_code=0;
+	       waiting=0;
+	       builtin=BUILTIN_NONE;
+	       return MOVED;
+	    }
+	    else
+	    {
+	       SignalHook::ResetCount(SIGTSTP);
+	    }
+	 }
 	 if(SignalHook::GetCount(SIGHUP))
 	 {
 	    interactive=0;
 	    return MOVED;
 	 }
-	 waiting->ShowRunStatus(status_line);
       }
+      if(status_line)
+	 waiting->ShowRunStatus(status_line);
       return STALL;
    }
 
@@ -343,11 +490,10 @@ int CmdExec::Do()
       if(waiting->Done())
       {
 	 waiting->Bg();
- 	 if(interactive)
-	 {
- 	    status_line->Show("");
+	 if(status_line)
+	    status_line->Clear();
+ 	 if(interactive || verbose)
 	    waiting->SayFinal(); // final phrase like 'rm succeed'
-	 }
 	 exit_code=waiting->ExitCode();
 	 delete waiting;
 	 waiting=0;
@@ -360,15 +506,18 @@ int CmdExec::Do()
 	 {
 	    waiting->Bg();
 	    SignalHook::ResetCount(SIGINT);
-	    status_line->WriteLine(_("Interrupt"));
+	    if(status_line)
+	       status_line->WriteLine(_("Interrupt"));
 	    return AcceptSig(SIGINT);
 	 }
 	 if(SignalHook::GetCount(SIGTSTP))
 	 {
 	    waiting->Bg();
-	    status_line->WriteLine("[%d] %s &",waiting->jobno,waiting->cmdline);
+	    if(status_line)
+	       status_line->WriteLine("[%d] %s &",waiting->jobno,waiting->cmdline);
 	    waiting->PrintStatus(1);
 	    exit_code=0;
+	    last_bg=waiting->jobno;
 	    waiting=0;
 	    return MOVED;
 	 }
@@ -377,13 +526,18 @@ int CmdExec::Do()
 	    interactive=0;
 	    return MOVED;
 	 }
-	 waiting->ShowRunStatus(status_line);
       }
+      if(status_line)
+	 waiting->ShowRunStatus(status_line);
       return STALL;
    }
 
    if(!interactive)
+   {
       BuryDoneJobs();
+      if(FindJob(last_bg)==0)
+	 last_bg=-1;
+   }
 
 try_get_cmd:
    if(next_cmd==0 || *next_cmd==0 || partial_cmd)
@@ -394,13 +548,25 @@ try_get_cmd:
 	 {
 	    ListDoneJobs();
 	    BuryDoneJobs();
+	    if(FindJob(last_bg)==0)
+	       last_bg=-1;
 	 }
 	 char *prompt=MakePrompt();
-	 char *cmd=feeder->NextCmd(this,prompt);
+	 feeder_called=true;
+	 const char *cmd=feeder->NextCmd(this,prompt);
+	 feeder_called=false;
 	 if(cmd==0)
 	 {
 	    if(next_cmd && *next_cmd && partial_cmd)
+	    {
+	       if(next_cmd[strlen(next_cmd)-1]!='\n')
+	       {
+		  // missing EOL on last line, add it
+		  FeedCmd("\n");
+		  goto try_get_cmd;
+	       }
 	       fprintf(stderr,_("Warning: discarding incomplete command\n"));
+	    }
 	    RemoveFeeder();
 	    m=MOVED;
 	    goto try_get_cmd;
@@ -460,15 +626,19 @@ void CmdExec::ShowRunStatus(StatusLine *s)
       switch(builtin)
       {
       case(BUILTIN_CD):
-	 s->Show("cd %s [%s]",args->getarg(1),session->CurrentStatus());
+	 s->Show("cd `%s' [%s]",args->getarg(1),session->CurrentStatus());
 	 break;
       case(BUILTIN_OPEN):
-	 s->Show("[%s]",session->CurrentStatus());
+	 s->Show("open `%s' [%s]",session->GetHostName(),session->CurrentStatus());
       	 break;
+      case(BUILTIN_GLOB):
+	 s->Show("%s",glob->Status());
+	 break;
+      case(BUILTIN_NONE):
+      case(BUILTIN_EXEC_RESTART):
+	 abort(); // can't happen
       }
    }
-   else if(Done())
-      s->Show("");
 }
 
 void CmdExec::PrintStatus(int v)
@@ -483,6 +653,27 @@ void CmdExec::PrintStatus(int v)
       free(s);
       return;
    }
+   if(is_queue)
+   {
+      if(!(next_cmd && next_cmd[0]))
+	 return;
+      printf(_("\tCommands queued:\n"));
+      char *cmd=next_cmd;
+      int n=1;
+      for(;;)
+      {
+	 char *cmd_end=strchr(cmd,'\n');
+	 if(!cmd_end)
+	    cmd_end=cmd+strlen(cmd);
+	 printf("\t%2d. %.*s\n",n++,int(cmd_end-cmd),cmd);
+	 if(*cmd_end==0)
+	    break;
+	 cmd=cmd_end+1;
+	 if(*cmd==0)
+	    break;
+      }
+      return;
+   }
    if(waiting)
    {
       printf(_("\tWaiting for job [%d] to terminate\n"),waiting->jobno);
@@ -494,6 +685,10 @@ void CmdExec::PrintStatus(int v)
 
 CmdExec::CmdExec(FileAccess *f) : SessionJob(f)
 {
+   // add this to chain
+   next=chain;
+   chain=this;
+
    cmd=0;
    args=0;
    waiting=0;
@@ -503,6 +698,7 @@ CmdExec::CmdExec(FileAccess *f) : SessionJob(f)
    interactive=false;
    status_line=0;
    feeder=0;
+   feeder_called=false;
    used_aliases=0;
 
    next_cmd=cmd_buf=0;
@@ -512,6 +708,7 @@ CmdExec::CmdExec(FileAccess *f) : SessionJob(f)
    condition=COND_ANY;
    prev_exit_code=0;
    exit_code=0;
+   last_bg=-1;
 
    cwd=0;
    SaveCWD();
@@ -521,6 +718,7 @@ CmdExec::CmdExec(FileAccess *f) : SessionJob(f)
    remote_completion=false;
    long_running=0;
    csh_history=false;
+   verify_host=verify_path=true;
 
    Reconfig();
 
@@ -528,12 +726,26 @@ CmdExec::CmdExec(FileAccess *f) : SessionJob(f)
    old_cwd=0;
    old_lcwd=0;
 
-   debug_file=0;
+   glob=0;
+   args_glob=0;
+
+   is_queue=false;
+   queue_cwd=0;
+   queue_lcwd=0;
 }
 
 CmdExec::~CmdExec()
 {
-   CloseDebug();
+   // remove this from chain.
+   for(CmdExec **scan=&chain; *scan; scan=&(*scan)->next)
+   {
+      if(this==*scan)
+      {
+	 *scan=(*scan)->next;
+	 break;
+      }
+   }
+
    free_used_aliases();
    xfree(cmd);
    if(args)
@@ -546,6 +758,13 @@ CmdExec::~CmdExec()
       cwd_owner=0;
    xfree(old_cwd);
    xfree(old_lcwd);
+   if(glob)
+      delete glob;
+   if(args_glob)
+      delete args_glob;
+
+   xfree(queue_cwd);
+   xfree(queue_lcwd);
 }
 
 char *CmdExec::MakePrompt()
@@ -611,6 +830,9 @@ char *CmdExec::MakePrompt()
 	 case 'u':
 	    to_add=session->GetUser();
 	    break;
+	 case '@': // @ if non-default user
+	    to_add=session->GetUser()?"@":"";
+	    break;
 	 case 'U':
 	    to_add=session->GetConnectURL();
 	    break;
@@ -672,8 +894,12 @@ char *CmdExec::MakePrompt()
       if(to_add==0)
 	 continue;
 
-      while(prompt_size<=store-prompt+(int)strlen(to_add))
-	 prompt=(char*)xrealloc(prompt,prompt_size*=2);
+      int store_index=store-prompt;
+      int need=store_index+(int)strlen(to_add)+1;
+      while(prompt_size<need)
+	 prompt_size*=2;
+      prompt=(char*)xrealloc(prompt,prompt_size);
+      store=prompt+store_index;
 
       strcpy(store,to_add);
       store+=strlen(to_add);
@@ -689,37 +915,36 @@ void CmdExec::beep_if_long()
       write(1,"\007",1);
 }
 
-const char *CmdExec::ResQuery(ResDecl& r)
+void CmdExec::Reconfig(const char *)
 {
-   const char *v;
+   const char *c=0;
+   if(session)
+      c = session->GetConnectURL(FA::NO_PATH);
 
-   v=ResMgr::Query(r.name,0);
-   if(!v) v=r.defvalue;
-
-   return v;
-}
-
-void CmdExec::Reconfig()
-{
-   long_running=atoi(ResQuery(res_long_running));
-   remote_completion=ResMgr::str2bool(ResQuery(res_remote_completion));
-   csh_history=ResMgr::str2bool(ResQuery(res_csh_history));
+   long_running = res_long_running.Query(c);
+   remote_completion = res_remote_completion.Query(c);
+   csh_history = res_csh_history.Query(0);
    xfree(var_ls);
-   var_ls=xstrdup(ResQuery(res_default_ls));
+   var_ls=xstrdup(res_default_ls.Query(c));
    xfree(var_prompt);
-   var_prompt=xstrdup(ResQuery(res_prompt));
+   var_prompt=xstrdup(res_prompt.Query(c));
+   verify_path=res_verify_path.Query(c);
+   verify_host=res_verify_host.Query(c);
+   verbose=res_verbose.Query(0);
 }
 
-void CmdExec::vfprintf(FILE *file,const char *f,va_list v)
+void CmdExec::pre_stdout()
 {
-   if(parent)
-      parent->vfprintf(file,f,v);
-   else
-   {
-      if(interactive && status_line)
-	 status_line->Show("");
-      ::vfprintf(file,f,v);
-   }
+   if(status_line)
+      status_line->Clear();
+   if(feeder_called)
+      feeder->clear();
+}
+
+void CmdExec::top_vfprintf(FILE *file,const char *f,va_list v)
+{
+   pre_stdout();
+   ::vfprintf(file,f,v);
 }
 
 void CmdExec::SetCmdFeeder(CmdFeeder *new_feeder)
@@ -737,8 +962,24 @@ int CmdExec::AcceptSig(int sig)
       return STALL;
    if(waiting==this) // builtin
    {
-      session->Close();
+      switch(builtin)
+      {
+      case(BUILTIN_CD):
+      case(BUILTIN_OPEN):
+	 session->Close();
+	 break;
+      case(BUILTIN_GLOB):
+	 delete glob;
+	 glob=0;
+	 delete args_glob;
+	 args_glob=0;
+	 break;
+      case(BUILTIN_NONE):
+      case(BUILTIN_EXEC_RESTART):
+	 abort(); // should not happen
+      }
       waiting=0;
+      builtin=BUILTIN_NONE;
       exit_code=1;
       return MOVED;
    }
@@ -748,12 +989,18 @@ int CmdExec::AcceptSig(int sig)
       if(res==WANTDIE)
       {
 	 exit_code=1;
+	 Job *new_waiting=waiting->waiting;
+	 waiting->waiting=0;
 	 delete waiting;
-	 waiting=0;
+	 waiting=new_waiting;
       }
+      if(waiting==0 && parent!=0)
+	 return WANTDIE;
       return MOVED;
    }
-   return WANTDIE;
+   if(parent!=0)
+      return WANTDIE;
+   return STALL;
 }
 
 void CmdExec::SetInteractive(bool i)
@@ -773,25 +1020,193 @@ void CmdExec::SetInteractive(bool i)
    interactive=i;
 }
 
-void CmdExec::CloseDebug()
+void CmdExec::unquote(char *buf,const char *str)
 {
-   if(debug_file)
+   while(*str)
    {
-      fclose(debug_file);
-      debug_file=0;
+      if(*str=='"' || *str=='\\')
+	 *buf++='\\';
+      *buf++=*str++;
    }
-
+   *buf=0;
 }
 
-int CmdExec::OpenDebug(const char *file)
+bool CmdExec::needs_quotation(const char *buf)
 {
-   CloseDebug();
-   debug_file=fopen(optarg,"a");
-   if(!debug_file)
+   while(*buf)
    {
-      perror(optarg);
-      return -1;
+      if(isspace(*buf))
+	 return true;
+      if(*buf=='"' || *buf=='\\')
+	 return true;
+      buf++;
+   }
+   return false;
+}
+
+void CmdExec::FeedQuoted(const char *c)
+{
+   char *buf=(char*)alloca(strlen(c)*2+2+1);
+   buf[0]='"';
+   unquote(buf+1,c);
+   strcat(buf,"\"");
+   FeedCmd(buf);
+}
+
+// implementation is here because it depends on CmdExec.
+char *ArgV::CombineQuoted(int start) const
+{
+   int	 i;
+   char  *res;
+   char	 *store,*arg;
+   int	 len=0;
+
+   for(i=start; i<c; i++)
+      len+=strlen(v[i])*2+3;
+
+   if(len==0)
+      return(xstrdup(""));
+
+   res=(char*)xmalloc(len);
+
+   store=res;
+   for(i=start; i<c; i++)
+   {
+      arg=v[i];
+      if(CmdExec::needs_quotation(arg))
+      {
+	 *store++='"';
+	 CmdExec::unquote(store,arg);
+	 store+=strlen(store);
+	 *store++='"';
+      }
+      else
+      {
+	 strcpy(store,arg);
+	 store+=strlen(store);
+      }
+      *store++=' ';
+   }
+   store[-1]=0;
+
+   return(res);
+}
+
+const char *CmdExec::GetFullCommandName(const char *cmd)
+{
+   const CmdExec::cmd_rec *c;
+   int part=CmdExec::find_cmd(cmd,&c);
+   if(part==1)
+      return c->name;
+   return cmd;
+}
+
+void CmdExec::AtExit()
+{
+   FeedCmd(res_at_exit.Query(0));
+   FeedCmd("\n");
+}
+
+void CmdExec::free_used_aliases()
+{
+   if(used_aliases)
+   {
+      TouchedAlias::FreeChain(used_aliases);
+      used_aliases=0;
+   }
+   alias_field=0;
+}
+
+
+CmdExec::cmd_rec *CmdExec::dyn_cmd_table=0;
+int CmdExec::dyn_cmd_table_count=0;
+void CmdExec::RegisterCommand(const char *name,cmd_creator_t creator,const char *short_desc,const char *long_desc)
+{
+   if(dyn_cmd_table==0)
+   {
+      dyn_cmd_table_count=1;
+      for(const cmd_rec *c=static_cmd_table; c->name; c++)
+	 dyn_cmd_table_count++;
+      dyn_cmd_table=(cmd_rec*)xmemdup(static_cmd_table,dyn_cmd_table_count*sizeof(cmd_rec));
+   }
+   else
+   {
+      dyn_cmd_table_count++;
+      dyn_cmd_table=(cmd_rec*)xrealloc(dyn_cmd_table,dyn_cmd_table_count*sizeof(cmd_rec));
+   }
+   for(cmd_rec *c=dyn_cmd_table; c->name; c++)
+   {
+      if(!strcmp(c->name,name))
+      {
+	 c->creator=creator;
+	 if(short_desc)
+	    c->short_desc=short_desc;
+	 if(long_desc)
+	    c->long_desc=long_desc;
+	 dyn_cmd_table_count--;
+	 return;
+      }
+   }
+   dyn_cmd_table[dyn_cmd_table_count-2].name=name;
+   dyn_cmd_table[dyn_cmd_table_count-2].creator=creator;
+   dyn_cmd_table[dyn_cmd_table_count-2].short_desc=short_desc;
+   dyn_cmd_table[dyn_cmd_table_count-2].long_desc=long_desc;
+   memset(&dyn_cmd_table[dyn_cmd_table_count-1],0,sizeof(cmd_rec));
+}
+
+void CmdExec::ChangeSession(FileAccess *new_session)
+{
+   Reuse(session);
+   session=new_session;
+   Reconfig(0);
+}
+
+const CmdExec::cmd_rec *CmdExec::CmdByIndex(int i)
+{
+   return (dyn_cmd_table?dyn_cmd_table:static_cmd_table)+i;
+}
+
+Job *CmdExec::default_cmd()
+{
+   const char *op=args->a0();
+#ifdef WITH_MODULES
+   char *modname=(char*)alloca(4+strlen(op)+1);
+   sprintf(modname,"cmd-%s",op);
+   if(module_load(modname,0,0)==0)
+   {
+      eprintf("%s\n",module_error_message());
+      return 0;
+   }
+   builtin=BUILTIN_EXEC_RESTART;
+   return this;
+#else
+   eprintf("%s: command `%s' is not compiled in.\n",op,op);
+   return 0;
+#endif
+}
+
+void CmdExec::FeedArgV(const ArgV *args,int start)
+{
+   char *cmd;
+
+   if(start+1==args->count())
+      cmd=args->Combine(start);
+   else
+      cmd=args->CombineQuoted(start);
+
+   FeedCmd(cmd);
+   FeedCmd("\n");
+   xfree(cmd);
+}
+
+CmdExec *CmdExec::FindQueue()
+{
+   for(CmdExec *scan=chain; scan; scan=scan->next)
+   {
+      if(scan->is_queue
+      && !strcmp(this->session->GetConnectURL(FA::NO_PATH),
+	         scan->session->GetConnectURL(FA::NO_PATH)))
+	 return scan;
    }
-   fcntl(fileno(debug_file),F_SETFD,FD_CLOEXEC);
    return 0;
 }
