@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-1997 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-1999 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,6 +18,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: CmdExec.h,v 1.29 1999/08/31 13:58:02 lav Exp $ */
+
 #ifndef CMDEXEC_H
 #define CMDEXEC_H
 
@@ -28,43 +30,46 @@
 #include "Filter.h"
 #include "alias.h"
 #include "history.h"
+#include "bookmark.h"
 
 extern History cwd_history;
+extern Bookmark lftp_bookmarks;
 
-#define	CMD(name) Job *CmdExec::do_##name()
-#define	in_CMD(name) Job *do_##name()
+#define	CMD(name) Job *cmd_##name(CmdExec *parent)
+
+typedef Job * (*cmd_creator_t)(class CmdExec *parent);
 
 class CmdFeeder
 {
 public:
    char *saved_buf;
    CmdFeeder *prev;
-   virtual char *NextCmd(class CmdExec *exec,const char *prompt) = 0;
+   virtual const char *NextCmd(class CmdExec *exec,const char *prompt) = 0;
+   virtual ~CmdFeeder() {}
+
+   virtual void clear() {}
 };
 
+extern CmdFeeder *lftp_feeder;	 // feeder to use after 'lftp' command
+
 class CmdExec : public SessionJob
 {
+public:
 // current command data
    char *cmd;
    ArgV *args;
    FDStream *output;
    bool background;
+   int	 exit_code;
 
+private:
    char *next_cmd;
    char *cmd_buf;
    bool partial_cmd;
    int alias_field; // length of expanded alias (and ttl for used_aliases)
 
    TouchedAlias *used_aliases;
-   void free_used_aliases()
-      {
-	 if(used_aliases)
-	 {
-	    TouchedAlias::FreeChain(used_aliases);
-	    used_aliases=0;
-	 }
-	 alias_field=0;
-      }
+   void free_used_aliases();
 
    enum
    {
@@ -74,19 +79,17 @@ class CmdExec : public SessionJob
    }
       condition;
 
-   int	 prev_exit_code;
-   int	 exit_code;
-
    struct cmd_rec
    {
       const char  *name;
-      Job   *(CmdExec::*func)();
+      cmd_creator_t creator;
       const char  *short_desc;
       const char  *long_desc;
    };
-   static const cmd_rec cmd_table[];
+   static const cmd_rec static_cmd_table[];
+   static cmd_rec *dyn_cmd_table;
+   static int dyn_cmd_table_count;
 
-   static void print_cmd_help(const char *cmd);
    static int find_cmd(const char *cmd_name,const cmd_rec **ret);
 
    void exec_parsed_command();
@@ -99,43 +102,44 @@ class CmdExec : public SessionJob
    };
    parse_result parse_one_cmd();
 
+   CmdFeeder *feeder;
+   bool feeder_called;
+
+   int	 prev_exit_code;
+
    enum builtins
    {
+      BUILTIN_NONE,
       BUILTIN_OPEN,
-      BUILTIN_CD
+      BUILTIN_CD,
+      BUILTIN_EXEC_RESTART,
+      BUILTIN_GLOB
    }
       builtin;
 
-   static CmdExec *debug_shell;
-   static void debug_callback(char *);
-
-   const char *ResQuery(ResDecl&);
-
-   CmdFeeder *feeder;
-
    char *old_cwd;
    char *old_lcwd;
 
-   FILE *debug_file;
-   void CloseDebug();
-   int  OpenDebug(const char *file);
+   Glob *glob;
+   ArgV *args_glob;
+
+   static CmdExec *chain;
+   CmdExec *next;
+
+   bool is_queue;
+   char *queue_cwd;
+   char *queue_lcwd;
+   CmdExec *FindQueue();
 
 public:
    void FeedCmd(const char *c);
+   void FeedArgV(const ArgV *,int start=0);
    void PrependCmd(const char *c);
    void ExecParsed(ArgV *a,FDStream *o=0,bool b=false);
-
-   in_CMD(alias); in_CMD(anon);  in_CMD(cd);    in_CMD(debug);
-   in_CMD(exit);  in_CMD(get);   in_CMD(help);  in_CMD(jobs);
-   in_CMD(kill);  in_CMD(lcd);   in_CMD(ls);    in_CMD(mget);
-   in_CMD(open);  in_CMD(pwd);   in_CMD(put);   in_CMD(set);
-   in_CMD(shell); in_CMD(source);in_CMD(user);  in_CMD(rm);
-   in_CMD(wait);  in_CMD(site);  in_CMD(subsh); in_CMD(mirror);
-   in_CMD(mput);  in_CMD(mv);	 in_CMD(cat);   in_CMD(cache);
-   in_CMD(mkdir); in_CMD(quote); in_CMD(scache);in_CMD(mrm);
-   in_CMD(ver);	  in_CMD(close); in_CMD(bookmark);
-
-   static const char * const var_list[];
+   static void unquote(char *buf,const char *str);
+   static bool needs_quotation(const char *buf);
+   void FeedQuoted(const char *c);
+   void AtExit();
 
    CmdExec(FileAccess *s);
    ~CmdExec();
@@ -151,20 +155,23 @@ public:
    char *MakePrompt();
 
    bool interactive;
+   bool verbose;
    StatusLine *status_line;
    void SetCmdFeeder(CmdFeeder *new_feeder);
    void	RemoveFeeder();
 
    friend char *command_generator(char *,int);	  // readline completor
-   friend int remote_cmd(int);
+   static const char *GetFullCommandName(const char *);
 
    char	 *var_ls;
    char	 *var_prompt;
    bool	 remote_completion;
    int	 long_running;
    bool	 csh_history;
+   bool	 verify_host;
+   bool	 verify_path;
 
-   void	 Reconfig();
+   void	 Reconfig(const char *name=0);
 
    void	 beep_if_long();
    time_t start_time;
@@ -172,13 +179,42 @@ public:
    static CmdExec *cwd_owner;
    char	 *cwd;
    void	 SaveCWD();
-   void	 RestoreCWD();
+   int	 RestoreCWD();
+   void  SetCWD(const char *c);
 
    FDStream *default_output;
 
-   void vfprintf(FILE *file,const char *f,va_list v);
+   void top_vfprintf(FILE *file,const char *f,va_list v);
 
    void SetInteractive(bool i);
+
+   static void RegisterCommand(const char *name,cmd_creator_t creator,
+      const char *short_name=0,const char *long_name=0);
+
+   Job *builtin_lcd();
+   Job *builtin_cd();
+   Job *builtin_open();
+   Job *builtin_exit();
+   Job *builtin_lftp();
+   Job *builtin_restart();
+   Job *builtin_glob();
+   Job *builtin_queue();
+
+   Job *default_cmd();
+
+   void ChangeSession(FileAccess *new_session);
+
+   void print_cmd_help(const char *cmd);
+   void print_cmd_index();
+
+   static const cmd_rec *CmdByIndex(int i);
+
+   int	 last_bg;
+
+   void pre_stdout();
 };
 
+extern const char * const bookmark_subcmd[];
+extern const char * const cache_subcmd[];
+
 #endif//CMDEXEC_H
