@@ -18,25 +18,35 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: FileAccess.cc,v 1.30.2.1 2000/02/20 14:51:56 lav Exp $ */
+
 #include <config.h>
 
 #include "FileAccess.h"
 #include "xmalloc.h"
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <netinet/in.h>
+#include <netinet/tcp.h>
 #include <errno.h>
 #include <ctype.h>
+#include <fcntl.h>
+#include <fnmatch.h>
+#include "LsCache.h"
 #include "xalloca.h"
+#include "log.h"
+#include "url.h"
+#include "misc.h"
+#ifdef WITH_MODULES
+# include "module.h"
+#endif
 
-
-FILE  *FileAccess::debug_file=0;
-void  (*FileAccess::debug_callback)(char *msg)=0;
-int   FileAccess::debug_level=0;
+FileAccess *FileAccess::chain=0;
 
 void FileAccess::Init()
 {
    home=0;
-   port=0;
+   portname=0;
    hostname=0;
    group=gpass=0;
    user=pass=0;
@@ -44,7 +54,8 @@ void FileAccess::Init()
    file=0;
    file1=0;
    real_cwd=0;
-   cwd=xstrdup("~");
+   default_cwd="~";
+   cwd=xstrdup(default_cwd);
    real_pos=-1;
    pos=0;
    mode=CLOSED;
@@ -52,14 +63,23 @@ void FileAccess::Init()
    time(&event_time);
    opt_date=0;
    opt_size=0;
-   last_error_resp=0;
+   error=0;
+   error_code=OK;
    saved_errno=0;
    mkdir_p=false;
-
-   sleep_time=30; // retry with 30 second interval
-   timeout=600;	  // 10 minutes with no events = reconnect
+   ascii=false;
 
    url=0;
+
+   entity_size=-1;
+   entity_date=(time_t)-1;
+
+   closure=0;
+
+   chmod_mode=0644;
+
+   next=chain;
+   chain=this;
 }
 
 FileAccess::FileAccess(const FileAccess *fa)
@@ -80,7 +100,8 @@ FileAccess::FileAccess(const FileAccess *fa)
    gpass=xstrdup(fa->gpass);
    xfree(hostname);
    hostname=xstrdup(fa->hostname);
-   port=fa->port;
+   xfree(portname);
+   portname=xstrdup(fa->portname);
 }
 
 FileAccess::~FileAccess()
@@ -88,59 +109,64 @@ FileAccess::~FileAccess()
    xfree(file); file=0;
    xfree(cwd); cwd=0;
    xfree(real_cwd); real_cwd=0;
-   xfree(last_error_resp); last_error_resp=0;
+   xfree(error); error=0;
    xfree(home); home=0;
    xfree(user); user=0;
    xfree(pass); pass=0;
    xfree(group); group=0;
    xfree(gpass); gpass=0;
    xfree(hostname); hostname=0;
+   xfree(portname); portname=0;
    xfree(url); url=0;
+   xfree(closure); closure=0;
+   for(FileAccess **scan=&chain; *scan; scan=&((*scan)->next))
+   {
+      if(*scan==this)
+      {
+	 *scan=next;
+	 break;
+      }
+   }
 }
 
 void  FileAccess::DebugPrint(const char *prefix,const char *str,int level)
 {
-   if(level>debug_level)
-      return;
-   if(!debug_file && !debug_callback)
+   if(!Log::global)
       return;
 
-   char *msg=(char*)alloca(strlen(prefix)+strlen(str)+6);
-   while(*str)
+   char *msg=(char*)alloca(xstrlen(hostname)+strlen(prefix)+strlen(str)+25);
+
+   do
    {
-      strcpy(msg,prefix);
-      if(!strncmp(str,"PASS ",5))
-      {
-	 // don't log PASS message in original form
-	 strcat(msg,"PASS XXXX");
-	 while(*str && *str!='\n')
-	    str++;
-      }
+      if(Log::global->IsTTY())
+	 strcpy(msg,prefix);
       else
       {
-	 int msglen=strlen(msg);
-	 while(*str && *str!='\n')
+	 sprintf(msg,"[%d] ",(int)getpid());
+	 if(hostname)
 	 {
-	    if(!(*str=='\r' && (str[1]=='\n' || str[1]=='\0')))
-	       msg[msglen++]=*str;
-	    str++;
+	    strcat(msg,hostname);
+	    strcat(msg," ");
 	 }
-	 msg[msglen]=0;
+	 strcat(msg,prefix);
       }
 
-      if(debug_file)
+      int msglen=strlen(msg);
+      while(*str && *str!='\n')
       {
-	 fputs(msg,debug_file);
-	 fputc('\n',debug_file);
+	 if(!(*str=='\r' && (str[1]=='\n' || str[1]=='\0')))
+	    msg[msglen++]=*str;
+	 str++;
       }
-      if(debug_callback)
-	 (*debug_callback)(msg);
+      msg[msglen++]='\n';
+      msg[msglen]=0;
+
+      Log::global->Write(level,msg);
 
       if(!*str++)
          break;
    }
-   if(debug_file)
-      fflush(debug_file);
+   while(*str);
 }
 
 int   FileAccess::Poll(int fd,int ev)
@@ -153,7 +179,7 @@ int   FileAccess::Poll(int fd,int ev)
    if(res<1)
       return 0;
    if(CheckHangup(&pfd,1))
-      return 0;
+      return -1;
    if(pfd.revents)
       time(&event_time);
    return pfd.revents;
@@ -161,43 +187,53 @@ int   FileAccess::Poll(int fd,int ev)
 
 int   FileAccess::CheckHangup(struct pollfd *pfd,int num)
 {
-#ifdef SO_ERROR
    for(int i=0; i<num; i++)
    {
+#ifdef SO_ERROR
       char  str[256];
       int   s_errno=0;
-      ADDRLEN_TYPE len;
+      socklen_t len;
 
       errno=0;
 
-/* When the poll() is emulated by select(), there is no error condition
-   flag, so we can only check for socket errors */
-
 // Where does the error number go - to errno or to the pointer?
 // It seems that to errno, but if the pointer is NULL it dumps core.
 // (solaris 2.5)
 // It seems to be different on glibc 2.0 - check both errno and s_errno
 
-#ifdef HAVE_POLL
-      if(pfd[i].revents&POLLERR)
-#endif
+      len=sizeof(s_errno);
+      getsockopt(pfd[i].fd,SOL_SOCKET,SO_ERROR,(char*)&s_errno,&len);
+      if(errno==ENOTSOCK)
+	 return 0;
+      if(errno!=0 || s_errno!=0)
       {
-	 len=sizeof(s_errno);
-	 getsockopt(pfd[i].fd,SOL_SOCKET,SO_ERROR,(char*)&s_errno,&len);
-	 if(errno!=0 || s_errno!=0)
-	 {
-	    sprintf(str,_("Socket error (%s) - reconnecting"),
-				       strerror(errno?errno:s_errno));
-	    DebugPrint("**** ",str);
-	    Disconnect();
-	    return 1;
-	 }
+	 sprintf(str,_("Socket error (%s) - reconnecting"),
+				    strerror(errno?errno:s_errno));
+	 DebugPrint("**** ",str);
+	 return 1;
+      }
+#endif /* SO_ERROR */
+      if(pfd[i].revents&POLLERR)
+      {
+	 DebugPrint("**** ","POLLERR");
+	 return 1;
       }
    } /* end for */
-#endif /* SO_ERROR */
    return 0;
 }
 
+void FileAccess::NonBlock(int fd)
+{
+   int fl=0;
+   fcntl(fd,F_GETFL,&fl);
+   fcntl(fd,F_SETFL,fl|O_NONBLOCK);
+}
+void FileAccess::CloseOnExec(int fd)
+{
+   fcntl(fd,F_SETFD,FD_CLOEXEC);
+}
+
+
 void  FileAccess::Open(const char *fn,int mode,long offs)
 {
    Close();
@@ -208,7 +244,21 @@ void  FileAccess::Open(const char *fn,int mode,long offs)
    this->mode=mode;
    mkdir_p=false;
    try_time=0;
-   return;
+
+   switch((open_mode)mode)
+   {
+   case STORE:
+   case REMOVE:
+   case MAKE_DIR:
+      LsCache::FileChanged(this,file);
+      break;
+   case REMOVE_DIR:
+      LsCache::FileChanged(this,file);
+      LsCache::TreeChanged(this,file);
+      break;
+   default:
+      break;
+   }
 }
 
 const char *FileAccess::StrError(int err)
@@ -216,35 +266,45 @@ const char *FileAccess::StrError(int err)
    static char *str=0;
    static unsigned str_allocated=0;
 
+   // note to translators: several errors should not be displayed to user;
+   // so no need to translate them.
    switch(err)
    {
    case(IN_PROGRESS):
-      return(_("Operation is in progress"));
+      return("Operation is in progress");
    case(OK):
-      return(_("Error 0"));
+      return("Error 0");
    case(SEE_ERRNO):
+      if(error)
+      {
+	 const char *e=strerror(saved_errno);
+	 if(str_allocated<strlen(e)+64+strlen(error))
+	    str=(char*)xrealloc(str,str_allocated=strlen(e)+64+strlen(error));
+   	 sprintf(str,"%s: %s",error,e);
+	 return(str);
+      }
       return(strerror(saved_errno));
    case(LOOKUP_ERROR):
-      return(last_error_resp);
+      return(error);
    case(NOT_OPEN):   // Actually this means an error in application
-      return(_("Class is not Open()ed"));
+      return("Class is not Open()ed");
    case(NO_FILE):
-      if(last_error_resp)
+      if(error)
       {
-	 if(str_allocated<64+strlen(last_error_resp))
-	    str=(char*)xrealloc(str,64+strlen(last_error_resp));
-   	 sprintf(str,_("Access failed: %s"),last_error_resp);
+	 if(str_allocated<64+strlen(error))
+	    str=(char*)xrealloc(str,64+strlen(error));
+   	 sprintf(str,_("Access failed: %s"),error);
 	 return(str);
       }
       return(_("File cannot be accessed"));
    case(NO_HOST):
       return(_("Not connected"));
    case(FATAL):
-      if(last_error_resp)
+      if(error)
       {
-	 if(str_allocated<64+strlen(last_error_resp))
-	    str=(char*)xrealloc(str,64+strlen(last_error_resp));
-   	 sprintf(str,_("Fatal error: %s"),last_error_resp);
+	 if(str_allocated<64+strlen(error))
+	    str=(char*)xrealloc(str,str_allocated=64+strlen(error));
+   	 sprintf(str,_("Fatal error: %s"),error);
 	 return(str);
       }
       return(_("Fatal error"));
@@ -266,14 +326,20 @@ void FileAccess::Close()
    opt_date=0;
    opt_size=0;
    array_for_info=0;
+   entity_size=-1;
+   entity_date=(time_t)-1;
+   ascii=false;
+   ClearError();
 }
 
 void FileAccess::Rename(const char *f,const char *f1)
 {
-   Close();
-   file=xstrdup(f);
+   Open(f,RENAME);
    file1=xstrdup(f1);
-   mode=RENAME;
+
+   LsCache::TreeChanged(this,file);
+   LsCache::FileChanged(this,file);
+   LsCache::FileChanged(this,file1);
 }
 
 void FileAccess::Mkdir(const char *fn,bool allp)
@@ -288,31 +354,75 @@ bool FileAccess::SameLocationAs(FileAccess *fa)
       return false;
    return true;
 }
+bool FileAccess::SameSiteAs(FileAccess *fa)
+{
+   return SameLocationAs(fa);
+}
+
+const char *FileAccess::GetFileURL(const char *f,int flags)
+{
+   int len=1;
+   const char *proto=GetProto();
+   if(proto[0]==0)
+      return "";
+   len+=strlen(proto)+strlen("://");
+   if(user)
+   {
+      len+=strlen(user)*3+1;
+      if(pass)
+	 len+=strlen(pass)*3+1;
+   }
+   if(hostname)
+      len+=strlen(hostname)*3;
+   if(portname)
+      len+=1+strlen(portname)*3;
+   f=dir_file(cwd?cwd:"~",f);
+   len+=1+strlen(f)*3;
+   url=(char*)xrealloc(url,len);
+   sprintf(url,"%s://",proto);
+   if(user)
+   {
+      url::encode_string(user,url+strlen(url),"/:@"URL_UNSAFE);
+      if(pass && (flags&WITH_PASSWORD))
+      {
+	 strcat(url,":");
+	 url::encode_string(pass,url+strlen(url),"/:@"URL_UNSAFE);
+      }
+      strcat(url,"@");
+   }
+   if(hostname)
+      url::encode_string(hostname,url+strlen(url),"/:"URL_UNSAFE);
+   if(portname)
+   {
+      strcat(url,":");
+      url::encode_string(portname,url+strlen(url),"/"URL_UNSAFE);
+   }
+   if(strcmp(f,"~") && !(flags&NO_PATH))
+   {
+      if(f[0]!='/') // e.g. ~/path
+	 strcat(url,"/");
+      url::encode_string(f,url+strlen(url));
+   }
+   return url;
+}
 
 const char *FileAccess::GetConnectURL(int flags)
 {
-   int len=1;
-   len+=strlen(GetProto())+strlen("://");
-   if(user)
-      len+=strlen(user)+1;
-   if(hostname)
-      len+=strlen(hostname);
-   if(port!=0)
-      len+=1+20;
-   if(cwd)
-      len+=1+strlen(cwd);
-   url=(char*)xrealloc(url,len);
-   sprintf(url,"%s://",GetProto());
-   if(user)
-      sprintf(url+strlen(url),"%s@",user);
-   if(hostname)
-      sprintf(url+strlen(url),"%s",hostname);
-   if(port!=0)
-      sprintf(url+strlen(url),":%d",port);
-   if(cwd && strcmp(cwd,"~") && !(flags&NO_CWD))
-      sprintf(url+strlen(url),"/%s",cwd+(cwd[0]=='/'));
+   return GetFileURL(0,flags);
+}
 
-   return url;
+void FileAccess::Connect(const char *host1,const char *port1)
+{
+   Close();
+   xfree(hostname);
+   hostname=xstrdup(host1);
+   xfree(portname);
+   portname=xstrdup(port1);
+   xfree(cwd);
+   cwd=xstrdup(default_cwd);
+   xfree(home); home=0;
+   try_time=0;
+   DontSleep();
 }
 
 void FileAccess::Login(const char *user1,const char *pass1)
@@ -323,7 +433,7 @@ void FileAccess::Login(const char *user1,const char *pass1)
    xfree(pass);
    pass=xstrdup(pass1);
    xfree(cwd);
-   cwd=xstrdup("~");
+   cwd=xstrdup(default_cwd);
    xfree(home);
    home=0;
 }
@@ -344,15 +454,17 @@ void FileAccess::AnonymousLogin()
    xfree(pass); pass=0;
    xfree(group); group=0;
    xfree(gpass); gpass=0;
+   xfree(cwd);
+   cwd=xstrdup(default_cwd);
+   xfree(home);  home=0;
 }
 
 void FileAccess::GetInfoArray(struct fileinfo *info,int count)
 {
-   Close();
+   Open(0,ARRAY_INFO);
    array_for_info=info;
    array_ptr=0;
    array_cnt=count;
-   mode=ARRAY_INFO;
 }
 
 static void expand_tilde(char **path, const char *home)
@@ -363,7 +475,10 @@ static void expand_tilde(char **path, const char *home)
       if(home_len==1 && home[0]=='/' && (*path)[1]=='/')
 	 home_len=0;
       int path_len=strlen(*path+1);
-      *path=(char*)xrealloc(*path,path_len+home_len+1);
+      // Note: don't shrink the array, because later memmove would break
+      // We could first memmove then shrink, but it won't give much
+      if(home_len>1)
+	 *path=(char*)xrealloc(*path,path_len+home_len+1);
       memmove(*path+home_len,*path+1,path_len+1);
       memmove(*path,home,home_len);
    }
@@ -389,7 +504,7 @@ int last_element_is_doubledot(char *path,char *end)
         || (end>path+2 && !strncmp(end-3,"/..",3)));
 }
 
-void FileAccess::Chdir(const char *path)
+void FileAccess::Chdir(const char *path,bool verify)
 {
    char	 *newcwd=(char*)alloca(strlen(cwd)+strlen(path)+2);
    int	 prefix_size=0;
@@ -497,7 +612,38 @@ void FileAccess::Chdir(const char *path)
       if((*out++=*in++)=='\0')
 	 break;
    }
-   Open(newcwd,CHANGE_DIR);
+   if(verify)
+      Open(newcwd,CHANGE_DIR);
+   else
+   {
+      xfree(cwd);
+      cwd=xstrdup(newcwd);
+   }
+}
+
+void FileAccess::Chmod(const char *file,int m)
+{
+   chmod_mode=m;
+   Open(file,CHANGE_MODE);
+}
+
+void FileAccess::SetError(int ec,const char *e)
+{
+   xfree(error);
+   error=xstrdup(e);
+   error_code=ec;
+}
+
+void FileAccess::ClearError()
+{
+   error_code=OK;
+   xfree(error);
+   error=0;
+}
+
+void FileAccess::Fatal(const char *e)
+{
+   SetError(FATAL,e);
 }
 
 
@@ -574,3 +720,262 @@ FileAccess *SessionPool::Walk(int *n,const char *proto)
    }
    return 0;
 }
+
+bool FileAccess::NotSerious(int e)
+{
+   switch(e)
+   {
+   case(ETIMEDOUT):
+#ifdef ECONNRESET
+   case(ECONNRESET):
+#endif
+   case(ECONNREFUSED):
+#ifdef EHOSTUNREACH
+   case(EHOSTUNREACH):
+#endif
+#ifdef EHOSTDOWN
+   case(EHOSTDOWN):
+#endif
+#ifdef ENETRESET
+   case(ENETRESET):
+#endif
+#ifdef ENETUNREACH
+   case(ENETUNREACH):
+#endif
+#ifdef ENETDOWN
+   case(ENETDOWN):
+#endif
+#ifdef ECONNABORTED
+   case(ECONNABORTED):
+#endif
+      return true;
+   }
+   return false;
+}
+
+void FileAccess::BumpEventTime(time_t t)
+{
+   if(event_time<t)
+      event_time=t;
+}
+
+ResValue FileAccess::Query(const char *name,const char *closure)
+{
+   const char *prefix=GetProto();
+   char *fullname=(char*)alloca(3+strlen(prefix)+1+strlen(name)+1);
+   sprintf(fullname,"%s:%s",prefix,name);
+   return ResMgr::Query(fullname,closure);
+}
+
+void FileAccess::Reconfig(const char *)
+{
+}
+
+void FileAccess::CleanupAll()
+{
+   for(FileAccess *o=chain; o!=0; o=o->next)
+      o->CleanupThis();
+}
+
+// FileAccess::Protocol implementation
+FileAccess::Protocol *FileAccess::Protocol::chain=0;
+
+FileAccess::Protocol::Protocol(const char *proto, SessionCreator *creator)
+{
+   this->proto=proto;
+   this->New=creator;
+   this->next=chain;
+   chain=this;
+}
+
+FileAccess::Protocol *FileAccess::Protocol::FindProto(const char *proto)
+{
+   for(Protocol *scan=chain; scan; scan=scan->next)
+      if(!strcmp(scan->proto,proto))
+	 return scan;
+   return 0;
+}
+
+FileAccess *FileAccess::Protocol::NewSession(const char *proto)
+{
+   Protocol *p;
+
+   p=FindProto(proto);
+   if(p)
+      return p->New();
+
+#ifdef WITH_MODULES
+#define PROTO_PREFIX "proto-"
+   char *mod=(char*)alloca(strlen(PROTO_PREFIX)+strlen(proto)+1);
+   sprintf(mod,"%s%s",PROTO_PREFIX,proto);
+   void *map=module_load(mod,0,0);
+   if(map==0)
+   {
+      fprintf(stderr,"%s\n",module_error_message());
+      return 0;
+   }
+   p=FindProto(proto);
+   if(p)
+      return p->New();
+#endif
+   return 0;
+}
+
+// FileAccessOperation implementation
+FileAccessOperation::FileAccessOperation()
+{
+   done=false;
+   error_text=0;
+   use_cache=true;
+}
+
+FileAccessOperation::~FileAccessOperation()
+{
+   xfree(error_text);
+}
+
+void FileAccessOperation::SetError(const char *e)
+{
+   xfree(error_text);
+   error_text=xstrdup(e);
+   done=true;
+}
+
+
+// ListInfo implementation
+ListInfo::ListInfo()
+{
+   result=0;
+
+   rxc_exclude=0;
+   rxc_include=0;
+   path=0;
+
+   need=0;
+
+   follow_symlinks=false;
+}
+
+ListInfo::~ListInfo()
+{
+   if(result)
+      delete result;
+}
+
+void ListInfo::SetExclude(const char *p,regex_t *x,regex_t *i)
+{
+   rxc_exclude=x;
+   rxc_include=i;
+   path=p;
+}
+
+// Glob implementation
+Glob::~Glob()
+{
+   free_list();
+   xfree(pattern);
+}
+
+Glob::Glob(const char *p)
+{
+   pattern=xstrdup(p);
+   list=0;
+   list_size=0;
+   list_alloc=0;
+}
+
+void Glob::free_list()
+{
+   for(int i=0; i<list_size; i++)
+      xfree(list[i]);
+   xfree(list);
+   list=0;
+   list_alloc=0;
+   list_size=0;
+}
+
+void Glob::add_force(const char *ptr,int len)
+{
+   // insert new file name into list
+   if(list_size>=list_alloc-1)
+   {
+      if(list_alloc==0)
+	 list_alloc=32;
+      list=(char**)xrealloc(list,(list_alloc*=2)*sizeof(*list));
+   }
+   list[list_size]=(char*)xmalloc(len+1);
+   memcpy(list[list_size],ptr,len);
+   list[list_size][len]=0;
+   list[++list_size]=0;
+}
+void Glob::add(const char *ptr,int len)
+{
+   char *s=(char*)alloca(len+1);
+   memcpy(s,ptr,len);
+   s[len]=0;
+
+   if(pattern[0]!=0
+   && fnmatch(pattern, s, FNM_PATHNAME|FNM_PERIOD)!=0)
+      return; // unmatched
+
+   add_force(ptr,len);
+}
+
+bool Glob::HasWildcards(const char *s)
+{
+   while(*s)
+   {
+      switch(*s)
+      {
+      case '\\':
+	 if(s[1])
+	    s++;
+	 break;
+      case '*':
+      case '[':
+      case '?':
+	 return true;
+      }
+      s++;
+   }
+   return false;
+}
+
+void Glob::UnquoteWildcards(char *s)
+{
+   char *store=s;
+   for(;;)
+   {
+      if(*s=='\\')
+      {
+	 if(s[1]=='*'
+	 || s[1]=='['
+	 || s[1]=='?'
+	 || s[1]=='\\')
+	    s++;
+      }
+      *store=*s;
+      if(*s==0)
+	 break;
+      s++;
+      store++;
+   }
+}
+
+int NoGlob::Do()
+{
+   if(!done)
+   {
+      done=true;
+      return MOVED;
+   }
+   return STALL;
+}
+NoGlob::NoGlob(const char *p) : Glob(p)
+{
+   char *p1=pattern;
+   pattern=xstrdup("");
+   UnquoteWildcards(p1);
+   add(p1);
+   xfree(p1);
+}
