@@ -23,7 +23,7 @@
 #include <unistd.h>
 #include "FileFeeder.h"
 
-char *FileFeeder::NextCmd(CmdExec *exec, const char *)
+const char *FileFeeder::NextCmd(CmdExec *exec, const char *)
 {
    int fd=in->getfd();
    if(fd<0)
