@@ -18,6 +18,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: FileSet.cc,v 1.7 1999/09/08 16:06:54 lav Exp $ */
+
 #include <config.h>
 #include "FileSet.h"
 
@@ -51,6 +53,15 @@ void FileSet::Add(FileInfo *fi)
       delete fi;
       return;
    }
+   for(int j=0; j<fnum; j++)
+   {
+      if(!strcmp(files[j]->name,fi->name))
+      {
+	 files[j]->Merge(*fi);
+	 delete fi;
+	 return;
+      }
+   }
    files=(FileInfo**)xrealloc(files,(++fnum)*sizeof(*files));
    files[fnum-1]=fi;
 }
@@ -126,42 +137,37 @@ FileSet::~FileSet()
    xfree(files);
 }
 
-void FileSet::SubtractSame(const FileSet *set,bool only_newer,time_t prec)
+void FileSet::SubtractSame(const FileSet *set,bool only_newer,time_t prec,int ignore)
 {
    for(int i=0; i<fnum; i++)
    {
-      for(int j=0; j<set->fnum; j++)
-      {
-	 if(files[i]->SameAs(set->files[j],only_newer,prec))
-	 {
-	    Sub(i);
-	    i--;
-	    break;
-	 }
-      }
+      FileInfo *f=set->FindByName(files[i]->name);
+      if(f && files[i]->SameAs(f,only_newer,prec,ignore))
+	 Sub(i--);
    }
 }
 
 void FileSet::SubtractAny(const FileSet *set)
 {
    for(int i=0; i<fnum; i++)
-   {
-      for(int j=0; j<set->fnum; j++)
-      {
-	 if(!strcmp(files[i]->name,set->files[j]->name))
-	 {
-	    Sub(i);
-	    i--;
-	    break;
-	 }
-      }
-   }
+      if(set->FindByName(files[i]->name))
+	 Sub(i--);
+}
+
+void FileSet::SubtractNotIn(const FileSet *set)
+{
+   for(int i=0; i<fnum; i++)
+      if(!set->FindByName(files[i]->name))
+	 Sub(i--);
 }
 
 void FileSet::SubtractOlderThan(time_t t)
 {
    for(int i=0; i<fnum; i++)
    {
+      if(files[i]->defined&FileInfo::TYPE
+      && files[i]->filetype!=FileInfo::NORMAL)
+	 continue;
       if(files[i]->OlderThan(t))
       {
 	 Sub(i);
@@ -182,7 +188,7 @@ void FileSet::ExcludeDots()
    }
 }
 
-bool  FileInfo::SameAs(const FileInfo *fi,bool only_newer,time_t prec)
+bool  FileInfo::SameAs(const FileInfo *fi,bool only_newer,time_t prec,int ignore)
 {
    if(defined&NAME && fi->defined&NAME)
       if(strcmp(name,fi->name))
@@ -198,7 +204,7 @@ bool  FileInfo::SameAs(const FileInfo *fi,bool only_newer,time_t prec)
    if(defined&SYMLINK_DEF && fi->defined&SYMLINK_DEF)
       return (strcmp(symlink,fi->symlink)==0);
 
-   if(defined&DATE && fi->defined&DATE)
+   if(defined&DATE && fi->defined&DATE && !(ignore&DATE))
    {
       if(only_newer && date<fi->date)
 	    return true;
@@ -206,7 +212,7 @@ bool  FileInfo::SameAs(const FileInfo *fi,bool only_newer,time_t prec)
 	 return false;
    }
 
-   if(defined&SIZE && fi->defined&SIZE)
+   if(defined&SIZE && fi->defined&SIZE && !(ignore&SIZE))
       if(size!=fi->size)
 	 return false;
 
@@ -238,7 +244,7 @@ void FileSet::Count(int *d,int *f,int *s,int *o)
    }
 }
 
-FileInfo *FileSet::FindByName(const char *name)
+FileInfo *FileSet::FindByName(const char *name) const
 {
    for(int i=0; i<fnum; i++)
    {
@@ -295,7 +301,7 @@ void FileSet::LocalRemove(const char *dir)
 }
 #endif
 
-void FileSet::LocalUtime(const char *dir)
+void FileSet::LocalUtime(const char *dir,bool only_dirs)
 {
    FileInfo *file;
    for(int i=0; i<fnum; i++)
@@ -303,8 +309,11 @@ void FileSet::LocalUtime(const char *dir)
       file=files[i];
       if(file->defined & (file->DATE|file->DATE_UNPREC))
       {
-	 if(file->defined & file->TYPE
-	 && file->filetype==file->SYMLINK)
+	 if(!(file->defined & file->TYPE))
+	    continue;
+	 if(file->filetype==file->SYMLINK)
+	    continue;
+	 if(only_dirs && file->filetype!=file->DIRECTORY)
 	    continue;
 
 	 const char *local_name=dir_file(dir,file->name);
@@ -339,3 +348,56 @@ void FileSet::LocalChmod(const char *dir,mode_t mask)
       }
    }
 }
+
+FileInfo *FileSet::curr()
+{
+   if(ind<fnum)
+      return files[ind];
+   return 0;
+}
+FileInfo *FileSet::next()
+{
+   if(ind<fnum)
+   {
+      ind++;
+      return curr();
+   }
+   return 0;
+}
+
+static int name_compare(const void *a,const void *b)
+{
+   FileInfo *pa=*(FileInfo*const*)a;
+   FileInfo *pb=*(FileInfo*const*)b;
+   return strcmp(pa->name,pb->name);
+}
+
+void FileSet::SortByName()
+{
+   qsort(files,fnum,sizeof(*files),name_compare);
+}
+
+void FileInfo::Init()
+{
+   name=NULL;
+   defined=0;
+   symlink=NULL;
+   data=0;
+}
+FileInfo::FileInfo(const FileInfo &fi)
+{
+   Init();
+   name=xstrdup(fi.name);
+   symlink=xstrdup(fi.symlink);
+   defined=fi.defined;
+   filetype=fi.filetype;
+   mode=fi.mode;
+   date=fi.date;
+   size=fi.size;
+}
+FileInfo::~FileInfo()
+{
+   xfree(name);
+   xfree(symlink);
+   xfree(data);
+}
