@@ -18,6 +18,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: FileSet.h,v 1.7 1999/09/08 16:06:54 lav Exp $ */
+
 #ifndef FILESET_H
 #define FILESET_H
 
@@ -37,6 +39,7 @@ public:
    mode_t   mode;
    time_t   date;
    long	    size;
+   void	    *data;
 
    enum	 type
    {
@@ -52,31 +55,18 @@ public:
    enum defined_bits
    {
       NAME=001,MODE=002,DATE=004,TYPE=010,SYMLINK_DEF=020,
-      DATE_UNPREC=040,SIZE=0100
+      DATE_UNPREC=040,SIZE=0100,
+
+      ALL_INFO=NAME|MODE|DATE|TYPE|SYMLINK_DEF|DATE_UNPREC|SIZE
    };
 
-   ~FileInfo()
-   {
-      xfree(name);
-      xfree(symlink);
-   }
-
+   ~FileInfo();
+   void Init();
    FileInfo()
-   {
-      name=NULL;
-      defined=0;
-      symlink=NULL;
-   }
-   FileInfo(const FileInfo &fi)
-   {
-      name=xstrdup(fi.name);
-      symlink=xstrdup(fi.symlink);
-      defined=fi.defined;
-      filetype=fi.filetype;
-      mode=fi.mode;
-      date=fi.date;
-      size=fi.size;
-   }
+      {
+	 Init();
+      }
+   FileInfo(const FileInfo &fi);
 
    void SetName(const char *n) { xfree(name); name=xstrdup(n); defined|=NAME; }
    void SetMode(mode_t m) { mode=m; defined|=MODE; }
@@ -94,8 +84,15 @@ public:
 
    void	 Merge(const FileInfo&);
 
-   bool	 SameAs(const FileInfo *,bool only_newer,time_t prec);
+   bool	 SameAs(const FileInfo *,bool only_newer,time_t prec,int ignore);
    bool	 OlderThan(time_t t);
+
+   void	 SetAssociatedData(void *d,int len)
+      {
+	 xfree(data);
+	 data=xmemdup(d,len);
+      }
+   void  *GetAssociatedData() { return data; }
 };
 
 class FileSet
@@ -122,37 +119,40 @@ public:
    void	 Add(FileInfo *);
    void	 Merge(const FileSet *);
    void	 Merge(char **);   // file list
-   void	 SubtractSame(const FileSet *,bool only_newer,time_t prec);
+   void	 SubtractSame(const FileSet *,bool only_newer,time_t prec,int ignore);
    void	 SubtractAny(const FileSet *);
    void  SubtractOlderThan(time_t t);
+   void  SubtractNotIn(const FileSet *);
 
    void	 Exclude(const char *prefix,regex_t *exclude,regex_t *include);
    void	 ExcludeDots();
 
    void	 rewind() { ind=0; }
-   FileInfo *curr() { return (ind<fnum ? files[ind] : 0); }
-   FileInfo *next() { return (ind<fnum&&++ind<fnum ? files[ind] : 0); }
+   FileInfo *curr();
+   FileInfo *next();
 
    void	 LocalRemove(const char *dir);
-   void	 LocalUtime(const char *dir);
+   void	 LocalUtime(const char *dir,bool only_dirs=false);
    void	 LocalChmod(const char *dir,mode_t mask=0);
 
    void Count(int *d,int *f,int *s,int *o);
 
-   FileInfo *FindByName(const char *name);
+   FileInfo *FindByName(const char *name) const;
 
-   void  SetSize(char *name,long size)
+   void  SetSize(const char *name,long size)
    {
       FileInfo *f=FindByName(name);
       if(f)
 	 f->SetSize(size);
    }
-   void  SetDate(char *name,time_t date)
+   void  SetDate(const char *name,time_t date)
    {
       FileInfo *f=FindByName(name);
       if(f)
 	 f->SetDate(date);
    }
+
+   void SortByName();
 };
 
 #endif // FILESET_H
