@@ -32,11 +32,12 @@
 #include "Filter.h"
 #include "xmalloc.h"
 #include "SignalHook.h"
+#include "misc.h"
 
 FDStream::FDStream(int new_fd,const char *new_name)
 {
    fd=new_fd;
-   name=xstrdup(new_name);
+   name=xstrdup(expand_home_relative(new_name));
    error_text=0;
 }
 FDStream::FDStream()
@@ -124,7 +125,7 @@ void InputFilter::Child(int *p)
 
 int OutputFilter::getfd()
 {
-   if(fd!=-1 || error())
+   if(fd!=-1 || error() || closed)
       return fd;
 
    if(second && second_fd==-1)
@@ -153,6 +154,8 @@ int OutputFilter::getfd()
       return -1;
    }
 
+   ProcWait::Signal(false);
+
    fflush(stderr);
    switch(pid=fork())
    {
@@ -177,7 +180,7 @@ int OutputFilter::getfd()
    case(-1): /* error */
       close(p[0]);
       close(p[1]);
-      return -1;
+      goto out;
    }
    /* parent */
    Parent(p);
@@ -192,9 +195,9 @@ int OutputFilter::getfd()
 
    int info;
    waitpid(pid,&info,WUNTRACED);
-
    w=new ProcWait(pid);
-
+out:
+   ProcWait::Signal(true);
    return fd;
 }
 
@@ -210,6 +213,7 @@ void OutputFilter::Init()
       oldcwd=0;
    }
    pg=0;
+   closed=false;
 }
 
 void OutputFilter::SetCwd(const char *cwd)
@@ -251,15 +255,28 @@ bool OutputFilter::Done()
    {
       close(fd);
       fd=-1;
+      closed=true;
    }
    w->Do();
    if(w->GetState()!=w->RUNNING)
       return true;
    return false;
 }
+bool OutputFilter::broken()
+{
+   if(w==0)
+      return false;
+   if(fd==-1)
+      return false;
+//    w->Do();
+   if(w->GetState()!=w->RUNNING)
+      return true; // filter process terminated - pipe is broken
+   return false;
+}
 
 void FileStream::setmtime(time_t t)
 {
+   getfd(); // this might create the file... But can fail retriably. FIXME.
    struct utimbuf ut;
    ut.actime=ut.modtime=t;
    utime(full_name,&ut);
