@@ -18,6 +18,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: Filter.h,v 1.4 1999/05/20 10:04:46 lav Exp $ */
+
 #ifndef FILTER_H
 #define FILTER_H
 
@@ -44,10 +46,11 @@ public:
    virtual void setmtime(time_t) {}
    virtual bool can_setmtime() { return false; }
    virtual void remove_if_empty() {}
-   virtual bool Done() { return 1; }
-   virtual bool usesfd(int fd) { return this->fd==fd; }
+   virtual bool Done() { return true; }
+   virtual bool usesfd(int n_fd) { return fd==n_fd; }
    virtual void Kill(int=SIGTERM) {}
    virtual pid_t GetProcGroup() { return 0; }
+   virtual bool broken() { return false; }
 };
 
 class OutputFilter : public FDStream
@@ -58,6 +61,8 @@ class OutputFilter : public FDStream
 
    char *oldcwd;
 
+   bool closed;
+
    void Init();
 
    virtual void Parent(int *p);	 // what to do with pipe if parent
@@ -78,9 +83,11 @@ public:
    int getfd();
    bool Done();
 
-   bool usesfd(int fd) { return FDStream::usesfd(fd) || fd<=2; }
+   bool usesfd(int n_fd) { return FDStream::usesfd(n_fd) || n_fd<=2; }
    void Kill(int sig=SIGTERM) { if(w) w->Kill(sig); }
    pid_t GetProcGroup() { return pg; }
+
+   bool broken();
 };
 
 class InputFilter : public OutputFilter
