@@ -18,12 +18,13 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: FtpListInfo.h,v 1.4 1999/04/22 13:41:33 lav Exp $ */
+
 #ifndef FTPLISTINFO_H
 #define FTPLISTINFO_H
 
-#include "ListInfo.h"
 #include "ftpclass.h"
-#include "rglob.h"
+#include "FtpGlob.h"
 
 class FtpListInfo : public ListInfo
 {
@@ -41,9 +42,10 @@ class FtpListInfo : public ListInfo
    Ftp::fileinfo *get_info;
    int get_info_cnt;
 
-   RemoteGlob *glob;
+   FtpGlob *glob;
 
-   FileSet *ParseFtpLongList(char **);
+public:
+   static FileSet *ParseFtpLongList(const char * const *lines,int *err);
 
 public:
    FtpListInfo(Ftp *session);
