@@ -18,20 +18,30 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: GetJob.cc,v 1.8.2.1 1999/12/18 23:48:37 lav Exp $ */
+
 #include <config.h>
-#include "GetJob.h"
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <fcntl.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+
+#include "GetJob.h"
 #include "misc.h"
 
+ResDecl res_clobber("xfer:clobber","yes",ResMgr::BoolValidate,0);
+
 int   GetJob::Do()
 {
    RateDrain();
 
    int m=STALL;
-   int fd,res;
+   int res;
+
+   if(deleting)
+      goto handle_deleting;
 
    if(!curr && args)
       NextFile();
@@ -42,14 +52,17 @@ int   GetJob::Do()
    if(in_buffer==0 && got_eof)
    {
       session->Close();
+      // now we can delete old file, since there is new one
+      RemoveBackupFile();
       if(delete_files)
       {
 	 if(!deleting)
 	 {
 	    deleting=true;
-	    session->Open(curr,Ftp::REMOVE);
+	    session->Open(curr,FA::REMOVE);
 	    m=MOVED;
 	 }
+      handle_deleting:
 	 res=session->Done();
 	 if(res<=0)
 	 {
@@ -71,20 +84,22 @@ int   GetJob::Do()
    {
       if(session->IsClosed())
       {
-	 if((fd=local->getfd())==-1)
-	 {
-	    if(!local->error())
-	    {
-	       block+=TimeOut(1000);
-	       return m;
-	    }
-	    fprintf(stderr,"%s: %s\n",op,local->error_text);
-	    failed++;
-	    NextFile();
-	    return MOVED;
-	 }
 	 if(cont)
 	 {
+	    // need to get current file size.
+	    // ensure the file is opened.
+	    if(local->getfd()==-1)
+	    {
+	       if(!local->error())
+	       {
+		  block+=TimeOut(1000);
+		  return m;
+	       }
+	       fprintf(stderr,"%s: %s\n",op,local->error_text);
+	       failed++;
+	       NextFile();
+	       return MOVED;
+	    }
 	    offset=local->getsize_and_seek_end();
 	    if(offset<0)
 	       offset=0;
@@ -98,7 +113,7 @@ int   GetJob::Do()
 	 session->WantSize(&size);
       }
       res=TryRead(session);
-      if(res<0 && res!=Ftp::DO_AGAIN)
+      if(res<0 && res!=FA::DO_AGAIN)
       {
 	 local->remove_if_empty();
 	 NextFile();
@@ -122,6 +137,7 @@ int   GetJob::Do()
 
 void GetJob::NextFile()
 {
+try_next:
    if(!args)
       return;
    if(local)
@@ -137,10 +153,31 @@ void GetJob::NextFile()
       return;
    }
    int flags=O_WRONLY|O_CREAT|(cont?0:O_TRUNC);
-   if(saved_cwd && l[0]!='/')
-      local=new FileStream(dir_file(saved_cwd,l),flags);
-   else
-      local=new FileStream(l,flags);
+   const char *f=(saved_cwd && l[0]!='/') ? dir_file(saved_cwd,l) : l;
+   made_backup=false;
+   if(!cont)
+   {
+      /* rename old file if exists */
+      struct stat st;
+      if(stat(f,&st)!=-1)
+      {
+	 if(st.st_size>0)
+	 {
+	    if(!(bool)res_clobber.Query(0))
+	    {
+	       eprintf(_("%s: %s: file already exists and xfer:clobber is unset\n"),op,l);
+	       failed++;
+	       goto try_next;
+	    }
+	    char *b=(char*)alloca(strlen(f)+2);
+	    strcpy(b,f);
+	    strcat(b,"~");
+	    if(rename(f,b)==0)
+	       made_backup=true;
+	 }
+      }
+   }
+   local=new FileStream(f,flags);
    XferJob::NextFile(r);
    file_time=(time_t)-2;
    if(set_file_time!=(time_t)-1)
@@ -149,3 +186,15 @@ void GetJob::NextFile()
       set_file_time=(time_t)-1;
    }
 }
+
+void GetJob::RemoveBackupFile()
+{
+   if(made_backup)
+   {
+      FileStream *f=(FileStream*)local; // we are sure it is FileStream
+      char *b=(char*)alloca(strlen(f->full_name)+2);
+      strcpy(b,f->full_name);
+      strcat(b,"~");
+      remove(b);
+   }
+}
