@@ -18,6 +18,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: GetJob.h,v 1.2 1998/09/18 19:12:32 lav Exp $ */
+
 #ifndef GETJOB_H
 #define GETJOB_H
 
@@ -32,6 +34,9 @@ protected:
    time_t file_time;
    bool delete_files:1;
    bool	deleting:1;
+   bool made_backup:1;
+
+   void RemoveBackupFile();
 
 public:
    int	 Do();
@@ -44,7 +49,7 @@ public:
    {
       set_file_time=(time_t)-1;
       file_time=(time_t)-2;
-      delete_files=deleting=false;
+      delete_files=deleting=made_backup=false;
    }
 };
 
