@@ -18,6 +18,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: Job.cc,v 1.7 1999/09/15 15:10:40 lav Exp $ */
+
 #include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -217,8 +219,13 @@ void  Job::ListDoneJobs()
       if(scan->jobno>=0 && (scan->parent==this || scan->parent==0)
          && scan->Done())
       {
-	 fprintf(f,_("[%d] Done (%s)\n"),scan->jobno,
+	 fprintf(f,_("[%d] Done (%s)"),scan->jobno,
 	    scan->cmdline?scan->cmdline:"?");
+	 const char *this_url=this->GetConnectURL();
+	 const char *that_url=scan->GetConnectURL();
+	 if(this_url && that_url && strcmp(this_url,that_url))
+	    fprintf(f," (wd: %s)",that_url);
+	 fprintf(f,"\n");
 	 scan->PrintStatus(0);
       }
    }
@@ -285,7 +292,7 @@ SessionJob::~SessionJob()
 
 void SessionJob::PrintStatus(int v)
 {
-   if(v<2)
+   if(v<2 || !session)
       return;
    const char *url=session->GetConnectURL();
    if(url)
@@ -294,10 +301,19 @@ void SessionJob::PrintStatus(int v)
 
 void Job::vfprintf(FILE *file,const char *fmt,va_list v)
 {
+   if(file!=stdout && file!=stderr)
+   {
+      ::vfprintf(file,fmt,v);
+      return;
+   }
    if(parent)
       parent->vfprintf(file,fmt,v);
    else
-      ::vfprintf(file,fmt,v);
+      top_vfprintf(file,fmt,v);
+}
+void Job::top_vfprintf(FILE *file,const char *fmt,va_list v)
+{
+   ::vfprintf(file,fmt,v);
 }
 
 void Job::perror(const char *f)
@@ -312,3 +328,45 @@ void Job::puts(const char *s)
 {
    printf("%s\n",s);
 }
+
+void Job::Bg()
+{
+   if(!fg)
+      return;
+   fg=false;
+   if(waiting && waiting!=this)
+      waiting->Bg();
+   if(fg_data)
+      fg_data->Bg();
+}
+void Job::Fg()
+{
+   if(fg)
+      return;
+   fg=true;
+   if(fg_data)
+      fg_data->Fg();
+   if(waiting && waiting!=this)
+      waiting->Fg();
+}
+
+int Job::AcceptSig(int s)
+{
+   if(waiting && waiting!=this)
+   {
+      if(waiting->AcceptSig(s)==WANTDIE)
+      {
+	 Job *new_waiting=waiting->waiting;
+	 waiting->waiting=0;
+	 delete waiting;
+	 waiting=new_waiting;
+      }
+   }
+   return WANTDIE;
+}
+
+void Job::ShowRunStatus(StatusLine *sl)
+{
+   if(waiting)
+      waiting->ShowRunStatus(sl);
+}
