@@ -18,6 +18,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: Job.h,v 1.5 1999/08/16 21:23:33 lav Exp $ */
+
 #ifndef JOB_H
 #define JOB_H
 
@@ -25,9 +27,9 @@
 #include <stdio.h>
 
 #include "SMTask.h"
-#include "ftpclass.h"
 #include "StatusLine.h"
 #include "fg.h"
+#include "FileAccess.h"
 
 class Job : public SMTask
 {
@@ -50,14 +52,14 @@ public:
    void	 AllocJobno();
 
    virtual void	  PrintStatus(int) {};
-   virtual void	  ShowRunStatus(StatusLine *) {}
+   virtual void	  ShowRunStatus(StatusLine *);
    virtual void	  SayFinal() {}; // final phrase of fg job
    virtual int	  Done()=0;
    virtual int	  ExitCode() { return 0; }
    virtual int	  Do()=0;
-   virtual int	  AcceptSig(int) { return WANTDIE; }
-   void	 Bg() { fg=false; if(fg_data) fg_data->Bg(); }
-   void	 Fg() { fg=true;  if(fg_data) fg_data->Fg(); }
+   virtual int	  AcceptSig(int);
+   virtual void	  Bg();
+   virtual void	  Fg();
 
    char	 *cmdline;
    virtual void ListJobs(int verbose_level,int indent=0);
@@ -75,8 +77,9 @@ public:
    void SendSig(int n,int sig);
    void KillAll();
 
+   void vfprintf(FILE *file,const char *fmt,va_list v);
    // CmdExec redefines this, and traps all messages of its children.
-   virtual void vfprintf(FILE *file,const char *fmt,va_list v);
+   virtual void top_vfprintf(FILE *file,const char *fmt,va_list v);
 
    // C-like functions calling vfprintf
    void eprintf(const char *fmt,...) PRINTF_LIKE(2,3);
@@ -87,6 +90,8 @@ public:
 
    Job();
    virtual ~Job();
+
+   virtual const char *GetConnectURL() { return 0; }
 };
 
 class SessionJob : public Job
@@ -101,6 +106,12 @@ protected:
 public:
    void PrintStatus(int v);
    FileAccess *session;
+   const char *GetConnectURL()
+      {
+	 if(!session)
+	    return 0;
+	 return session->GetConnectURL();
+      }
 };
 
 #endif /* JOB_H */
