@@ -18,7 +18,10 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: LocalAccess.cc,v 1.12.2.1 1999/12/16 20:02:39 lav Exp $ */
+
 /* this is not very useful, just a proof of concept */
+/* this is used in mirror command to build local files list */
 
 #include <config.h>
 #include <errno.h>
@@ -27,9 +30,10 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <dirent.h>
+#include <glob.h>
+#include <utime.h>
 
 #include "LocalAccess.h"
-#include "ProtoList.h"
 #include "xmalloc.h"
 #include "xalloca.h"
 #include "xstring.h"
@@ -38,7 +42,7 @@
 void LocalAccess::ClassInit()
 {
    // register the class
-   (void)new Protocol("file",LocalAccess::New);
+   Register("file",LocalAccess::New);
 }
 
 void LocalAccess::Init()
@@ -66,8 +70,13 @@ LocalAccess::~LocalAccess()
 
 void LocalAccess::errno_handle()
 {
-   xfree(last_error_resp);
-   last_error_resp=xstrdup(strerror(errno));
+   xfree(error);
+   const char *err=strerror(errno);
+   error=(char*)xmalloc(xstrlen(file)+xstrlen(file1)+strlen(err)+20);
+   if(mode==RENAME)
+      sprintf(error,"rename(%s, %s): %s",file,file1,err);
+   else
+      sprintf(error,"%s: %s",file,err);
 }
 
 int LocalAccess::Done()
@@ -88,7 +97,7 @@ int LocalAccess::Done()
 
 int LocalAccess::Do()
 {
-   if(error_code<0 || done)
+   if(Error() || done)
       return STALL;
    int m=STALL;
    switch((open_mode)mode)
@@ -100,7 +109,7 @@ int LocalAccess::Do()
    case(QUOTE_CMD):
       if(stream==0)
       {
-	 char cmd[10+xstrlen(file)];
+	 char *cmd=(char*)alloca(10+xstrlen(file));
 	 if(mode==LIST)
 	 {
 	    if(file && file[0])
@@ -128,9 +137,7 @@ int LocalAccess::Do()
       {
 	 if(stream->error())
 	 {
-	    error_code=FATAL;
-	    xfree(last_error_resp);
-	    last_error_resp=xstrdup(stream->error_text);
+	    Fatal(stream->error_text);
 	    return MOVED;
 	 }
 	 block+=TimeOut(1000);
@@ -188,6 +195,14 @@ int LocalAccess::Do()
       }
       done=true;
       return MOVED;
+   case(CHANGE_MODE):
+      if(chmod(dir_file(cwd,file),chmod_mode)==-1)
+      {
+	 errno_handle();
+	 error_code=NO_FILE;
+      }
+      done=true;
+      return MOVED;
 
    case(RETRIEVE):
    case(STORE):
@@ -208,15 +223,29 @@ int LocalAccess::Do()
       {
 	 if(stream->error())
 	 {
-	    error_code=NO_FILE;
-	    xfree(last_error_resp);
-	    last_error_resp=xstrdup(stream->error_text);
+	    SetError(NO_FILE,stream->error_text);
 	    return MOVED;
 	 }
 	 block+=TimeOut(1000);
 	 return m;
       }
       stream->Kill(SIGCONT);
+      if(opt_size || opt_date)
+      {
+	 struct stat st;
+	 if(fstat(stream->getfd(),&st)==-1)
+	 {
+	    if(opt_size) *opt_size=-1L;
+	    if(opt_date) *opt_date=(time_t)-1;
+	 }
+	 else
+	 {
+	    if(opt_size) *opt_size=st.st_size;
+	    if(opt_date) *opt_date=st.st_mtime;
+	 }
+	 opt_size=0;
+	 opt_date=0;
+      }
       block+=PollVec(stream->getfd(),(mode==STORE?POLLOUT:POLLIN));
       return m;
 
@@ -243,6 +272,11 @@ void LocalAccess::fill_array_info()
 	 f->size=st.st_size;
 	 f->time=st.st_mtime;
       }
+      else
+      {
+	 f->size=-1L;
+	 f->time=(time_t)-1;
+      }
    }
 }
 
@@ -264,35 +298,27 @@ int LocalAccess::Read(void *buf,int size)
    }
    stream->Kill(SIGCONT);
 read_again:
-   int res=Poll(fd,POLLIN);
-   if(res&(POLLIN|POLLNVAL))
+   int res=read(fd,buf,size);
+   if(res<0)
    {
-      int res=read(fd,buf,size);
-      if(res<0)
-      {
-	 saved_errno=errno;
-	 return SEE_ERRNO;
-      }
-      if(res==0)
-	 return res;
-      real_pos+=res;
-      if(real_pos<=pos)
-	 goto read_again;
-      int shift;
-      if((shift=pos+res-real_pos)>0)
-      {
-	 memmove(buf,(char*)buf+shift,size-shift);
-	 res-=shift;
-      }
-      pos+=res;
-      return(res);
+      if(errno==EINTR || errno==EAGAIN)
+	 return DO_AGAIN;
+      saved_errno=errno;
+      return SEE_ERRNO;
    }
-   if(res)
+   if(res==0)
+      return res; // eof
+   real_pos+=res;
+   if(real_pos<=pos)
+      goto read_again;
+   int shift;
+   if((shift=pos+res-real_pos)>0)
    {
-      // looks like eof
-      return 0;
+      memmove(buf,(char*)buf+shift,size-shift);
+      res-=shift;
    }
-   return DO_AGAIN;
+   pos+=res;
+   return(res);
 }
 
 int LocalAccess::Write(const void *buf,int size)
@@ -317,21 +343,34 @@ int LocalAccess::Write(const void *buf,int size)
       }
    }
    stream->Kill(SIGCONT);
-   int res=Poll(fd,POLLOUT);
-   if(res&(POLLOUT|POLLNVAL))
+
+   int res=write(fd,buf,size);
+   if(res<0)
    {
-      int res=write(fd,buf,size);
-      if(res>=0)
-	 return res;
+      if(errno==EINTR || errno==EAGAIN)
+	 return DO_AGAIN;
       saved_errno=errno;
       return SEE_ERRNO;
    }
 
-   return 0;
+   pos=(real_pos+=res);
+   return res;
 }
 
 int LocalAccess::StoreStatus()
 {
+   if(stream)
+   {
+      delete stream;
+      stream=0;
+      if(entity_date!=(time_t)-1)
+      {
+	 static struct utimbuf ut;
+	 ut.actime=ut.modtime=entity_date;
+	 utime(dir_file(cwd,file),&ut);
+      }
+   }
+
    if(error_code<0)
       return error_code;
 
@@ -412,6 +451,7 @@ int LocalListInfo::Do()
       if(lstat(name,&st)==-1)
 	 continue;
 
+check_again:
       FileInfo::type t;
       if(S_ISDIR(st.st_mode))
 	 t=FileInfo::DIRECTORY;
@@ -419,7 +459,15 @@ int LocalListInfo::Do()
 	 t=FileInfo::NORMAL;
 #ifdef HAVE_LSTAT
       else if(S_ISLNK(st.st_mode))
+      {
+	 if(follow_symlinks)
+	 {
+	    if(stat(name,&st)!=-1)
+	       goto check_again;
+	    // dangling symlink, don't follow it.
+	 }
 	 t=FileInfo::SYMLINK;
+      }
 #endif
       else
 	 continue;   // ignore other type files
@@ -446,3 +494,54 @@ int LocalListInfo::Do()
    done=true;
    return MOVED;
 }
+
+Glob *LocalAccess::MakeGlob(const char *pattern)
+{
+   return new LocalGlob(cwd,pattern);
+}
+
+LocalGlob::LocalGlob(const char *cwd,const char *pattern)
+   : Glob(pattern)
+{
+   glob_t g;
+   char *oldcwd=xgetcwd();
+   if(!oldcwd)
+   {
+      SetError("cannot get current directory");
+      return;
+   }
+   if(chdir(cwd)==-1)
+   {
+      const char *se=strerror(errno);
+      char *err=(char*)alloca(strlen(cwd)+strlen(se)+20);
+      sprintf(err,"chdir(%s): %s",cwd,se);
+      SetError(err);
+      xfree(oldcwd);
+      return;
+   }
+
+   glob(pattern, 0, NULL, &g);
+
+   if(chdir(oldcwd)==-1)
+      fprintf(stderr,"chdir(%s): %s",oldcwd,strerror(errno));
+   xfree(oldcwd);
+   for(unsigned i=0; i<g.gl_pathc; i++)
+      add(g.gl_pathv[i],strlen(g.gl_pathv[i]));
+   globfree(&g);
+}
+int LocalGlob::Do()
+{
+   if(!done)
+   {
+      done=true;
+      return MOVED;
+   }
+   return STALL;
+}
+
+#ifdef MODULE
+CDECL void module_init()
+{
+   LocalAccess::ClassInit();
+}
+#endif
