@@ -18,11 +18,12 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: LocalAccess.h,v 1.2 1999/03/17 14:15:50 lav Exp $ */
+
 #ifndef LOCALACCESS_H
 #define LOCALACCESS_H
 
 #include "FileAccess.h"
-#include "ListInfo.h"
 #include "Filter.h"
 
 class LocalAccess : public FileAccess
@@ -54,6 +55,7 @@ public:
    static void ClassInit();
 
    ListInfo *MakeListInfo();
+   Glob	    *MakeGlob(const char *pattern);
 };
 
 class LocalListInfo : public ListInfo
@@ -65,4 +67,12 @@ public:
    int Do();
 };
 
+class LocalGlob : public Glob
+{
+public:
+   LocalGlob(const char *cwd,const char *pattern);
+   const char *Status() { return "..."; }
+   int Do();
+};
+
 #endif//LOCALACCESS_H
