@@ -18,14 +18,18 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: LsCache.cc,v 1.6 1999/09/19 19:58:24 lav Exp $ */
+
 #include <config.h>
 #include "LsCache.h"
 #include "xmalloc.h"
+#include "plural.h"
+#include "misc.h"
 
 LsCache *LsCache::chain=0;
 bool	 LsCache::use=true;
 long	 LsCache::sizelimit=1024*1024;
-time_t	 LsCache::ttl=60*60;  // time to live = 60 minutes
+TimeInterval LsCache::ttl("60m");  // time to live = 60 minutes
 LsCache::ExpireHelper LsCache::expire_helper;
 
 void LsCache::CheckSize()
@@ -59,8 +63,11 @@ void LsCache::CheckSize()
    }
 }
 
-void LsCache::Add(FileAccess *p_loc,const char *a,int m,char *d,int l)
+void LsCache::Add(FileAccess *p_loc,const char *a,int m,const char *d,int l)
 {
+   if(!strcmp(p_loc->GetProto(),"file"))
+      return;  // don't cache local objects
+
    CheckSize();
 
    LsCache *scan;
@@ -93,6 +100,15 @@ void LsCache::Add(FileAccess *p_loc,const char *a,int m,char *d,int l)
    return;
 }
 
+void LsCache::Add(FileAccess *p_loc,const char *a,int m,const Buffer *ubuf)
+{
+   const char *cache_buffer;
+   int cache_buffer_size;
+   ubuf->GetSaved(&cache_buffer,&cache_buffer_size);
+   if(cache_buffer && cache_buffer_size>0)
+      LsCache::Add(p_loc,a,m,cache_buffer,cache_buffer_size);
+}
+
 int LsCache::Find(FileAccess *p_loc,const char *a,int m,char **d,int *l)
 {
    if(!use)
@@ -103,7 +119,8 @@ int LsCache::Find(FileAccess *p_loc,const char *a,int m,char **d,int *l)
    {
       if(scan->mode==m && !strcmp(scan->arg,a) && p_loc->SameLocationAs(scan->loc))
       {
-	 *d=(char*)xmemdup(scan->data,*l=scan->data_len);
+	 if(d && l)
+	    *d=(char*)xmemdup(scan->data,*l=scan->data_len);
 	 return 1;
       }
    }
@@ -135,48 +152,42 @@ void LsCache::List()
       puts(_("Cache is on"));
    else
       puts(_("Cache is off"));
+
    long vol=0;
    for(LsCache *scan=chain; scan; scan=scan->next)
       vol+=scan->data_len;
-   if(vol==1)
-   	   // xgettext:c-format
-	   printf(_("1 byte cached"));
-   else
-	   printf(_("%ld bytes cached"),vol);
+
+   printf(plural("%ld $#l#byte|bytes$ cached",vol),vol);
+
    if(sizelimit<0)
       puts(_(", no size limit"));
    else
       printf(_(", maximum size %ld\n"),sizelimit);
-   if(ttl==0)
+
+   if(ttl.IsInfty() || ttl.Seconds()==0)
       puts(_("Cache entries do not expire"));
-   else if(ttl<60)
-   {
-      if(ttl==1)
-	      puts(_("Cache entries expire in 1 second"));
-      else
-	      printf(_("Cache entries expire in %ld seconds\n"),(long)ttl);
-   }
+   else if(ttl.Seconds()<60)
+      printf(plural("Cache entries expire in %ld $#l#second|seconds$\n",
+		     long(ttl.Seconds())),long(ttl.Seconds()));
    else
    {
-      long ttl_min=(long)(ttl+30)/60;
-      if(ttl_min==1)
-	      puts(_("Cache entries expire in 1 minute"));
-      else
-	      printf(_("Cache entries expire in %ld minutes\n"),ttl_min);
+      long ttl_min=(long)(ttl.Seconds()+30)/60;
+      printf(plural("Cache entries expire in %ld $#l#minute|minutes$\n",
+		     ttl_min),ttl_min);
    }
 }
 
 int LsCache::ExpireHelper::Do()
 {
-   if(ttl==0)
+   if(ttl.IsInfty() || ttl.Seconds()==0)
       return STALL;
    time_t ct=time(0);
-   if(!expiring || expiring->timestamp+ttl <= ct)
+   if(!expiring || expiring->timestamp+ttl.Seconds() <= ct)
    {
       LsCache **scan=&LsCache::chain;
       while(*scan)
       {
-	 if((*scan)->timestamp+ttl <= ct)
+	 if((*scan)->timestamp+ttl.Seconds() <= ct)
 	 {
 	    LsCache *tmp=*scan;
 	    *scan=tmp->next;
@@ -190,6 +201,42 @@ int LsCache::ExpireHelper::Do()
       if(!expiring)
 	 return STALL;
    }
-   block+=TimeOut(expiring->timestamp+ttl-ct);
+   time_t t_out=expiring->timestamp+ttl.Seconds()-ct;
+   if(t_out>1024)
+      t_out=1024;
+   Timeout(t_out*1000);
    return STALL;
 }
+
+void LsCache::Changed(change_mode m,FileAccess *f,const char *dir)
+{
+   char *fdir=alloca_strdup(dir_file(f->GetCwd(),dir));
+   if(m==FILE_CHANGED)
+   {
+      char *slash=strrchr(fdir,'/');
+      if(!slash)
+	 fdir[0]=0;
+      else if(slash>fdir)
+	 *slash=0;
+      else
+	 fdir[1]=0;
+   }
+   int fdir_len=strlen(fdir);
+
+   LsCache **scan=&LsCache::chain;
+   while(*scan)
+   {
+      FileAccess *sloc=(*scan)->loc;
+      if(f->SameLocationAs(sloc) || (f->SameSiteAs(sloc)
+	       && (m==TREE_CHANGED?
+		     !strncmp(fdir,dir_file(sloc->GetCwd(),(*scan)->arg),fdir_len)
+		   : !strcmp (fdir,dir_file(sloc->GetCwd(),(*scan)->arg)))))
+      {
+	 LsCache *tmp=*scan;
+	 *scan=tmp->next;
+	 delete tmp;
+	 continue;
+      }
+      scan=&scan[0]->next;
+   }
+}
