@@ -18,11 +18,15 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: LsCache.h,v 1.8 1999/09/19 19:58:24 lav Exp $ */
+
 #ifndef LSCACHE_H
 #define LSCACHE_H
 
 #include <time.h>
-#include "ftpclass.h"
+#include "FileAccess.h"
+
+class Buffer;
 
 class LsCache
 {
@@ -38,9 +42,11 @@ class LsCache
    static LsCache *chain;
    static bool use;
    static long sizelimit;
-   static time_t ttl;
+   static TimeInterval ttl;
 
-   friend class ExpireHelper : public SMTask
+   class ExpireHelper;
+   friend class LsCache::ExpireHelper;
+   class ExpireHelper : public SMTask
    {
    public:
       LsCache *expiring;
@@ -52,15 +58,33 @@ class LsCache
    static void CheckSize();
 
 public:
-   static void Add(FileAccess *p_loc,const char *a,int m,char *d,int l);
+   static void Add(FileAccess *p_loc,const char *a,int m,const char *d,int l);
+   static void Add(FileAccess *p_loc,const char *a,int m,const Buffer *ubuf);
    static int Find(FileAccess *p_loc,const char *a,int m,char **d,int *l);
 
+   enum change_mode { FILE_CHANGED, DIR_CHANGED, TREE_CHANGED };
+   static void Changed(change_mode m,FileAccess *f,const char *what);
+   static void FileChanged(FileAccess *f,const char *file)
+      {
+	 Changed(FILE_CHANGED,f,file);
+      }
+   static void DirectoryChanged(FileAccess *f,const char *dir)
+      {
+	 Changed(DIR_CHANGED,f,dir);
+      }
+   static void TreeChanged(FileAccess *f,const char *dir)
+      {
+	 Changed(TREE_CHANGED,f,dir);
+      }
+
    static void List();
    static void Flush();
    static void On() { use=true; }
    static void Off() { use=false; }
+   static bool IsEnabled() { return use; }
    static void SetSizeLimit(long l) { sizelimit=l; }
-   static void SetExpire(time_t t) { ttl=t; }
+   static long SizeLimit() { return sizelimit; }
+   static void SetExpire(const TimeInterval &t) { ttl=t; }
 
    ~LsCache();
 
