@@ -18,6 +18,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: LsJob.cc,v 1.10 1999/08/04 17:01:18 lav Exp $ */
+
 #include <config.h>
 #include "LsJob.h"
 #include "LsCache.h"
@@ -25,19 +27,7 @@
 #include <unistd.h>
 #endif
 
-int strip_cr(char *buffer,int size)
-{
-   for(int i=0; i<size; i++)
-   {
-      if(buffer[i]=='\r')
-      {
-	 memmove(buffer+i,buffer+i+1,size-i-1);
-	 i--;
-	 size--;
-      }
-   }
-   return size;
-}
+#define super XferJob
 
 int   LsJob::Done()
 {
@@ -68,52 +58,75 @@ int   LsJob::Do()
    if(from_cache && !got_eof)
    {
       // we have cached data
-
-      int rest=cache_buffer_size-(cache_buffer_ptr-cache_buffer);
-      if(rest==0)
+      buffer=cache_buffer;
+      in_buffer=cache_buffer_size;
+      cache_buffer=0;
+      got_eof=true;
+   }
+   if(in_buffer==0 && got_eof)
+   {
+      // we are done
+      NextFile();
+      m=MOVED;
+      return m;
+   }
+   if(!got_eof)
+   {
+      if(dl)
       {
-	 cache_buffer=cache_buffer_ptr=0;
-	 got_eof=true;
-	 m=MOVED;
-      }
-      else
-      {
-	 if(in_buffer<buffer_size)
+	 if(dl->Error())
 	 {
-	    if(rest>buffer_size-in_buffer)
-	       rest=buffer_size-in_buffer;
-	    memcpy(buffer+in_buffer,cache_buffer_ptr,rest);
-	    in_buffer+=rest;
-	    cache_buffer_ptr+=rest;
+	    eprintf("%s: %s\n",op,dl->ErrorText());
+	    failed++;
+	    NextFile();
+	    return MOVED;
+	 }
+	 if(in_buffer==buffer_size)
+	 {
+	    if(buffer_size<0x10000)
+	    {
+	       if(buffer==0)
+		  buffer=(char*)xmalloc(buffer_size=0x1000);
+	       else
+		  buffer=(char*)xrealloc(buffer,buffer_size*=2);
+	    }
+	    else
+	    {
+	       dl->Suspend();
+	       goto try_write;
+	    }
+	 }
+	 dl->Resume();
+	 const char *tmpbuf=0;
+	 dl->Get(&tmpbuf,&res);
+	 if(tmpbuf==0)
+	 {
+	    // EOF
+	    got_eof=true;
 	    m=MOVED;
 	 }
+	 else if(res>0)
+	 {
+	    if((unsigned)res > buffer_size-in_buffer)
+	       res=buffer_size-in_buffer;
+	    memcpy(buffer+in_buffer,tmpbuf,res);
+	    in_buffer+=res;
+	    offset+=res;
+	    dl->Skip(res);
+
+	    CountBytes(res);
+	 }
       }
-   }
-   else
-   {
-      if(in_buffer==0 && got_eof)
-      {
-	 // we are done
-	 NextFile();
-	 m=MOVED;
-	 return m;
-      }
-      if(!got_eof)
+      else // !dl
       {
 	 res=TryRead(session);
-	 if(res<0 && res!=Ftp::DO_AGAIN)
+	 if(res<0 && res!=FA::DO_AGAIN)
 	 {
-   //	    local->remove_if_empty();
 	    NextFile();
 	    return MOVED;
 	 }
 	 else if(res>=0)
 	 {
-	    int diff=res-strip_cr(buffer+in_buffer-res,res);
-	    res-=diff;
-	    in_buffer-=diff;
-	    bytes_transferred-=diff;
-	    offset-=diff;
 	    m=MOVED;
 
 	    if(res>0)
@@ -128,14 +141,14 @@ int   LsJob::Do()
 	       {
 		  LsCache::Add(session,curr,mode,
 				 cache_buffer,cache_buffer_size);
-		  free(cache_buffer);
+		  xfree(cache_buffer);
 		  cache_buffer=0;
 	       }
 	    }
 	 }
       }
    }
-
+try_write:
    res=TryWrite(local);
    if(res<0)
    {
@@ -148,7 +161,7 @@ int   LsJob::Do()
    return m;
 }
 
-LsJob::LsJob(FileAccess *s,FDStream *l,char *a,int mode) : XferJob(s)
+LsJob::LsJob(FileAccess *s,FDStream *l,ArgV *a,int mode) : XferJob(s)
 {
    op="ls";
    local=l;
@@ -158,51 +171,85 @@ LsJob::LsJob(FileAccess *s,FDStream *l,char *a,int mode) : XferJob(s)
    this->mode=mode;
 
    print_run_status=!local->usesfd(1);
-   arg=a;
+   args=a;
+   arg=a->Combine(1);
    XferJob::NextFile(arg);
 
    from_cache=false;
-   if(LsCache::Find(session,arg,mode,
-		     &cache_buffer,&cache_buffer_size))
+   cache_buffer=0;
+   cache_buffer_size=0;
+
+   dl=0;
+   if(mode==FA::LONG_LIST)
+      dl=session->MakeDirList(args);
+   if(!dl)
    {
-      cache_buffer_ptr=cache_buffer;
-      from_cache=true;
-   }
-   else
-   {
-      session->Open(arg,mode);
-      cache_buffer=cache_buffer_ptr=0;
-      cache_buffer_size=0;
-      from_cache=false;
+      if(LsCache::Find(session,arg,mode,
+			&cache_buffer,&cache_buffer_size))
+      {
+	 from_cache=true;
+      }
+      else
+      {
+	 session->Open(arg,mode);
+      }
    }
 }
 LsJob::~LsJob()
 {
-   free(arg);
+   xfree(arg); arg=0;
+   delete args;
    if(local)
    {
       delete local;
       local=0;
    }
-   if(cache_buffer)
-   {
-      free(cache_buffer);
-      cache_buffer=cache_buffer_ptr=0;
-   }
+   xfree(cache_buffer);
+   cache_buffer=0;
+   if(dl)
+      delete dl;
 }
 
 void LsJob::NextFile()
 {
    XferJob::NextFile(0);
+   if(dl)
+   {
+      delete dl;
+      dl=0;
+   }
 }
 
 void LsJob::NoCache()
 {
+   if(dl)
+   {
+      dl->UseCache(false);
+      return;
+   }
    if(from_cache)
    {
-      session->Open(arg,Ftp::LONG_LIST);
-      cache_buffer=cache_buffer_ptr=0;
-      cache_buffer_size=0;
+      session->Open(arg,mode);
+      if(cache_buffer)
+      {
+	 xfree(cache_buffer);
+	 cache_buffer=0;
+	 cache_buffer_size=0;
+      }
       from_cache=false;
    }
+   session->UseCache(false);
+}
+
+void  LsJob::ShowRunStatus(StatusLine *s)
+{
+   if(!print_run_status)
+      return;
+   if(Done())
+      return;
+
+   if(!dl)
+      super::ShowRunStatus(s);
+   else
+      s->Show("%s",dl->Status());
 }
