@@ -18,6 +18,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: LsJob.h,v 1.5 1999/05/26 13:47:10 lav Exp $ */
+
 #ifndef LSJOB_H
 #define LSJOB_H
 
@@ -28,25 +30,30 @@ class LsJob : public XferJob
 {
 protected:
    char	 *arg;
+   ArgV  *args;
    FDStream *local;
-   
+
    void NextFile();
 
    char	 *cache_buffer;
-   char	 *cache_buffer_ptr;
-   int	 cache_buffer_size;
+/*   char	 *cache_buffer_ptr;*/
+   int   cache_buffer_size;
    bool	 from_cache;
 
    int	 mode;
 
+   DirList *dl;
+
 public:
    int	 Do();
    int	 Done();
 
-   LsJob(FileAccess *s,FDStream *g,char *a,int mode=Ftp::LONG_LIST);
+   LsJob(FileAccess *s,FDStream *g,ArgV *args,int mode=FA::LONG_LIST);
    ~LsJob();
 
    void	 NoCache();
+
+   void	 ShowRunStatus(StatusLine *s);
 };
 
 #endif /* LSJOB_H */
