@@ -18,6 +18,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: MirrorJob.cc,v 1.28.2.1 2000/01/21 19:41:12 lav Exp $ */
+
 #include <config.h>
 
 #include <sys/types.h>
@@ -25,17 +27,20 @@
 #include <errno.h>
 #include <unistd.h>
 #include <assert.h>
-#include "LocalAccess.h"
 #include "MirrorJob.h"
+#include "CmdExec.h"
 #include "rmJob.h"
 #include "mkdirJob.h"
 #include "PutJob.h"
+#include "ChmodJob.h"
 #include "misc.h"
 #include "xalloca.h"
+#include "plural.h"
+#include "getopt.h"
 
 void  MirrorJob::PrintStatus(int v)
 {
-   char *tab="\t";
+   const char *tab="\t";
 
    if(v!=-1)
       SessionJob::PrintStatus(v);
@@ -45,25 +50,23 @@ void  MirrorJob::PrintStatus(int v)
    if(!Done())
       return;
 
-   // FIXME: i18n
-   printf("%sTotal: %d director%s, %d file%s, %d symlink%s\n",tab,
-      dirs,dirs==1?"y":"ies",
-      tot_files,tot_files==1?"":"s",
-      tot_symlinks,tot_symlinks==1?"":"s");
+   printf(plural(N_("%sTotal: %d director$y|ies$, %d file$|s$, %d symlink$|s$\n"),
+		     dirs,tot_files,tot_symlinks),
+      tab,dirs,tot_files,tot_symlinks);
    if(new_files || new_symlinks)
-      printf("%sNew: %d file%s, %d symlink%s\n",tab,
-	 new_files,new_files==1?"":"s",
-	 new_symlinks,new_symlinks==1?"":"s");
+      printf(plural(N_("%sNew: %d file$|s$, %d symlink$|s$\n"),
+		     new_files,new_symlinks),
+	 tab,new_files,new_symlinks);
    if(mod_files || mod_symlinks)
-      printf("%sModified: %d file%s, %d symlink%s\n",tab,
-	 mod_files,mod_files==1?"":"s",
-	 mod_symlinks,mod_symlinks==1?"":"s");
+      printf(plural(N_("%sModified: %d file$|s$, %d symlink$|s$\n"),
+		     mod_files,mod_symlinks),
+	 tab,mod_files,mod_symlinks);
    if(del_dirs || del_files || del_symlinks)
-      printf("%s%semoved: %d director%s, %d file%s, %d symlink%s\n",tab,
-	 flags&DELETE?"R":"To be r",
-	 del_dirs,del_dirs==1?"y":"ies",
-	 del_files,del_files==1?"":"s",
-	 del_symlinks,del_symlinks==1?"":"s");
+      printf(plural(flags&DELETE ?
+	       N_("%sRemoved: %d director$y|ies$, %d file$|s$, %d symlink$|s$\n")
+	      :N_("%sTo be removed: %d director$y|ies$, %d file$|s$, %d symlink$|s$\n"),
+	      del_dirs,del_files,del_symlinks),
+	 tab,del_dirs,del_files,del_symlinks);
 }
 
 void  MirrorJob::ShowRunStatus(StatusLine *s)
@@ -74,7 +77,6 @@ void  MirrorJob::ShowRunStatus(StatusLine *s)
    {
    case(INITIAL_STATE):
    case(DONE):
-      s->Show("");
       break;
 
    // these have a sub-job
@@ -83,6 +85,7 @@ void  MirrorJob::ShowRunStatus(StatusLine *s)
    case(WAITING_FOR_RM_BEFORE_PUT):
    case(WAITING_FOR_MKDIR_BEFORE_SUBMIRROR):
    case(REMOTE_REMOVE_OLD):
+   case(REMOTE_CHMOD):
       if(waiting && !waiting->Done())
 	 waiting->ShowRunStatus(s);
       break;
@@ -121,15 +124,12 @@ void  MirrorJob::HandleFile(int how)
    {
       switch(file->filetype)
       {
-      case(file->NORMAL):
+      case(FileInfo::NORMAL):
       {
       try_get:
 	 bool cont_this=false;
 	 if(how!=0)
-	 {
-	    to_transfer->next();
-	    break;
-	 }
+	    goto skip;
 	 if(flags&REVERSE)
 	 {
 	    if(remote_set->FindByName(file->name)!=0)
@@ -138,6 +138,17 @@ void  MirrorJob::HandleFile(int how)
 			dir_file(remote_relative_dir,file->name));
 	       args=new ArgV("rm");
 	       args->Append(file->name);
+	       if(script)
+	       {
+		  char *cmd=args->CombineQuoted();
+		  fprintf(script,"%s",cmd);
+		  xfree(cmd);
+		  if(script_only)
+		  {
+		     delete args;
+		     goto skip;
+		  }
+	       }
 	       waiting=new rmJob(Clone(),args);
 	       waiting->parent=this;
 	       waiting->cmdline=args->Combine();
@@ -150,15 +161,31 @@ void  MirrorJob::HandleFile(int how)
 	 }
 	 if(lstat(local_name,&st)!=-1)
 	 {
+	    // few safety checks.
+	    FileInfo *old=new_files_set->FindByName(file->name);
+	    if(old)
+	       goto skip;  // file has appeared after mirror start
+	    old=old_files_set->FindByName(file->name);
+	    if(old && ((old->defined&old->SIZE && old->size!=st.st_size)
+		     ||(old->defined&old->DATE && old->date!=st.st_mtime)))
+	       goto skip;  // the file has changed after mirror start
+
 	    if((flags&CONTINUE) && S_ISREG(st.st_mode)
 	    && (file->defined&(file->DATE|file->DATE_UNPREC))
-	    && file->date + prec < st.st_mtime
+	    && file->date + prec.Seconds() < st.st_mtime
 	    && (file->defined&file->SIZE) && file->size >= st.st_size)
 	       cont_this=true;
 	    else
 	    {
 	       Report(_("Removing old local file `%s'"),
 			dir_file(local_relative_dir,file->name));
+	       if(script)
+	       {
+		  // FIXME: shell-quote file name.
+		  fprintf(script,"! rm %s",file->name);
+		  if(script_only)
+		     goto skip;
+	       }
 	       remove(local_name);
 	    }
 	    mod_files++;
@@ -172,28 +199,38 @@ void  MirrorJob::HandleFile(int how)
 		  dir_file(remote_relative_dir,file->name));
 	 args=new ArgV("get");
 	 args->Append(file->name);
+	 if(script)
+	 {
+	    char *cmd=args->CombineQuoted();
+	    fprintf(script,"%s",cmd);
+	    xfree(cmd);
+	    if(script_only)
+	    {
+	       delete args;
+	       goto skip;
+	    }
+	 }
 	 args->Append(local_name);
 	 gj=new GetJob(Clone(),args,cont_this);
 	 if(file->defined&(file->DATE|file->DATE_UNPREC))
 	    gj->SetTime(file->date);
+#if 0
+	 if(file->defined&file->SIZE)
+	    gj->SetSize(file->size);
+#endif
 	 waiting=gj;
 	 waiting->parent=this;
 	 waiting->cmdline=(char*)xmalloc(10+strlen(file->name));
 	 sprintf(waiting->cmdline,"\\get %s",file->name);
 	 break;
       }
-      case(file->DIRECTORY):
+      case(FileInfo::DIRECTORY):
       {
+      try_recurse:
 	 if(how!=1 || (flags&NO_RECURSION))
-	 {
-	    to_transfer->next();
-	    break;
-	 }
+	    goto skip;
 	 if(!strcmp(file->name,".") || !strcmp(file->name,".."))
-	 {
-	    to_transfer->next();
-	    break;
-	 }
+	    goto skip;
 	 if(flags&REVERSE)
 	 {
 	    if(!dir_made)
@@ -236,6 +273,11 @@ void  MirrorJob::HandleFile(int how)
 		  }
 	       }
 	    }
+	    else if(res==-1) // other error
+	    {
+	       eprintf("mirror: mkdir(%s): %s\n",local_name,strerror(errno));
+	       goto skip;
+	    }
 	 }
 	 // launch sub-mirror
 	 MirrorJob *mj=new MirrorJob(Clone(),local_name,remote_name);
@@ -259,22 +301,19 @@ void  MirrorJob::HandleFile(int how)
 	       xstrdup(dir_file(remote_relative_dir,file->name));
 
 	 mj->verbose_report=verbose_report;
+	 mj->newer_than=newer_than;
 
-	 dir_made=false;
+	 dir_made=false;   // for next directory
 
 	 break;
       }
-      case(file->SYMLINK):
+      case(FileInfo::SYMLINK):
 	 if(how!=0)
-	 {
-	    to_transfer->next();
-	    break;
-	 }
+	    goto skip;
 	 if(flags&REVERSE)
 	 {
 	    // can't create symlink remotely
-	    to_transfer->next();
-	    break;
+	    goto skip;
 	 }
 
 #ifdef HAVE_LSTAT
@@ -299,15 +338,26 @@ void  MirrorJob::HandleFile(int how)
 	       perror(local_name);
 	 }
 #endif /* LSTAT */
-	 to_transfer->next();
-	 break;
+	 goto skip;
       }
    }
    else
    {
+      FileInfo *local=local_set->FindByName(file->name);
+      if(local && (local->defined&local->TYPE)
+      && local->filetype==local->DIRECTORY)
+      {
+	 // assume it's a directory
+	 goto try_recurse;
+      }
       // no info on type -- try to get
       goto try_get;
    }
+   return;
+
+skip:
+   to_transfer->next();
+   return;
 }
 
 void  MirrorJob::InitSets(FileSet *source,FileSet *dest)
@@ -319,18 +369,27 @@ void  MirrorJob::InitSets(FileSet *source,FileSet *dest)
 
    same=new FileSet(source);
    to_transfer=new FileSet(source);
-   to_transfer->SubtractSame(dest,flags&ONLY_NEWER,prec);
+   int ignore=0;
+   if(prec.IsInfty())
+      ignore|=FileInfo::DATE;
+   to_transfer->SubtractSame(dest,flags&ONLY_NEWER,prec.Seconds(),ignore);
 
    same->SubtractAny(to_transfer);
 
    if(newer_than!=(time_t)-1)
       to_transfer->SubtractOlderThan(newer_than);
+
+   new_files_set=new FileSet(to_transfer);
+   new_files_set->SubtractAny(dest);
+   old_files_set=new FileSet(dest);
+   old_files_set->SubtractNotIn(to_transfer);
 }
 
 int   MirrorJob::Do()
 {
    int	 res;
    int	 m=STALL;
+   FileInfo *fi;
 
    switch(state)
    {
@@ -338,6 +397,37 @@ int   MirrorJob::Do()
       return STALL;
 
    case(INITIAL_STATE):
+      if(!local_set)
+      {
+	 local_session=FileAccess::New("file");
+	 if(!local_session)
+	 {
+	    eprintf("mirror: cannot create `file:' access object, installation error?\n");
+	    state=DONE;
+	    return MOVED;
+	 }
+	 local_session->Chdir(local_dir,false);
+	 list_info=local_session->MakeListInfo();
+	 list_info->UseCache(false);
+	 if(flags&RETR_SYMLINKS)
+	    list_info->FollowSymlinks();
+	 list_info->SetExclude(local_relative_dir,
+			rx_exclude?&rxc_exclude:0,rx_include?&rxc_include:0);
+
+	 while(!list_info->Done())
+	    list_info->Do();  // this should be fast
+
+	 if(list_info->Error())
+	    goto list_info_error;
+
+	 local_set=list_info->GetResult();
+	 delete list_info;
+	 list_info=0;
+	 delete local_session;
+	 local_session=0;
+
+	 local_set->ExcludeDots();  // don't need .. and .
+      }
       if(create_remote_dir)
       {
 	 create_remote_dir=false;
@@ -346,12 +436,13 @@ int   MirrorJob::Do()
 	 return MOVED;
       }
       session->Chdir(remote_dir);
+      while(session->Do()==MOVED);
       state=CHANGING_REMOTE_DIR;
       return MOVED;
 
    case(MAKE_REMOTE_DIR):
       res=session->Done();
-      if(res==Ftp::IN_PROGRESS)
+      if(res==FA::IN_PROGRESS)
 	 return m;
       session->Close();
       state=INITIAL_STATE;
@@ -359,7 +450,7 @@ int   MirrorJob::Do()
 
    case(CHANGING_REMOTE_DIR):
       res=session->Done();
-      if(res==Ftp::IN_PROGRESS)
+      if(res==FA::IN_PROGRESS)
 	 return STALL;
       if(res<0)
       {
@@ -381,6 +472,10 @@ int   MirrorJob::Do()
       	 state=DONE;
       	 return MOVED;
       }
+      list_info->UseCache(false);
+      list_info->Need(FileInfo::ALL_INFO);
+      if(flags&RETR_SYMLINKS)
+	 list_info->FollowSymlinks();
 
       list_info->SetExclude(remote_relative_dir,
 		     rx_exclude?&rxc_exclude:0,rx_include?&rxc_include:0);
@@ -409,21 +504,7 @@ int   MirrorJob::Do()
 
       remote_set->ExcludeDots(); // don't need .. and .
 
-      list_info=new LocalListInfo(local_dir);
-      list_info->SetExclude(local_relative_dir,
-		     rx_exclude?&rxc_exclude:0,rx_include?&rxc_include:0);
-
-      while(!list_info->Done())
-	 list_info->Do();  // this should be fast
-
-      if(list_info->Error())
-	 goto list_info_error;
-
-      local_set=list_info->GetResult();
-      delete list_info;
-      list_info=0;
-
-      local_set->ExcludeDots();  // don't need .. and .
+      // now we have both local and remote file sets.
 
       if(flags&REVERSE)
 	 InitSets(local_set,remote_set);
@@ -443,6 +524,7 @@ int   MirrorJob::Do()
 	 if(!waiting->Done())
 	    return STALL;
 	 delete waiting;
+	 waiting=0;
       }
       Report(_("Sending local file `%s'"),
 	       dir_file(local_relative_dir,file->name));
@@ -535,19 +617,19 @@ int   MirrorJob::Do()
 		  waiting=new rmJob(Clone(),args);
 		  waiting->parent=this;
 		  waiting->cmdline=args->Combine();
-		  state=REMOTE_REMOVE_OLD;
-		  return MOVED;
 	       }
-/*
-	       for(file=to_rm->curr(); file; file=to_rm->next())
+	       else if(flags&REPORT_NOT_DELETED)
 	       {
-		  Report(_("Not removing old remote file `%s'"),
-			   dir_file(remote_relative_dir,file->name));
+		  for(file=to_rm->curr(); file; file=to_rm->next())
+		  {
+		     Report(_("Old remote file `%s' is not removed"),
+			      dir_file(remote_relative_dir,file->name));
+		  }
 	       }
-*/
-	       state=DONE;
+	       state=REMOTE_REMOVE_OLD;
 	       return MOVED;
 	    }
+	    // else not REVERSE
 	    if(flags&DELETE)
 	    {
 	       while((file=to_rm->curr())!=0)
@@ -567,22 +649,31 @@ int   MirrorJob::Do()
 		  to_rm->next();
 	       }
 	    }
-/*
-	    else
+	    else if(flags&REPORT_NOT_DELETED)
 	    {
 	       for(file=to_rm->curr(); file; file=to_rm->next())
 	       {
-		  Report(_("Not removing old local file `%s'"),
+		  Report(_("Old local file `%s' is not removed"),
 			   dir_file(local_relative_dir,file->name));
 	       }
 	    }
-*/
+	    mode_t mode_mask=0;
+	    if(!(flags&ALLOW_SUID))
+	       mode_mask|=S_ISUID|S_ISGID;
+	    if(!(flags&NO_UMASK))
+	    {
+	       mode_t u=umask(022); // get+set
+	       umask(u);	    // retore
+	       mode_mask|=u;
+	    }
 	    if(!(flags&NO_PERMS))
-	       to_transfer->LocalChmod(local_dir,flags&ALLOW_SUID?0:S_ISUID|S_ISGID);
-	    to_transfer->LocalUtime(local_dir);
+	       to_transfer->LocalChmod(local_dir,mode_mask);
+	    to_transfer->LocalUtime(local_dir,/*only_dirs=*/true);
 	    if(!(flags&NO_PERMS))
-	       same->LocalChmod(local_dir,flags&ALLOW_SUID?0:S_ISUID|S_ISGID);
+	       same->LocalChmod(local_dir,mode_mask);
+#if 0 // this can cause problems if files really differ
 	    same->LocalUtime(local_dir); // the old mtime can differ up to prec
+#endif
 	    state=DONE;
 	    return MOVED;
 	 }
@@ -592,11 +683,46 @@ int   MirrorJob::Do()
       return m;
 
    case(REMOTE_REMOVE_OLD):
+      if(!waiting)
+	 goto pre_REMOTE_CHMOD;
       if(waiting->Done())
       {
 	 delete waiting;
+	 waiting=0;
 	 state=DONE;
-	 return MOVED;
+	 m=MOVED;
+	 goto pre_REMOTE_CHMOD;
+      }
+      return m;
+
+   pre_REMOTE_CHMOD:
+      to_transfer->rewind();
+      goto remote_chmod_next;
+   case(REMOTE_CHMOD):
+      if(waiting->Done())
+      {
+	 delete waiting;
+	 waiting=0;
+
+      remote_chmod_next:
+	 fi=to_transfer->curr();
+	 if(!fi)
+	 {
+	    state=DONE;
+	    return MOVED;
+	 }
+	 to_transfer->next();
+	 if(!(fi->defined&fi->MODE))
+	    goto remote_chmod_next;
+	 ArgV *a=new ArgV("chmod");
+	 a->Append(fi->name);
+	 ChmodJob *cj=new ChmodJob(Clone(),fi->mode,a);
+	 waiting=cj;
+	 waiting->parent=this;
+	 waiting->cmdline=a->Combine();
+	 cj->BeQuiet();   // chmod is not supported on all servers; be quiet.
+
+	 m=MOVED;
       }
       return m;
    }
@@ -605,7 +731,7 @@ int   MirrorJob::Do()
 }
 
 MirrorJob::MirrorJob(FileAccess *f,const char *new_local_dir,const char *new_remote_dir)
-   : SessionJob(f)
+   : SessionJob(f), prec("0")
 {
    verbose_report=0;
    parent_mirror=0;
@@ -617,8 +743,10 @@ MirrorJob::MirrorJob(FileAccess *f,const char *new_local_dir,const char *new_rem
 
    to_transfer=to_rm=same=0;
    remote_set=local_set=0;
+   new_files_set=old_files_set=0;
    file=0;
    list_info=0;
+   local_session=0;
 
    tot_files=new_files=mod_files=del_files=
    tot_symlinks=new_symlinks=mod_symlinks=del_symlinks=0;
@@ -627,14 +755,18 @@ MirrorJob::MirrorJob(FileAccess *f,const char *new_local_dir,const char *new_rem
    flags=0;
 
    rx_include=rx_exclude=0;
-
-   prec=0;  // time must be exactly same by default
+   memset(&rxc_include,0,sizeof(regex_t));   // for safety
+   memset(&rxc_exclude,0,sizeof(regex_t));   // for safety
 
    state=INITIAL_STATE;
 
    dir_made=false;
    create_remote_dir=false;
    newer_than=(time_t)-1;
+
+   script=0;
+   script_only=false;
+   script_needs_closing=false;
 }
 
 MirrorJob::~MirrorJob()
@@ -653,9 +785,15 @@ MirrorJob::~MirrorJob()
       delete to_rm;
    if(same)
       delete same;
+   if(new_files_set)
+      delete new_files_set;
+   if(old_files_set)
+      delete old_files_set;
    // don't delete this->file -- it is a reference
    if(list_info)
       delete list_info;
+   if(local_session)
+      delete local_session;
    if(rx_include)
    {
       free(rx_include);
@@ -666,54 +804,52 @@ MirrorJob::~MirrorJob()
       free(rx_exclude);
       regfree(&rxc_exclude);
    }
+   if(script && script_needs_closing)
+      fclose(script);
 }
 
-int MirrorJob::SetInclude(const char *s)
+const char *MirrorJob::SetRX(const char *s,char **rx,regex_t *rxc)
 {
-   if(rx_include)
+   if(*rx)
    {
-      free(rx_include);
-      rx_include=0;
-      regfree(&rxc_include);
+      *rx=(char*)xrealloc(*rx,strlen(*rx)+1+strlen(s)+1);
+      strcat(*rx,"|");
+      strcat(*rx,s);
+      regfree(rxc);
+      memset(rxc,0,sizeof(*rxc));   // for safety
    }
-   int res=regcomp(&rxc_include,s,REG_NOSUB|REG_EXTENDED);
+   else
+   {
+      *rx=xstrdup(s);
+   }
+   int res=regcomp(rxc,*rx,REG_NOSUB|REG_EXTENDED);
    if(res!=0)
    {
-      char err[1024];
-      regerror(res,&rxc_include,err,sizeof(err));
-      eprintf("mirror: --include - %s\n",err);
-      return -1;
+      xfree(*rx);
+      *rx=0;
+
+      static char err[256];
+      regerror(res,rxc,err,sizeof(err));
+
+      return err;
    }
-   rx_include=xstrdup(s);
-   return 0;
-}
-int MirrorJob::SetExclude(const char *s)
-{
-   if(rx_exclude)
-   {
-      free(rx_exclude);
-      rx_exclude=0;
-      regfree(&rxc_exclude);
-   }
-   int res=regcomp(&rxc_exclude,s,REG_NOSUB|REG_EXTENDED);
-   if(res!=0)
-   {
-      char err[1024];
-      regerror(res,&rxc_exclude,err,sizeof(err));
-      eprintf("mirror: --exclude - %s\n",err);
-      return -1;
-   }
-   rx_exclude=xstrdup(s);
    return 0;
 }
 
 void MirrorJob::va_Report(const char *fmt,va_list v)
 {
    if(parent_mirror)
-      parent_mirror->va_Report(fmt,v);
-
-   if(verbose_report && fg)
    {
+      parent_mirror->va_Report(fmt,v);
+      return;
+   }
+
+   if(verbose_report)
+   {
+      pid_t p=tcgetpgrp(fileno(stdout));
+      if(p!=-1 && p!=getpgrp())
+	 return;
+
       vfprintf(stdout,fmt,v);
       printf("\n");
       fflush(stdout);
@@ -740,3 +876,227 @@ void MirrorJob::SetNewerThan(const char *f)
    }
    newer_than=st.st_mtime;
 }
+
+
+CMD(mirror)
+{
+#define args (parent->args)
+   static struct option mirror_opts[]=
+   {
+      {"delete",no_argument,0,'e'},
+      {"allow-suid",no_argument,0,'s'},
+      {"include",required_argument,0,'i'},
+      {"exclude",required_argument,0,'x'},
+      {"time-prec",required_argument,0,'t'},
+      {"only-newer",no_argument,0,'n'},
+      {"no-recursion",no_argument,0,'r'},
+      {"no-perms",no_argument,0,'p'},
+      {"no-umask",no_argument,0,256+'u'},
+      {"continue",no_argument,0,'c'},
+      {"reverse",no_argument,0,'R'},
+      {"verbose",optional_argument,0,'v'},
+      {"newer-than",required_argument,0,'N'},
+      {"dereference",no_argument,0,'L'},
+      {0}
+   };
+
+   char cwd[2*1024];
+   const char *rcwd;
+   int opt;
+   int	 flags=0;
+
+   static char *include=0;
+   static int include_alloc=0;
+   static char *exclude=0;
+   static int exclude_alloc=0;
+#define APPEND_STRING(s,a,s1) \
+   {			                  \
+      int len,len1=strlen(s1);            \
+      if(!s)		                  \
+      {			                  \
+	 s=(char*)xmalloc(a = len1+1);    \
+      	 strcpy(s,s1);	                  \
+      }			                  \
+      else				  \
+      {					  \
+	 len=strlen(s);		       	  \
+	 if(a < len+1+len1+1)		  \
+	    s=(char*)xrealloc(s, a = len+1+len1+1); \
+	 if(s[0]) strcat(s,"|");	  \
+	 strcat(s,s1);			  \
+      }					  \
+   } /* END OF APPEND_STRING */
+
+   if(include)
+      include[0]=0;
+   if(exclude)
+      exclude[0]=0;
+
+   TimeInterval prec("12h");
+   bool	 create_remote_dir=false;
+   int	 verbose=0;
+   const char *newer_than=0;
+
+   args->rewind();
+   while((opt=args->getopt_long("esi:x:t:nrpcRvN:L",mirror_opts,0))!=EOF)
+   {
+      switch(opt)
+      {
+      case('e'):
+	 flags|=MirrorJob::DELETE;
+	 break;
+      case('s'):
+	 flags|=MirrorJob::ALLOW_SUID;
+	 break;
+      case('r'):
+	 flags|=MirrorJob::NO_RECURSION;
+	 break;
+      case('n'):
+	 flags|=MirrorJob::ONLY_NEWER;
+	 break;
+      case('p'):
+	 flags|=MirrorJob::NO_PERMS;
+	 break;
+      case('c'):
+	 flags|=MirrorJob::CONTINUE;
+	 break;
+      case('t'):
+	 prec=TimeInterval(optarg);
+	 if(prec.Error())
+	 {
+	    parent->eprintf("%s: %s: %s\n",args->a0(),optarg,prec.ErrorText());
+	    parent->eprintf(_("Try `help %s' for more information.\n"),args->a0());
+	    return 0;
+	 }
+	 break;
+      case('x'):
+	 APPEND_STRING(exclude,exclude_alloc,optarg);
+	 break;
+      case('i'):
+	 APPEND_STRING(include,include_alloc,optarg);
+	 break;
+      case('R'):
+	 flags|=MirrorJob::REVERSE|MirrorJob::ONLY_NEWER;
+	 break;
+      case('L'):
+	 flags|=MirrorJob::RETR_SYMLINKS;
+	 break;
+      case('v'):
+	 if(optarg)
+	    verbose=atoi(optarg);
+	 else
+	    verbose++;
+	 if(verbose>1)
+	    flags|=MirrorJob::REPORT_NOT_DELETED;
+	 break;
+      case('N'):
+	 newer_than=optarg;
+	 break;
+      case(256+'u'):
+	 flags|=MirrorJob::NO_UMASK;
+	 break;
+      case('?'):
+	 return 0;
+      }
+   }
+
+   if(getcwd(cwd,sizeof(cwd)/2)==0)
+   {
+      perror("getcwd()");
+      return 0;
+   }
+
+   args->back();
+   if(flags&MirrorJob::REVERSE)
+   {
+      char *arg=args->getnext();
+      if(!arg)
+	 rcwd=".";
+      else
+      {
+	 if(arg[0]=='/')
+	    strcpy(cwd,arg);
+	 else
+	 {
+	    strcat(cwd,"/");
+	    strcat(cwd,arg);
+	 }
+	 rcwd=args->getnext();
+	 if(!rcwd)
+	 {
+	    rcwd=basename_ptr(cwd);
+	    if(rcwd[0]=='/')
+	       rcwd=".";
+	    else
+	       create_remote_dir=true;
+	 }
+	 else
+	    create_remote_dir=true;
+      }
+   }
+   else	/* !REVERSE (normal) */
+   {
+      rcwd=args->getnext();
+      if(!rcwd)
+	 rcwd=".";
+      else
+      {
+	 strcat(cwd,"/");
+	 char *arg=args->getnext();
+	 if(arg)
+	 {
+	    if(arg[0]=='/')
+	       strcpy(cwd,arg);
+	    else
+	       strcat(cwd,arg);
+	    if(create_directories(arg)==-1)
+	       return 0;
+	 }
+	 else
+	 {
+	    int len=strlen(cwd);
+	    const char *base=basename_ptr(rcwd);
+	    if(base[0]!='/')
+	    {
+	       strcat(cwd,base);
+	       if(create_directories(cwd+len)==-1)
+		  return 0;
+	    }
+	 }
+      }
+   }
+   MirrorJob *j=new MirrorJob(parent->session->Clone(),cwd,rcwd);
+   j->SetFlags(flags,1);
+   j->SetVerbose(verbose);
+   if(create_remote_dir)
+      j->CreateRemoteDir();
+
+   const char *err;
+   const char *err_tag;
+
+   err_tag="include";
+   if(include && include[0] && (err=j->SetInclude(include)))
+      goto err_out;
+   err_tag="exclude";
+   if(exclude && exclude[0] && (err=j->SetExclude(exclude)))
+      goto err_out;
+
+   j->SetPrec(prec);
+   if(newer_than)
+      j->SetNewerThan(newer_than);
+   return j;
+
+err_out:
+   parent->eprintf("%s: %s: %s\n",args->a0(),err_tag,err);
+   delete j;
+   return 0;
+#undef args
+}
+
+
+#ifdef MODULE
+CDECL void module_init()
+{
+   CmdExec::RegisterCommand("mirror",cmd_mirror);
+}
+#endif
