@@ -18,10 +18,11 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: MirrorJob.h,v 1.9 1999/09/20 09:29:00 lav Exp $ */
+
 #ifndef MIRRORJOB_H
 #define MIRRORJOB_H
 
-#include "ListInfo.h"
 #include "GetJob.h"
 
 
@@ -56,6 +57,7 @@ class MirrorJob : public SessionJob
       WAITING_FOR_RM_BEFORE_PUT,
       WAITING_FOR_MKDIR_BEFORE_SUBMIRROR,
       REMOTE_REMOVE_OLD,
+      REMOTE_CHMOD,
       DONE
    };
    state_t state;
@@ -65,12 +67,15 @@ class MirrorJob : public SessionJob
    FileSet *to_transfer;
    FileSet *same;
    FileSet *to_rm;
+   FileSet *old_files_set;
+   FileSet *new_files_set;
    void	 InitSets(FileSet *src,FileSet *dst);
 
    FileInfo *file;
    void	 HandleFile(int);
 
    ListInfo *list_info;
+   FileAccess *local_session;
 
    char	 *local_dir;
    char	 *local_relative_dir;
@@ -86,7 +91,7 @@ class MirrorJob : public SessionJob
    char  *rx_include,*rx_exclude;
    regex_t rxc_include,rxc_exclude;
 
-   time_t   prec;
+   TimeInterval prec;
 
    bool	 dir_made;
    bool	 create_remote_dir;
@@ -98,6 +103,12 @@ class MirrorJob : public SessionJob
 
    time_t newer_than;
 
+   const char *SetRX(const char *s,char **rx,regex_t *rxc);
+
+   FILE *script;
+   bool script_only;
+   bool script_needs_closing;
+
 public:
    enum
    {
@@ -107,7 +118,10 @@ public:
       ONLY_NEWER=8,
       NO_PERMS=16,
       CONTINUE=32,
-      REVERSE=64
+      REVERSE=64,
+      REPORT_NOT_DELETED=128,
+      RETR_SYMLINKS=256,
+      NO_UMASK=512
    };
 
    void SetFlags(int f,int v)
@@ -127,17 +141,21 @@ public:
    void	 PrintStatus(int v);
    void	 SayFinal() { PrintStatus(-1); }
 
-   int	 SetInclude(const char *s);
-   int	 SetExclude(const char *s);
+   const char *SetInclude(const char *s)
+      {
+	 return SetRX(s,&rx_include,&rxc_include);
+      }
+   const char *SetExclude(const char *s)
+      {
+	 return SetRX(s,&rx_exclude,&rxc_exclude);
+      }
 
    void	 SetVerbose(int v) { verbose_report=v; }
 
-   void	 SetPrec(time_t p) { prec=p; }
+   void	 SetPrec(const TimeInterval &p) { prec=p; }
    void	 CreateRemoteDir() { create_remote_dir=true; }
 
    void	 SetNewerThan(const char *file);
 };
 
-FileSet *local_files(char *dir);
-
 #endif//MIRRORJOB_H
