@@ -20,6 +20,7 @@
 
 #include <config.h>
 #include <stdio.h>
+#include <assert.h>
 #include "PollVec.h"
 #include "xalloca.h"
 
@@ -66,6 +67,7 @@ PollVec::PollVec(Waiting::Type t)
 
 PollVec::PollVec(int fd,int ev)
 {
+   assert(fd>=0);
    chain=new Waiting;
    chain->wait_type=Waiting::POLLFD;
    chain->next=0;
@@ -127,8 +129,24 @@ void  PollVec::Block() const
    int i;
 
    for(i=0,scan=chain; scan; scan=scan->next)
+   {
       if(scan->wait_type==Waiting::POLLFD)
-	 pfd[i++]=scan->pfd;
+      {
+	 int j;
+	 for(j=0; j<i; j++)
+	 {
+	    if(pfd[j].fd==scan->pfd.fd)
+	    {
+	       // merge two pollfd's (workaround for some systems)
+	       pfd[j].events|=scan->pfd.events;
+	       nfd--;
+	       break;
+	    }
+	 }
+	 if(j>=i)
+	    pfd[i++]=scan->pfd;
+      }
+   }
 
    poll(pfd,nfd,cur_timeout);
    return;
