@@ -18,6 +18,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: ProcWait.h,v 1.2 1998/09/05 18:22:22 lav Exp $ */
+
 #ifndef PROCWAIT_H
 #define PROCWAIT_H
 
@@ -35,13 +37,18 @@ public:
       ERROR
    };
 
-private:
+protected:
+   static ProcWait *chain;
+   ProcWait *next;
+
    pid_t pid;
    State status;
    int	 term_info;
    int	 saved_errno;
    bool  auto_die;
 
+   bool  handle_info(int info); // true if finished
+
 public:
    int	 Do();
    int	 GetState() { return status; }
@@ -52,6 +59,11 @@ public:
    void Auto() { auto_die=true; }
 
    ProcWait(pid_t p);
+   ~ProcWait();
+
+   static void SIGCHLD_handler(int);
+
+   static void Signal(bool yes);
 };
 
 #endif /* PROCWAIT_H */
