@@ -18,6 +18,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: PutJob.cc,v 1.8 1999/09/14 11:51:06 lav Exp $ */
+
 #include <config.h>
 #include "PutJob.h"
 #include <assert.h>
@@ -27,8 +29,13 @@
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <errno.h>
 #include "misc.h"
 
+#ifndef S_ISREG
+# define S_ISREG(mode) (S_IFREG==(mode&S_IFMT))
+#endif
+
 int   PutJob::Do()
 {
    RateDrain();
@@ -43,6 +50,8 @@ int   PutJob::Do()
    if(Done())
       return m;
 
+   session->Resume(); // later we can suspend it again if needed.
+
    // now deal with next todo
    if(remote_size==-1)
    {
@@ -55,7 +64,7 @@ int   PutJob::Do()
 	 session->GetInfoArray(&info,1);
       }
       int res=session->Done();
-      if(res==Ftp::IN_PROGRESS)
+      if(res==FA::IN_PROGRESS)
       	 return m;
       m=MOVED;
       if(res<0)
@@ -70,13 +79,17 @@ int   PutJob::Do()
    {
       if(size!=-1)
       {
-	 if(remote_size>=size)	  // assume done
+	 if(size>=0 && remote_size>=size) // assume done
 	 {
 	    NextFile();
 	    return MOVED;
 	 }
 	 m=MOVED;
-	 session->Open(curr,Ftp::STORE,remote_size<0?0:remote_size);
+	 session->Open(curr,FA::STORE,remote_size<0?0:remote_size);
+	 if(size>=0)
+	    session->SetSize(size);
+	 if(entity_date!=(time_t)-1)
+	    session->SetDate(entity_date);
       }
    }
    // in store mode position can jump back
@@ -84,6 +97,7 @@ int   PutJob::Do()
    {
       got_eof=false;
       offset+=in_buffer;
+      CountBytes(in_buffer);
       in_buffer=0;
    }
 
@@ -91,32 +105,41 @@ int   PutJob::Do()
    {
       res=session->StoreStatus();
 remote_error:
-      if(res==Ftp::DO_AGAIN)
+      if(res==FA::DO_AGAIN)
 	 return m;
-      if(res==Ftp::IN_PROGRESS)
+      if(res==FA::IN_PROGRESS)
 	 return m;
       m=MOVED;
-      if(res==Ftp::STORE_FAILED)
+      if(res==FA::STORE_FAILED)
       {
 	 offset+=in_buffer;
+	 CountBytes(in_buffer);
 	 in_buffer=0;	// flush buffer
 	 remote_size=-1;
 	 session->Close();
 	 return m;
       }
-      if(res!=Ftp::OK)
+      if(res!=FA::OK)
       {
 	 fprintf(stderr,"%s: %s\n",op,session->StrError(res));
 	 failed++;
 	 NextFile();
 	 return MOVED;
       }
+      if(delete_files)
+	 remove(local->name); // name is absolute path
       NextFile();
       m=MOVED;
       return m;
    }
    if(!got_eof)
    {
+      if(buffer==0)
+      {
+	 // allocate buffer
+	 buffer=(char*)xmalloc(buffer_size=0x4000);
+      }
+
       if(in_buffer==buffer_size)
 	 goto try_write;
 
@@ -140,8 +163,11 @@ remote_error:
 
 	 offset+=diff;
 
-	 if(diff<0)
-	    CountBytes(diff);
+	 // don't subtract too much.
+	 if(offset<remote_size && offset-diff>=remote_size)
+	    diff+=remote_size-offset;
+
+	 CountBytes(diff);
 
 	 res=lseek(fd,offset,SEEK_SET);
 	 if(res==-1)
@@ -153,36 +179,40 @@ remote_error:
 	 }
       }
 
-      if(size==-1 || size<offset)
+      if(size==-1 || (size>=0 && size<offset))
       {
 	 struct stat st;
 	 res=fstat(fd,&st);
-	 if(res==-1)
+	 if(res==-1 || (st.st_size<=0 && !S_ISREG(st.st_mode)))
 	    size=-2;
 	 else
+	 {
 	    size=st.st_size;
+	    session->SetSize(size);
+	    session->SetDate(entity_date=st.st_mtime);
+	 }
       }
 
-      block+=PollVec(fd,POLLIN);
-      struct pollfd pfd={fd,POLLIN};
-      int res=poll(&pfd,1,0);
-      if(res==1 && pfd.revents&(POLLIN|POLLNVAL))
+      res=read(fd,buffer+in_buffer,buffer_size-in_buffer);
+      if(res==-1)
       {
-	 res=read(fd,buffer+in_buffer,buffer_size-in_buffer);
-	 if(res==-1)
+	 if(errno==EAGAIN || errno==EINTR)
 	 {
-	    perror(local->name);
-	    failed++;
-	    NextFile();
-	    return MOVED;
+	    block+=PollVec(fd,POLLIN);
+	    goto try_write;
 	 }
-	 if(res==0)
-	 {
-	    // EOF
-	    got_eof=true;
-	 }
-	 in_buffer+=res;
+	 perror(local->name);
+	 failed++;
+	 NextFile();
+	 return MOVED;
       }
+      if(res==0)
+      {
+	 // EOF
+	 got_eof=true;
+      }
+      in_buffer+=res;
+      m=MOVED;
    }
 
 try_write:
@@ -190,10 +220,10 @@ try_write:
    {
       if(!got_eof)
 	 session->Suspend();
-      return MOVED;
+      return m;
    }
    res=session->Write(buffer,in_buffer);
-   if(res==Ftp::DO_AGAIN)
+   if(res==FA::DO_AGAIN)
       return m;
    m=MOVED;
    if(res<0)
@@ -231,4 +261,7 @@ void PutJob::NextFile()
       remote_size=-1;
    else
       remote_size=-2;	// -2 means no file
+
+   size=-1;
+   entity_date=(time_t)-1;
 }
