@@ -18,10 +18,14 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: PutJob.h,v 1.3 1999/05/31 13:02:42 lav Exp $ */
+
 #ifndef PUTJOB_H
 #define PUTJOB_H
 
 #include "FileXfer.h"
+#include <sys/types.h>
+#include <time.h>
 
 class PutJob : public FileXfer
 {
@@ -32,6 +36,10 @@ protected:
 
    long	 remote_size;
 
+   bool delete_files;
+
+   time_t   entity_date;
+
 public:
    int	 Do();
 
@@ -39,10 +47,8 @@ public:
       : FileXfer(s,args,cont)
    {
       remote_size=0;
-#if 0
-      if(!curr && args && args->count()>1)
-	 NextFile();
-#endif
+      delete_files=false;
+      entity_date=(time_t)-1;
    }
 };
 
