@@ -18,9 +18,12 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: ResMgr.cc,v 1.14 1999/10/12 22:44:09 lav Exp $ */
+
 #include <config.h>
 
 #include <fnmatch.h>
+#include <ctype.h>
 #include "ResMgr.h"
 #include "SMTask.h"
 #include "xmalloc.h"
@@ -141,14 +144,14 @@ const char *ResMgr::Set(const char *name,const char *cclosure,const char *cvalue
 	 free(to_free);
 	 free(closure);
       }
-      SMTask::ReconfigAll();
+      SMTask::ReconfigAll(type->name);
    }
    else
    {
       if(value)
       {
 	 chain=new Resource(chain,type,closure,value);
-	 SMTask::ReconfigAll();
+	 SMTask::ReconfigAll(type->name);
       }
       else
 	 free(closure);
@@ -156,10 +159,8 @@ const char *ResMgr::Set(const char *name,const char *cclosure,const char *cvalue
    return 0;
 }
 
-int ResMgr::ResourceCompare(const void *a,const void *b)
+int ResMgr::ResourceCompare(const Resource *ar,const Resource *br)
 {
-   const Resource *ar=*(const Resource*const*)a;
-   const Resource *br=*(const Resource*const*)b;
    int diff=strcmp(ar->type->name,br->type->name);
    if(diff)
       return diff;
@@ -172,59 +173,122 @@ int ResMgr::ResourceCompare(const void *a,const void *b)
    return strcmp(ar->closure,br->closure);
 }
 
-void ResMgr::Print(FILE *f)
+int ResMgr::VResourceCompare(const void *a,const void *b)
 {
+   const ResMgr::Resource *ar=*(const ResMgr::Resource*const*)a;
+   const ResMgr::Resource *br=*(const ResMgr::Resource*const*)b;
+   return ResMgr::ResourceCompare(ar,br);
+}
+
+char *ResMgr::Format(bool with_defaults,bool only_defaults)
+{
+   char *res;
+
    Resource *scan;
+   ResDecl  *dscan;
 
    int n=0;
-   for(scan=chain; scan; scan=scan->next)
-      n++;
-
-   Resource **arr=(Resource**)alloca(n*sizeof(Resource*));
-   n=0;
-   for(scan=chain; scan; scan=scan->next)
-      arr[n++]=scan;
-
-   qsort(arr,n,sizeof(*arr),ResourceCompare);
-
-   for(int i=0; i<n; i++)
+   int dn=0;
+   int size=0;
+   if(!only_defaults)
    {
-      fprintf(f,"set %s",arr[i]->type->name);
+      for(scan=chain; scan; scan=scan->next)
+      {
+	 size+=4+strlen(scan->type->name);
+	 if(scan->closure)
+	    size+=1+2*strlen(scan->closure);
+	 size+=1+1+2*strlen(scan->value)+1+1;
+	 n++;
+      }
+   }
+   if(with_defaults || only_defaults)
+   {
+      for(dscan=type_chain; dscan; dscan=dscan->next)
+      {
+	 size+=4+strlen(dscan->name);
+	 size+=1+1+2*xstrlen(dscan->defvalue)+1+1;
+	 dn++;
+      }
+   }
+
+   res=(char*)xmalloc(size+1);
+   char *store=res;
+
+   Resource **created=(Resource**)alloca((dn+1)*sizeof(Resource*));
+   Resource **c_store=created;
+   dn=0;
+   if(with_defaults || only_defaults)
+   {
+      for(dscan=type_chain; dscan; dscan=dscan->next)
+      {
+	 if(only_defaults || SimpleQuery(dscan->name,0)==0)
+	 {
+	    dn++;
+	    *c_store++=new Resource(0,dscan,
+	       0,xstrdup(dscan->defvalue?dscan->defvalue:"(nil)"));
+	 }
+      }
+   }
+
+   Resource **arr=(Resource**)alloca((n+dn)*sizeof(Resource*));
+   n=0;
+   if(!only_defaults)
+   {
+      for(scan=chain; scan; scan=scan->next)
+	 arr[n++]=scan;
+   }
+   int i;
+   if(with_defaults || only_defaults)
+   {
+      for(i=0; i<dn; i++)
+	 arr[n++]=created[i];
+   }
+
+   qsort(arr,n,sizeof(*arr),&ResMgr::VResourceCompare);
+
+   for(i=0; i<n; i++)
+   {
+      sprintf(store,"set %s",arr[i]->type->name);
+      store+=strlen(store);
       char *s=arr[i]->closure;
       if(s)
       {
-	 fputc('/',f);
+	 *store++='/';
 	 while(*s)
 	 {
 	    if(strchr("\" \t\\>|",*s))
-	       fputc('\\',f);
-	    fputc(*s++,f);
+	       *store++='\\';
+	    *store++=*s++;
 	 }
       }
-      fputc(' ',f);
+      *store++=' ';
       s=arr[i]->value;
 
       bool par=false;
       if(*s==0 || strcspn(s," \t>|")!=strlen(s))
 	 par=true;
       if(par)
-	 putc('"',f);
+	 *store++='"';
       while(*s)
       {
 	 if(strchr("\"\\",*s))
-	    putc('\\',f);
-	 putc(*s++,f);
+	    *store++='\\';
+	 *store++=*s++;
       }
       if(par)
-	 putc('"',f);
-      fputc('\n',f);
+	 *store++='"';
+      *store++='\n';
    }
+   *store=0;
+   for(i=0; i<dn; i++)
+      delete created[i];
+   return res;
 }
 
 const char *ResMgr::BoolValidate(char **value)
 {
    char *v=*value;
-   char *newval=0;
+   const char *newval=0;
 
    switch(v[0])
    {
@@ -298,7 +362,7 @@ bool ResMgr::Resource::ClosureMatch(const char *cl_data)
    return 0==fnmatch(closure,cl_data,FNM_PATHNAME);
 }
 
-const char *ResMgr::Query(const char *name,const char *closure)
+const char *ResMgr::SimpleQuery(const char *name,const char *closure)
 {
    const char *msg;
 
@@ -309,7 +373,7 @@ const char *ResMgr::Query(const char *name,const char *closure)
       return 0;
 
    Resource **scan;
-   // find the old value
+   // find the value
    for(scan=&chain; *scan; scan=&(*scan)->next)
       if((*scan)->type==type && (*scan)->ClosureMatch(closure))
 	 break;
@@ -321,7 +385,127 @@ const char *ResMgr::Query(const char *name,const char *closure)
    return 0;
 }
 
+ResValue ResMgr::Query(const char *name,const char *closure)
+{
+   const char *msg;
+
+   ResDecl *type;
+   // find type of given variable
+   msg=FindVar(name,&type);
+   if(msg)
+   {
+      // debug only
+      fprintf(stderr,"Query of variable `%s' failed: %s\n",name,msg);
+      return 0;
+   }
+
+   for(;;)
+   {
+      Resource **scan;
+      // find the value
+      for(scan=&chain; *scan; scan=&(*scan)->next)
+	 if((*scan)->type==type && (*scan)->ClosureMatch(closure))
+	    break;
+      // if found
+      if(*scan)
+	 return (*scan)->value;
+      if(!closure)
+	 break;
+      closure=0;
+   }
+
+   return type->defvalue;
+}
+
+ResValue ResDecl::Query(const char *closure)
+{
+   const char *v=0;
+
+   if(closure)
+      v=ResMgr::SimpleQuery(name,closure);
+   if(!v)
+      v=ResMgr::SimpleQuery(name,0);
+   if(!v)
+      v=defvalue;
+
+   return v;
+}
+
 bool ResMgr::str2bool(const char *s)
 {
    return(strchr("TtYy1+",s[0])!=0 || !strcasecmp(s,"on"));
 }
+
+ResDecl::ResDecl(const char *a_name,const char *a_defvalue,
+		  ResValValid *a_val_valid,ResClValid *a_closure_valid)
+{
+   name=a_name;
+   defvalue=a_defvalue;
+   val_valid=a_val_valid;
+   closure_valid=a_closure_valid;
+   next=ResMgr::type_chain;
+   ResMgr::type_chain=this;
+#if 0
+   if(defvalue)
+      ResMgr::Set(name,0,defvalue);
+#endif
+}
+
+#define MINUTE (60)
+#define HOUR   (60*MINUTE)
+#define DAY    (24*HOUR)
+
+TimeInterval::TimeInterval(const char *s)
+{
+   interval=0;
+   infty=false;
+   error_text=0;
+
+   if(!strncasecmp(s,"inf",3)
+   || !strcasecmp(s,"forever")
+   || !strcasecmp(s,"never"))
+   {
+      infty=true;
+      return;
+   }
+   int pos=0;
+   for(;;)
+   {
+      long prec;
+      char ch='s';
+      int pos1=strlen(s+pos);
+      int n=sscanf(s+pos,"%lu%c%n",&prec,&ch,&pos1);
+      if(n<1)
+	 break;
+      ch=tolower((unsigned char)ch);
+      if(ch=='m')
+	 prec*=MINUTE;
+      else if(ch=='h')
+	 prec*=HOUR;
+      else if(ch=='d')
+	 prec*=DAY;
+      else if(ch!='s')
+      {
+	 error_text=_("Invalid time unit letter, only [smhd] are allowed.");
+	 return;
+      }
+      interval+=prec;
+      pos+=pos1;
+   }
+   if(pos==0)
+   {
+      error_text=_("Invalid time format. Format is <time><unit>, e.g. 2h30m.");
+      return;
+   }
+}
+TimeInterval::~TimeInterval()
+{
+}
+
+const char *ResMgr::TimeIntervalValidate(char **s)
+{
+   TimeInterval t(*s);
+   if(t.Error())
+      return t.ErrorText();
+   return 0;
+}
