@@ -18,19 +18,24 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: ResMgr.h,v 1.12 1999/09/29 13:52:22 lav Exp $ */
+
 #ifndef RESMGR_H
 #define RESMGR_H
 
 #include <stdio.h>
+#include <sys/types.h>
+#include <time.h>
 #include "xmalloc.h"
 
 typedef const char *ResValValid(char **value);
 typedef const char *ResClValid(char **closure);
 
+class ResValue;
+
 class ResMgr
 {
    friend class ResDecl;
-
    class Resource
    {
       friend class ResMgr;
@@ -61,10 +66,9 @@ class ResMgr
    static Resource *chain;
    static ResDecl *type_chain;
 
-   static int ResourceCompare(const void *a,const void *b);
-
 public:
-   static const char *Query(const char *name,const char *closure);
+   static const char *SimpleQuery(const char *name,const char *closure);
+   static ResValue Query(const char *name,const char *closure);
 
    enum CmpRes { EXACT,SUBSTR,DIFFERENT };
 
@@ -73,11 +77,18 @@ public:
    static const char *Set(const char *name,const char *closure,const char *value);
 
    static void Print(FILE *);
+   static char *Format(bool with_defaults,bool only_defaults);
 
    static const char *BoolValidate(char **value);
    static const char *NumberValidate(char **value);
    static const char *UNumberValidate(char **value);
+   static const char *TimeIntervalValidate(char **value);
    static bool str2bool(const char *value);
+
+   static void ClassInit();
+
+   static int ResourceCompare(const Resource *a,const Resource *b);
+   static int VResourceCompare(const void *a,const void *b);
 };
 
 class ResDecl
@@ -92,17 +103,53 @@ public:
    const char *defvalue;
 
    ResDecl(const char *name,const char *defvalue,
-	   ResValValid *val_valid,ResClValid *closure_valid)
-   {
-      this->name=name;
-      this->defvalue=defvalue;
-      this->val_valid=val_valid;
-      this->closure_valid=closure_valid;
-      this->next=ResMgr::type_chain;
-      ResMgr::type_chain=this;
-      if(defvalue)
-	 ResMgr::Set(name,0,defvalue);
-   }
+	   ResValValid *val_valid,ResClValid *closure_valid=0);
+
+   ResValue Query(const char *closure);
+};
+
+class ResValue
+{
+   const char *s;
+public:
+   ResValue(const char *s_new)
+      {
+	 s=s_new;
+      }
+   operator bool()
+      {
+	 return ResMgr::str2bool(s);
+      }
+   operator int()
+      {
+	 return atoi(s);
+      }
+   operator const char*()
+      {
+	 return s;
+      }
+   bool is_nil() { return s==0; }
+};
+
+class TimeInterval
+{
+   time_t interval;
+   bool infty;
+   const char *error_text;
+public:
+   TimeInterval(const char *);
+   TimeInterval(time_t i)
+      {
+	 interval=i;
+	 infty=false;
+	 error_text=0;
+      }
+   ~TimeInterval();
+   bool Error() { return error_text!=0; };
+   const char *ErrorText() { return error_text; }
+
+   bool IsInfty() { return infty; }
+   time_t Seconds() { return interval; }
 };
 
 #endif //RESMGR_H
