@@ -18,13 +18,23 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: SMTask.cc,v 1.8 1999/08/26 18:19:37 lav Exp $ */
+
 #include <config.h>
 #include <assert.h>
+#include <sys/types.h>
+#include <sys/time.h>
+#ifdef TIME_WITH_SYS_TIME
+# include <time.h>
+#endif
+
 #include "SMTask.h"
 
 SMTask	 *SMTask::chain=0;
 SMTask	 *SMTask::sched_scan=0;
 PollVec	 SMTask::sched_total;
+time_t	 SMTask::now=time(0);
+int	 SMTask::now_ms; // milliseconds
 
 SMTask::SMTask()
 {
@@ -44,8 +54,8 @@ SMTask::SMTask()
    running=false;
 }
 
-void  SMTask::Suspend() { suspended=1; }
-void  SMTask::Resume()  { suspended=0; }
+void  SMTask::Suspend() { suspended=true; }
+void  SMTask::Resume()  { suspended=false; }
 
 SMTask::~SMTask()
 {
@@ -64,13 +74,28 @@ SMTask::~SMTask()
    }
 }
 
+void SMTask::UpdateNow()
+{
+#ifdef HAVE_GETTIMEOFDAY
+   struct timeval tv;
+   gettimeofday(&tv,0);
+   now=tv.tv_sec;
+   now_ms=tv.tv_usec/1000;
+#else
+   time(&now);
+#endif
+}
+
 void SMTask::Schedule()
 {
+   // get time onec and assume Do() don't take much time
+   UpdateNow();
+
    bool repeat=false;
    sched_scan=chain;
    while(sched_scan)
    {
-      if(sched_scan->running)
+      if(sched_scan->running || sched_scan->suspended)
       {
 	 sched_scan=sched_scan->next;
 	 continue;
@@ -94,14 +119,23 @@ void SMTask::Schedule()
       return;
    }
 
+   // now collect all wake up conditions; excluding suspended and
+   // already running tasks, because they can't run again on this
+   // level of recursion, and thus could cause spinning by their wake up
+   // conditions.
    for(sched_scan=chain; sched_scan; sched_scan=sched_scan->next)
-      if(sched_scan->suspended==0)
+   {
+      if(!sched_scan->suspended
+      && !sched_scan->running)
 	 sched_total.Merge(sched_scan->block);
+   }
    return;
 }
 
-void SMTask::ReconfigAll()
+void SMTask::ReconfigAll(const char *name)
 {
+   UpdateNow();
    for(SMTask *scan=chain; scan; scan=scan->next)
-      scan->Reconfig();
+      scan->Reconfig(name);
+   sched_total+=NoWait();  // for new values handling
 }
