@@ -18,6 +18,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: StatusLine.cc,v 1.5 1999/08/04 17:01:18 lav Exp $ */
+
 #include <config.h>
 #include <sys/types.h>
 #ifdef HAVE_TERMIOS_H
@@ -27,7 +29,8 @@
 #include <sys/ioctl.h>
 #include <stdio.h>
 #include <stdarg.h>
-#include <string.h>
+#include "xstring.h"
+#include "xmalloc.h"
 
 #include "StatusLine.h"
 
@@ -41,7 +44,7 @@ int  StatusLine::GetWidth()
       sz.ws_col=80;
    if(sz.ws_row==0)
       sz.ws_row=24;
-   return(sz.ws_col);
+   return(LastWidth=sz.ws_col);
 #else /* !TIOCGWINSZ */
    return 80;
 #endif
@@ -50,17 +53,33 @@ int  StatusLine::GetWidth()
 StatusLine::StatusLine(int new_fd)
 {
    fd=new_fd;
+   update_delayed=false;
+   update_time=0;
    strcpy(shown,"");
    not_term=!isatty(fd);
+   LastWidth=GetWidth();
+}
+StatusLine::~StatusLine()
+{
+}
+
+void StatusLine::Clear()
+{
+   char newstr[sizeof(shown)];
+
+   newstr[0]=0;
+   update(newstr);
+   update_delayed=false;
+   update_time=0;
 }
 
 void StatusLine::Show(const char *f,...)
 {
-   if(not_term)
-      return;
-
-   if(tcgetpgrp(fd)!=getpgrp())
+   if(f==0 || f[0]==0)
+   {
+      Clear();
       return;
+   }
 
    char newstr[sizeof(shown)];
 
@@ -69,6 +88,28 @@ void StatusLine::Show(const char *f,...)
    vsprintf(newstr,f,v);
    va_end(v);
 
+   if(now>update_time)
+   {
+      update(newstr);
+      update_delayed=false;
+      update_time=now;
+   }
+   else if(strcmp(to_be_shown,newstr))
+   {
+      strcpy(to_be_shown,newstr);
+      update_delayed=true;
+      SMTask::block+=TimeOut(1000);
+   }
+}
+
+void StatusLine::update(char *newstr)
+{
+   if(not_term)
+      return;
+
+   if(tcgetpgrp(fd)!=getpgrp())
+      return;
+
    char *end=newstr+strlen(newstr);
 
    int w=GetWidth();
@@ -104,17 +145,18 @@ void StatusLine::Show(const char *f,...)
 
 void StatusLine::WriteLine(const char *f,...)
 {
-   char newstr[sizeof(shown)+strlen(f)+64];
+   char *newstr=(char*)alloca(sizeof(shown)+strlen(f)+64);
 
    va_list v;
    va_start(v,f);
    vsprintf(newstr,f,v);
    va_end(v);
 
-   if(not_term)
+   if(not_term || shown[0]==0)
    {
       strcat(newstr,"\n");
       write(fd,newstr,strlen(newstr));
+      update_delayed=false;
       return;
    }
 
@@ -141,4 +183,20 @@ void StatusLine::WriteLine(const char *f,...)
    write(fd,newstr,strlen(newstr));
 
    strcpy(shown,"");
+   update_delayed=false;
+}
+
+int StatusLine::Do()
+{
+   if(!update_delayed)
+      return STALL;
+   if(now>update_time)
+   {
+      update(to_be_shown);
+      update_delayed=false;
+      update_time=now;
+      return STALL;
+   }
+   Timeout(1000);
+   return STALL;
 }
