@@ -18,6 +18,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: StatusLine.h,v 1.4 1999/08/04 17:01:18 lav Exp $ */
+
 #ifndef STATUSLINE_H
 #define STATUSLINE_H
 
@@ -27,20 +29,32 @@
 # define PRINTF_LIKE(n,m)
 #endif
 
-class StatusLine
+#include "SMTask.h"
+
+class StatusLine : public SMTask
 {
    int fd;
    char shown[0x800];
    bool	not_term;
+   time_t update_time;
+   char to_be_shown[0x800];
+   bool update_delayed;
+   void update(char *);
+   int LastWidth;
 
 public:
    int GetWidth();
+   int GetWidthDelayed() { return LastWidth; }
    void Show(const char *f,...) PRINTF_LIKE(2,3);
    void WriteLine(const char *f,...) PRINTF_LIKE(2,3);
+   void Clear();
 
    int getfd() { return fd; }
 
    StatusLine(int new_fd);
+   ~StatusLine();
+
+   int Do();
 };
 
 #endif // STATUSLINE_H
