@@ -18,6 +18,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: SysCmdJob.cc,v 1.3 1999/05/20 10:50:40 lav Exp $ */
+
 #include <config.h>
 
 #include <sys/types.h>
@@ -38,6 +40,9 @@ SysCmdJob::SysCmdJob(const char *c)
 
 SysCmdJob::~SysCmdJob()
 {
+   Bg();
+   AcceptSig(SIGTERM);
+
    if(w)
    {
       w->Auto();
@@ -47,13 +52,17 @@ SysCmdJob::~SysCmdJob()
 
 int SysCmdJob::Do()
 {
-   if(w)
-      return STALL;
+   int m=STALL;
 
-   char *shell=getenv("SHELL");
+   if(w)
+      return m;
+
+   const char *shell=getenv("SHELL");
    if(!shell)
       shell="/bin/sh";
 
+   ProcWait::Signal(false);
+
    pid_t pid;
    fflush(stderr);
    switch(pid=fork())
@@ -70,13 +79,30 @@ int SysCmdJob::Do()
       fflush(stderr);
       _exit(1);
    case(-1): /* error */
-      block+=TimeOut(1000);   // wait a second
-      return STALL;
+      block+=TimeOut(1000);   // wait a second and retry
+      goto out;
    }
    /* parent */
    int info;
    waitpid(pid,&info,WUNTRACED); // wait until the process stops
    w=new ProcWait(pid);
    fg_data=new FgData(pid,fg);
+   m=MOVED;
+out:
+   ProcWait::Signal(true);
+   return m;
+}
+
+int SysCmdJob::AcceptSig(int sig)
+{
+   if(!w)
+   {
+      if(sig==SIGINT)
+	 return WANTDIE;
+      return STALL;
+   }
+   w->Kill(sig);
+   if(sig!=SIGCONT)
+      AcceptSig(SIGCONT);  // for the case of stopped process
    return MOVED;
 }
