@@ -18,74 +18,171 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: XferJob.cc,v 1.26 1999/09/18 15:05:42 lav Exp $ */
+
 #include <config.h>
 #include <stdio.h>
 #include <errno.h>
 #include <unistd.h>
 #include <assert.h>
 #include <ctype.h>
-#include <netdb.h>
 
 #include "xmalloc.h"
 
 #include "XferJob.h"
-#include "ProtoList.h"
-#include "rglob.h"
+#include "plural.h"
+#include "ResMgr.h"
+
+static ResDecl
+   res_use_urls	  ("xfer:use-urls",    "no", ResMgr::BoolValidate,0),
+   res_eta_terse  ("xfer:eta-terse",   "yes",ResMgr::BoolValidate,0);
 
 char *XferJob::Percent()
 {
    static char p[10];
    p[0]=0;
    if(size>0)
-      sprintf(p,"(%d%%) ",percent(offset,size));
+      sprintf(p,"(%d%%) ",percent(Offset(),size));
    return p;
 }
 
+bool XferJob::CanShowRate(float rate)
+{
+   return(now-start_time>3 && now-last_bytes<30 && rate>1);
+}
+
 char *XferJob::CurrRate(float rate)
 {
    static char speed[40];
    speed[0]=0;
-   if(time(0)-start_time>3 && time(0)-last_bytes<30 && rate!=0)
+   rate_shown=false;
+   if(CanShowRate(rate))
    {
+      rate_shown=true;
       if(rate<1024)
-	 sprintf(speed,"%.0fb/s ",rate);
+	 sprintf(speed,_("%.0fb/s "),rate);
       else if(rate<1024*1024)
-	 sprintf(speed,"%.1fK/s ",rate/1024.);
+	 sprintf(speed,_("%.1fK/s "),rate/1024.);
       else
-	 sprintf(speed,"%.2fM/s ",rate/1024./1024.);
+	 sprintf(speed,_("%.2fM/s "),rate/1024./1024.);
    }
    return speed;
 }
 
+#define MINUTE (60)
+#define HOUR   (60*MINUTE)
+#define DAY    (24*HOUR)
+
+char *XferJob::CurrETA(float rate,long offs)
+{
+   static char eta_str[20];
+   eta_str[0]=0;
+   if(size>0 && size>=offs && CanShowRate(rate))
+   {
+      long eta=(long)((size-offs) / rate + 0.5);
+      long eta2=0;
+      long ueta=0;
+      long ueta2=0;
+      char letter=0;
+      char letter2=0;
+
+      // for translator: only first letter matters
+      const char day_c=_("day")[0];
+      const char hour_c=_("hour")[0];
+      const char minute_c=_("minute")[0];
+      const char second_c=_("second")[0];
+
+      const char *tr_eta=_("eta:");
+
+      if((bool)res_eta_terse.Query(0))
+      {
+	 if(eta>=DAY)
+	 {
+	    ueta=(eta+DAY/2)/DAY;
+	    eta2=eta-ueta*DAY;
+	    letter=day_c;
+	    if(ueta<10)
+	    {
+	       letter2=hour_c;
+	       ueta2=((eta2<0?eta2+DAY:eta2)+HOUR/2)/HOUR;
+	       if(ueta2>0 && eta2<0)
+		  ueta--;
+	    }
+	 }
+	 else if(eta>=HOUR)
+	 {
+	    ueta=(eta+HOUR/2)/HOUR;
+	    eta2=eta-ueta*HOUR;
+	    letter=hour_c;
+	    if(ueta<10)
+	    {
+	       letter2=minute_c;
+	       ueta2=((eta2<0?eta2+HOUR:eta2)+MINUTE/2)/MINUTE;
+	       if(ueta2>0 && eta2<0)
+		  ueta--;
+	    }
+	 }
+	 else if(eta>=MINUTE)
+	 {
+	    ueta=(eta+MINUTE/2)/MINUTE;
+	    letter=minute_c;
+	 }
+	 else
+	 {
+	    ueta=eta;
+	    letter=second_c;
+	 }
+	 if(letter2 && ueta2>0)
+	    sprintf(eta_str,"%s%ld%c%ld%c ",tr_eta,ueta,letter,ueta2,letter2);
+	 else
+	    sprintf(eta_str,"%s%ld%c ",tr_eta,ueta,letter);
+      }
+      else // verbose eta (by Ben Winslow)
+      {
+	 long unit;
+	 strcpy(eta_str, tr_eta);
+
+	 if(eta>=DAY)
+	 {
+	    unit=eta/DAY;
+	    sprintf(eta_str+strlen(eta_str), "%ld%c", unit, day_c);
+	 }
+	 if(eta>=HOUR)
+	 {
+	    unit=(eta/HOUR)%24;
+	    sprintf(eta_str+strlen(eta_str), "%ld%c", unit, hour_c);
+	 }
+	 if(eta>=MINUTE)
+	 {
+	    unit=(eta/MINUTE)%60;
+	    sprintf(eta_str+strlen(eta_str), "%ld%c", unit, minute_c);
+	 }
+	 unit=eta%60;
+	 sprintf(eta_str+strlen(eta_str), "%ld%c ", unit, second_c);
+      }
+   }
+   return eta_str;
+}
+
 void  XferJob::ShowRunStatus(StatusLine *s)
 {
    if(!print_run_status)
       return;
    if(Done())
-   {
-      s->Show("");
       return;
-   }
 
-   time_t now=time(0);
    const char *st=session->CurrentStatus();
 
-   if(now==last_status_update && !strncmp(st,last_status,sizeof(last_status)))
-      return;
-
-   strncpy(last_status,st,sizeof(last_status));
-   last_status_update=now;
-
    if(curr && session->IsOpen())
    {
-      int w=s->GetWidth()-30;
+      int w=s->GetWidthDelayed()-40;
       if(w<=0)
 	 return;
-      char *n=curr;
-      if((int)strlen(n)>w)
-	 n=n+strlen(n)-w;
+      const char *n=curr;
+      if((int)strlen(n)>w-2)
+	 n+=strlen(n)-(w-2);
 
-      s->Show(_("`%s' at %lu %s%s[%s]"),n,offset,Percent(),CurrRate(),st);
+      s->Show(_("`%s' at %lu %s%s%s[%s]"),n,Offset(),Percent(),CurrRate(),CurrETA(),st);
    }
 }
 
@@ -97,18 +194,31 @@ void  XferJob::SayFinal()
       return;
    if(bytes_transferred)
    {
-      if(end_time>start_time+1)
+      if(end_time>start_time)
       {
-	 printf(_("%ld bytes transferred in %ld seconds (%g bytes/s)\n"),
-	    bytes_transferred,(long)(end_time-start_time),xfer_rate());
+	 long sec=end_time-start_time;
+	 printf(plural("%ld $#l#byte|bytes$ transferred"
+			" in %ld $#l#second|seconds$ (%g bytes/s)\n",
+			bytes_transferred,sec),
+	    bytes_transferred,sec,xfer_rate());
       }
       else
-	 printf(_("%ld bytes transferred\n"),bytes_transferred);
+      {
+	 printf(plural("%ld $#l#byte|bytes$ transferred\n",
+			bytes_transferred),
+	    bytes_transferred);
+      }
    }
    if(failed>0)
-      printf(_("Transfer of %d of %d files failed\n"),failed,file_count);
+   {
+      printf(plural("Transfer of %d of %d $file|files$ failed\n",file_count),
+	 failed,file_count);
+   }
    else if(file_count>1)
-      printf(_("%d files total\n"),file_count);
+   {
+      printf(plural("Total %d $file|files$ transferred\n",file_count),
+	 file_count);
+   }
 }
 
 void  XferJob::PrintStatus(int verbose)
@@ -127,13 +237,14 @@ void  XferJob::PrintStatus(int verbose)
       if(failed>0)
       {
 	 putchar('\t');
-	 printf(_("Transfer of %d of %d files failed\n"),failed,file_count);
+	 printf(plural("Transfer of %d of %d $file|files$ failed\n",file_count),
+	    failed,file_count);
       }
       else if(file_count>1)
       {
       	 putchar('\t');
-	 printf(_("%d files tranferred"),file_count);
-	 putchar('\n');
+	 printf(plural("Total %d $file|files$ transferred\n",file_count),
+	    file_count);
       }
       if(end_time>start_time)
       {
@@ -146,8 +257,8 @@ void  XferJob::PrintStatus(int verbose)
    if(curr && session->IsOpen())
    {
       putchar('\t');
-      printf(_("`%s' at %lu %s%s[%s]"),curr,offset,Percent(),CurrRate(),
-	       session->CurrentStatus());
+      printf(_("`%s' at %lu %s%s%s[%s]"),curr,Offset(),Percent(),CurrRate(),
+	       CurrETA(),session->CurrentStatus());
       putchar('\n');
    }
 }
@@ -159,16 +270,19 @@ void XferJob::NextFile(char *f)
    if(curr)
       file_count++;
 
-   session->Close();
+   if(session)
+      session->Close();
    offset=0;
    size=-1;
    got_eof=false;
    in_buffer=0;
    curr=f;
+   session_buffered=0;
 
    if(!curr)
    {
-      time(&end_time);
+      end_time=now;
+      end_time_ms=now_ms;
    }
    else
    {
@@ -177,9 +291,11 @@ void XferJob::NextFile(char *f)
 	 if(url)
 	    delete url;
 	 url=new ParsedURL(curr);
-	 if(url->proto && url->path)
+	 if((url->proto || non_strict_urls) && url->path)
 	 {
-	    FileAccess *new_session=Protocol::NewSession(url->proto);
+	    FileAccess *new_session=session;
+	    if(url->proto)
+	       new_session=FileAccess::New(url->proto);
 	    if(!new_session)
 	    {
 	       eprintf(_("%s: %s - not supported protocol\n"),
@@ -189,27 +305,13 @@ void XferJob::NextFile(char *f)
 	    if(url->user && url->pass)
 	       new_session->Login(url->user,url->pass);
 	    if(url->host)
-	    {
-	       int port=0;
-	       if(url->port)
-	       {
-		  if(isdigit(url->port[0]))
-		     port=atoi(url->port);
-		  else
-		  {
-		     struct servent *serv=getservbyname(url->port,"tcp");
-		     if(serv)
-			port=serv->s_port;
-		     else
-			eprintf(_("%s: %s - no such tcp service, using default\n"),
-				 op,url->port);
-		  }
-	       }
-	       new_session->Connect(url->host,port);
-	    }
+	       new_session->Connect(url->host,url->port);
 	    curr=url->path;
-	    SessionPool::Reuse(session);
-	    session=new_session;
+	    if(new_session!=session)
+	    {
+	       SessionPool::Reuse(session);
+	       session=new_session;
+	    }
 	 }
       }
    }
@@ -218,11 +320,16 @@ void XferJob::NextFile(char *f)
 XferJob::XferJob(FileAccess *f) : SessionJob(f)
 {
    in_buffer=0;
+   buffer_size=0;
+   buffer=0;
    file_count=0;
    failed=0;
    offset=0;
-   time(&start_time);
-   end_time=start_time;
+   UpdateNow();
+   start_time=now;
+   start_time_ms=now_ms;
+   end_time=now;
+   end_time_ms=now_ms;
    last_second=start_time;
    last_bytes=0;
    bytes_transferred=0;
@@ -234,24 +341,36 @@ XferJob::XferJob(FileAccess *f) : SessionJob(f)
    line_buf=false;
    op="";
    size=-1;
-   use_urls=false;
+   use_urls=res_use_urls.Query(0);  // Query once, can't change on fly
+   non_strict_urls=false;
    url=0;
-   last_status_update=0;
-   last_status[0]=0;
+   session_buffered=0;
+   rate_shown=false;
 }
 
 XferJob::~XferJob()
 {
    if(url)
       delete url;
+   xfree(buffer);
 }
 
 int XferJob::TryRead(FileAccess *s)
 {
    if(in_buffer==buffer_size)
    {
-      s->Suspend();
-      return s->DO_AGAIN;
+      if(buffer_size<0x10000)
+      {
+	 if(buffer==0)
+	    buffer=(char*)xmalloc(buffer_size=0x1000);
+	 else
+	    buffer=(char*)xrealloc(buffer,buffer_size*=2);
+      }
+      else
+      {
+	 s->Suspend();
+	 return s->DO_AGAIN;
+      }
    }
    int res=s->Read(buffer+in_buffer,buffer_size-in_buffer);
    if(res==s->DO_AGAIN)
@@ -289,40 +408,55 @@ void XferJob::CountBytes(long res)
       return;
    }
 
-   time_t t=time(0);
+   if(session)
+   {
+      int now_buffered=session->Buffered();
+      res-=(now_buffered-session_buffered);
+      if(res<0)
+	 res=0;
+      session_buffered=now_buffered;
+   }
 
    float div=60;
-   if(t-start_time<div)
-      div=t-start_time+1;
-   if(t-last_second>div)
-      div=t-last_second;
+   if(now-start_time<div)
+      div=now-start_time+1;
+   if(now-last_second>div)
+      div=now-last_second;
 
-   minute_xfer_rate*=1.-(t-last_second)/div;
+   minute_xfer_rate*=1.-(now-last_second)/div;
    minute_xfer_rate+=res/div;
-   last_second=t;
+   last_second=now;
 
    if(res>0)
-      last_bytes=t;
+      last_bytes=now;
 }
 
 void  XferJob::RateDrain()
 {
-   if(session->IsOpen())
+   if(!session || session->IsOpen())
    {
       CountBytes(0);
-      block+=TimeOut(4000);
+      if(rate_shown)
+	 block+=TimeOut(1000);
    }
    else
    {
-      last_second=time(0);
+      last_second=now;
    }
 }
 
 int XferJob::TryWrite(FDStream *f)
 {
+   if(f->broken())
+   {
+      failed++;
+      return -1;
+   }
+
    if(in_buffer==0)
    {
-      session->Resume();
+      if(session)
+	 session->Resume();
       return 0;
    }
 
@@ -361,35 +495,33 @@ int XferJob::TryWrite(FDStream *f)
 	 to_write=nl-buffer+1;
    }
 
-   struct pollfd pfd={fd,POLLOUT};
-   int res=poll(&pfd,1,0);
-   if(res==1 && pfd.revents&(POLLOUT|POLLNVAL))
+   // fd should be non-blocking.
+   int res=write(fd,buffer,to_write);
+   if(res==-1)
    {
-      res=write(fd,buffer,to_write);
-      if(res==-1)
+      if(errno==EAGAIN || errno==EINTR)
       {
+	 res=0;
+	 goto normal_return;
+      }
+      if(errno!=EPIPE)
 	 perror(f->name);
-	 failed++;
-	 return -1;
-      }
-      if(res==0)
-      {
-	 fprintf(stderr,_("%s: cannot write -- disk full?\n"),op);
-	 failed++;
-	 return -1;
-      }
-      in_buffer-=res;
-      memmove(buffer,buffer+res,in_buffer);
-   }
-   else if(res==1)   // what's up?
-   {
-//       fprintf(stderr,"Broken pipe\n");
       failed++;
       return -1;
    }
+   if(res==0)
+   {
+      fprintf(stderr,_("%s: cannot write -- disk full?\n"),op);
+      failed++;
+      return -1;
+   }
+   in_buffer-=res;
+   memmove(buffer,buffer+res,in_buffer);
+
+normal_return:
    if(in_buffer>0)
       block+=PollVec(fd,POLLOUT);
-   if(in_buffer<buffer_size)
+   if(in_buffer<buffer_size && session)
       session->Resume();
    return res;
 }
@@ -401,3 +533,21 @@ int   XferJob::Done()
    NextFile();
    return curr==0;
 }
+
+float XferJob::xfer_rate()
+{
+   float how_long = (end_time-start_time) + (end_time_ms-start_time_ms)/1000.0;
+   return bytes_transferred/how_long;
+}
+
+long XferJob::Offset()
+{
+   int buf=0;
+   long off=offset;
+   if(session)
+      buf=session->Buffered();
+   off-=buf;
+   if(off<0)
+      off=0;
+   return off;
+}
