@@ -18,6 +18,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: XferJob.h,v 1.10 1999/08/06 18:33:47 lav Exp $ */
+
 #ifndef XFERJOB_H
 #define XFERJOB_H
 
@@ -33,28 +35,34 @@
 class XferJob : public SessionJob
 {
 protected:
-   char	 *op;
+   const char *op;
 
-   char	 *curr;
+   const char *curr;
 
-   static const int buffer_size=0x1000;
-   char	 buffer[buffer_size];
-   int	 in_buffer;
+   size_t buffer_size;
+   char  *buffer;
+   size_t in_buffer;
 
    int	 file_count;
    int	 failed;
 
    time_t start_time;
    time_t end_time;
+   int	 start_time_ms;
+   int	 end_time_ms;
 
    long	 offset;
    long	 size;
 
+   long	 Offset();   // uses session->Buffered()
+
    float minute_xfer_rate;
    time_t last_second;
    time_t last_bytes;
 
-   float xfer_rate() { return (float)bytes_transferred/(end_time-start_time); }
+   int	 session_buffered;
+
+   float xfer_rate();
 
    bool	 got_eof;
    bool	 print_run_status;
@@ -71,17 +79,20 @@ protected:
    int	 TryWrite(FDStream *f);
 
    bool	 use_urls;
+   bool  non_strict_urls;
    ParsedURL *url;
 
    void  CountBytes(long);
    void	 RateDrain();
 
-   time_t   last_status_update;
-   char	    last_status[80];
-
    char *Percent(); // return either string ending with a space or empty string
    char *CurrRate(float);
    char *CurrRate() { return CurrRate(minute_xfer_rate); }
+   char *CurrETA(float rate,long offs);
+   char *CurrETA() { return CurrETA(minute_xfer_rate,Offset()); }
+
+   bool CanShowRate(float rate);
+   bool rate_shown;
 
 public:
    long	 bytes_transferred;
@@ -93,7 +104,15 @@ public:
    void	 PrintStatus(int);
    void	 SayFinal();
 
-   void UseURLs() { use_urls=true; }
+   void UseURLs()
+      {
+#if 0
+	 if(use_urls)
+	    non_strict_urls=true;
+	 else
+#endif
+	    use_urls=true;
+      }
 };
 
 #endif /* XFERJOB_H */
