@@ -18,6 +18,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: bookmark.cc,v 1.3 1999/05/20 10:06:22 lav Exp $ */
+
 #include <config.h>
 #include "bookmark.h"
 #include <sys/stat.h>
@@ -29,7 +31,7 @@
 
 Bookmark::Bookmark()
 {
-   char *home=getenv("HOME");
+   const char *home=getenv("HOME");
    if(home==0)
       home=".";
    int home_len=strlen(home);
@@ -131,9 +133,18 @@ const char *Bookmark::Lookup(const char *key)
    return super::Lookup(key);
 }
 
+#if 0
 void Bookmark::List()
 {
    Refresh();
    Close();
    super::Write(dup(0));
 }
+#endif
+
+char *Bookmark::Format()
+{
+   Refresh();
+   Close();
+   return super::Format();
+}
