@@ -18,6 +18,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: complete.cc,v 1.23 1999/10/13 17:44:44 lav Exp $ */
+
 #include <config.h>
 
 #include <stdio.h>
@@ -25,25 +27,28 @@
 #include <time.h>
 #include "xalloca.h"
 #include "xmalloc.h"
-#include "ftpclass.h"
-#include "lftp.h"
-#include "rglob.h"
+#include "FileAccess.h"
 #include "CmdExec.h"
 #include "alias.h"
-#include "GetPass.h"
 #include "SignalHook.h"
 #include "CharReader.h"
+#include "LsCache.h"
+#include "complete.h"
+#include "lftp_rl.h"
 
-extern "C" {
+CDECL_BEGIN
 #include "readline/readline.h"
-}
+CDECL_END
 
 static char *bash_dequote_filename (char *text, int quote_char);
 static int lftp_char_is_quoted(char *string,int eindex);
 
 static int len;    // lenght of the word to complete
 static int cindex; // index in completion array
-static char **glob_res=NULL;
+static const char *const*glob_res=NULL;
+
+static bool shell_cmd;
+static bool quote_glob;
 
 char *command_generator(char *text,int state)
 {
@@ -60,7 +65,7 @@ char *command_generator(char *text,int state)
    }
 
    /* Return the next name which partially matches from the command list. */
-   while ((name=CmdExec::cmd_table[cindex].name)!=0)
+   while ((name=CmdExec::CmdByIndex(cindex)->name)!=0)
    {
       cindex++;
       if(strncmp(name,text,len)==0)
@@ -79,9 +84,9 @@ char *command_generator(char *text,int state)
    return(NULL);
 }
 
-char *remote_generator(char *text,int state)
+static char *remote_generator(char *text,int state)
 {
-   char *name;
+   const char *name;
 
    /* If this is a new word to complete, initialize now.  This includes
       saving the length of TEXT for efficiency, and initializing the cindex
@@ -94,20 +99,62 @@ char *remote_generator(char *text,int state)
 
    while((name=glob_res[cindex++])!=NULL)
    {
-      if(!strchr(name,'/') && memchr(text,'/',len))
-      {
-	 // workaround for servers returning only names without dir
-	 char *slash=text+len;
-	 while(slash[-1]!='/')
-	    slash--;
-	 if(strncmp(name,slash,len-(slash-text)))
-	    continue;
+      if(!name[0])
+	 continue;
+      if(strncmp(name,text,len)==0)
+	 return(xstrdup(name));
+   }
 
-	 char *combined=(char*)xmalloc(strlen(name)+len+1);
-	 strncpy(combined,text,slash-text);
-	 strcpy(combined+(slash-text),name);
-	 return combined;
+   glob_res=NULL;
+   return NULL;
+}
+
+static char *bookmark_generator(char *text,int s)
+{
+   static int state;
+   const char *t;
+   if(!s)
+   {
+      state=0;
+      lftp_bookmarks.Rewind();
+   }
+   for(;;)
+   {
+      switch(state)
+      {
+      case 0:
+	 t=lftp_bookmarks.CurrentKey();
+	 if(!t)
+	 {
+	    state=1;
+	    break;
+	 }
+	 if(!lftp_bookmarks.Next())
+	    state=1;
+	 if(strncmp(t,text,len)==0)
+	    return xstrdup(t);
+	 break;
+      case 1:
+	 return 0;
       }
+   }
+}
+
+static char *array_generator(char *text,int state)
+{
+   const char *name;
+
+   /* If this is a new word to complete, initialize now. */
+   if(!state)
+      cindex=0;
+
+   if(glob_res==NULL)
+      return NULL;
+
+   while((name=glob_res[cindex++])!=NULL)
+   {
+      if(!name[0])
+	 continue;
       if(strncmp(name,text,len)==0)
 	 return(xstrdup(name));
    }
@@ -136,23 +183,31 @@ static bool copy_word(char *buf,const char *p,int n)
 }
 
 
-// returns:
-//    2 - remote dir
-//    1 - remote file
-//    0 - local
-int   remote_cmd(int start)
+enum completion_type
 {
+   LOCAL, REMOTE_FILE, REMOTE_DIR, BOOKMARK, COMMAND,
+   STRING_ARRAY, NO_COMPLETION
+};
+
+static completion_type cmd_completion_type(int start)
+{
+   const char *cmd=rl_line_buffer;
+
+   if(find_word(cmd)-cmd == start) // first word is command
+      return COMMAND;
+
    // try to guess whether the completion word is remote
 
    char buf[20];  // no commands longer
-   const char *cmd=rl_line_buffer;
    TouchedAlias *used_aliases=0;
 
    for(;;)
    {
       const char *w=find_word(cmd);
+      if(w[0]=='!')
+	 shell_cmd=true;
       if(w[0]=='!' || w[0]=='#')
-	 return 0;
+	 return LOCAL;
       if(w[0]=='(')
       {
 	 cmd=w+1;
@@ -162,7 +217,7 @@ int   remote_cmd(int start)
       || buf[0]==0)
       {
 	 TouchedAlias::FreeChain(used_aliases);
-	 return 0;
+	 return LOCAL;
       }
       const char *alias=Alias::Find(buf);
       if(alias && !TouchedAlias::IsTouched(alias,used_aliases))
@@ -171,30 +226,55 @@ int   remote_cmd(int start)
 	 cmd=alias;
 	 continue;
       }
-      const CmdExec::cmd_rec *c;
-      int part=CmdExec::find_cmd(buf,&c);
-      if(part==1)
-	 strcpy(buf,c->name);
+      const char *full=CmdExec::GetFullCommandName(buf);
+      if(full!=buf)
+	 strcpy(buf,full);
       TouchedAlias::FreeChain(used_aliases);
       break;
    }
 
+   for(char *p=rl_line_buffer+start; p>rl_line_buffer; )
+   {
+      p--;
+      if((*p=='>' || *p=='|')
+      && !lftp_char_is_quoted(rl_line_buffer,p-rl_line_buffer))
+	 return LOCAL;
+      if(!isspace((unsigned char)*p))
+	 break;
+   }
+
+   if(!strcmp(buf,"shell"))
+      shell_cmd=true;
+   if(!strcmp(buf,"glob")
+   || !strcmp(buf,"mget")
+   || !strcmp(buf,"mput")
+   || !strcmp(buf,"mrm"))
+      quote_glob=true;
+
    if(!strcmp(buf,"cd")
    || !strcmp(buf,"mkdir"))
-      return 2;	/* append slash automatically */
+      return REMOTE_DIR; /* append slash automatically */
 
-   if(!strcmp(buf,"ls")
+   if(!strcmp(buf,"cat")
+   || !strcmp(buf,"ls")
    || !strcmp(buf,"mget")
-   || !strcmp(buf,"rm")
-   || !strcmp(buf,"mrm")
-   || !strcmp(buf,"rmdir")
    || !strcmp(buf,"more")
-   || !strcmp(buf,"cat")
+   || !strcmp(buf,"mrm")
+   || !strcmp(buf,"mv")
+   || !strcmp(buf,"nlist")
+   || !strcmp(buf,"rm")
+   || !strcmp(buf,"rmdir")
    || !strcmp(buf,"zcat")
    || !strcmp(buf,"zmore"))
-      return 1;
+      return REMOTE_FILE;
+
+   if(!strcmp(buf,"open")
+   || !strcmp(buf,"lftp"))
+      return BOOKMARK;
 
    bool was_o=false;
+   bool was_N=false;
+   bool second=false;
    for(int i=start; i>4; i--)
    {
       if(!isspace(rl_line_buffer[i-1]))
@@ -204,20 +284,101 @@ int   remote_cmd(int start)
 	 was_o=true;
 	 break;
       }
+      if(!strncmp(rl_line_buffer+i-3,"-N",2) && isspace(rl_line_buffer[i-4]))
+      {
+	 was_N=true;
+	 break;
+      }
+   }
+   const char *w=find_word(cmd);
+   while(*w && !isspace(*w))
+      w++;
+   if(*w)
+   {
+      w=find_word(w);
+      if(w-cmd==start)	// we complete second word
+	 second=true;
    }
 
    if(!strcmp(buf,"get")
    || !strcmp(buf,"pget"))
       if(!was_o)
-	 return 1;
+	 return REMOTE_FILE;
    if(!strcmp(buf,"put"))
       if(was_o)
-	 return 1;
+	 return REMOTE_FILE;
+   if(!strcmp(buf,"mirror"))
+      if(!was_N)
+	 return REMOTE_FILE;
+   if(!strcmp(buf,"bookmark"))
+   {
+      if(second)
+      {
+	 glob_res=bookmark_subcmd;
+	 return STRING_ARRAY;
+      }
+      else
+	 return BOOKMARK;
+   }
+   if(!strcmp(buf,"chmod"))
+   {
+      if(second)
+	 return NO_COMPLETION;
+      else
+	 return REMOTE_FILE;
+   }
+   if(!strcmp(buf,"glob"))
+   {
+      if(second)
+	 return COMMAND;
+      else
+	 return REMOTE_FILE;  // FIXME: it would be better to check argv[1].
+   }
+   if(!strcmp(buf,"cache"))
+   {
+      if(second)
+      {
+	 glob_res=cache_subcmd;
+	 return STRING_ARRAY;
+      }
+      else
+	 return NO_COMPLETION;
+   }
 
-   return 0;
+   return LOCAL;
+}
+
+static void glob_quote(char *out,const char *in,int len)
+{
+   while(len>0)
+   {
+      switch(*in)
+      {
+      case '*': case '?': case '[': case ']':
+	 if(!quote_glob)
+	    *out++='\\';
+	 break;
+      case '\\':
+	 switch(in[1])
+	 {
+	 case '*': case '?': case '[': case ']': case '\\':
+	    *out++=*in++;  // copy the backslash.
+	    break;
+	 default:
+	    in++; // skip it.
+	    break;
+	 }
+	 break;
+      }
+      *out++=*in;
+      in++;
+      len--;
+   }
+   *out=0;
 }
 
 CmdExec *completion_shell;
+int remote_completion=0;
 
 static bool force_remote=false;
 
@@ -225,76 +386,103 @@ static bool force_remote=false;
    region of TEXT that contains the word to complete.  We can use the
    entire line in case we want to do some simple parsing.  Return the
    array of matches, or NULL if there aren't any. */
-char **lftp_completion (char *text,int start,int end)
+static char **lftp_completion (char *text,int start,int end)
 {
+   if(start>end)  // workaround for a bug in readline
+      start=end;
+
+   Glob *rg=0;
+
    rl_completion_append_character=' ';
+   shell_cmd=false;
+   quote_glob=false;
 
-   if(start==0)
-   {
-      len=end;
-      return completion_matches(text,command_generator);
-   }
-   int remote_type=remote_cmd(start);
-   if((remote_completion && remote_type) || force_remote)
-   {
-      len=end-start;
-      char *pat=(char*)alloca(len+5);
-      strncpy(pat,text,len);
-      pat[len]=0;
-      if(strchr(pat,'*') || strchr(pat,'?'))
-	 return(NULL);
-      char *sl=pat+strlen(pat);
-      while(sl>pat && sl[-1]!='/')
-	 sl--;
-      while(sl>pat && sl[-1]=='/')
-	 sl--;
-      if(sl>pat)  // 'dir///file'
-	 sl[0]=0;
-      else if(pat[0]=='/')
-	 pat[1]=0;	// '///'
-      else
-	 pat[0]=0;	// no slashes
+   completion_type type=cmd_completion_type(start);
 
-//       strcat(pat,"/*")
+   len=end-start;
+
+   char *(*generator)(char *text,int state) = 0;
+
+   switch(type)
+   {
+   case NO_COMPLETION:
+      rl_attempted_completion_over = 1;
+      return 0;
+   case COMMAND:
+      generator = command_generator;
+      break;
+   case BOOKMARK:
+      generator = bookmark_generator;
+      break;
+   case STRING_ARRAY:
+      generator = array_generator;
+      break;
+   case LOCAL:
+      if(force_remote)
+	 goto really_remote;
+      break;
+   case REMOTE_FILE:
+   case REMOTE_DIR:
+      if(!remote_completion && !force_remote)
+	 break; // local
+   really_remote:
+      char *pat=(char*)alloca(len*2+10);
+      glob_quote(pat,text,len);
+
+      strcat(pat,"*");
 
       completion_shell->session->DontSleep();
 
       SignalHook::ResetCount(SIGINT);
-      RemoteGlob g(completion_shell->session,pat,Ftp::LIST);
-      g.SetSlashFilter(1);
-      for(;;)
+      glob_res=NULL;
+      rg=completion_shell->session->MakeGlob(pat);
+      if(rg)
       {
-	 SMTask::Schedule();
-	 if(g.Done())
-	    break;
-	 if(SignalHook::GetCount(SIGINT))
+	 for(;;)
 	 {
-	    rl_attempted_completion_over = 1;
-	    return 0;
+	    SMTask::Schedule();
+	    if(rg->Done())
+	       break;
+	    if(SignalHook::GetCount(SIGINT))
+	    {
+	       SignalHook::ResetCount(SIGINT);
+	       rl_attempted_completion_over = 1;
+	       delete rg;
+	       return 0;
+	    }
+	    SMTask::Block();
 	 }
-	 SMTask::Block();
+	 glob_res=rg->GetResult();
       }
-      glob_res=g.GetResult();
-
       rl_filename_completion_desired=1;
+      generator = remote_generator;
+      break;
 
-      char quoted=((lftp_char_is_quoted(rl_line_buffer,start) &&
-		    strchr(rl_completer_quote_characters,rl_line_buffer[start-1]))
-		   ? rl_line_buffer[start-1] : 0);
-      text=bash_dequote_filename(text, quoted);
-      len=strlen(text);
-      char **matches=completion_matches(text,remote_generator);
-      free(text);
-      if(!matches)
-      {
-	 rl_attempted_completion_over = 1;
-	 return 0;
-      }
-      if(remote_type==2 /* cd/mkdir */)
-	 rl_completion_append_character='/';
-      return matches;
+   } /* end switch */
+
+   if(generator==0)
+      return 0; // default to local
+
+   char quoted=((lftp_char_is_quoted(rl_line_buffer,start) &&
+		 strchr(rl_completer_quote_characters,rl_line_buffer[start-1]))
+		? rl_line_buffer[start-1] : 0);
+   text=bash_dequote_filename(text, quoted);
+   len=strlen(text);
+
+   char **matches=completion_matches(text,(CPFunction*)generator);
+
+   if(rg)
+      delete rg;
+
+   free(text);
+   if(!matches)
+   {
+      rl_attempted_completion_over = 1;
+      return 0;
    }
-   return(NULL);
+   if(type==REMOTE_DIR)
+      rl_completion_append_character='/';
+   return matches;
 }
 
 
@@ -309,7 +497,7 @@ enum { COMPLETE_DQUOTE,COMPLETE_SQUOTE,COMPLETE_BSQUOTE };
 
 /* Return a new string which is the single-quoted version of STRING.
    Used by alias and trap, among others. */
-char *
+static char *
 single_quote (char *string)
 {
   register int c;
@@ -338,7 +526,7 @@ single_quote (char *string)
 }
 
 /* Quote STRING using double quotes.  Return a new string. */
-char *
+static char *
 double_quote (char *string)
 {
   register int c;
@@ -352,12 +540,14 @@ double_quote (char *string)
     {
       switch (c)
         {
+	case '$':
+	case '`':
+	  if(!shell_cmd)
+	     goto def;
 	case '"':
-// 	case '$':
-// 	case '`':
 	case '\\':
 	  *r++ = '\\';
-	default:
+	default: def:
 	  *r++ = c;
 	  break;
         }
@@ -371,7 +561,7 @@ double_quote (char *string)
 
 /* Quote special characters in STRING using backslashes.  Return a new
    string. */
-char *
+static char *
 backslash_quote (char *string)
 {
   int c;
@@ -383,27 +573,31 @@ backslash_quote (char *string)
     {
       switch (c)
 	{
+ 	case '\'':
+ 	case '(': case ')':
+ 	case '!': case '{': case '}':		/* reserved words */
+ 	case '^':
+ 	case '$': case '`':			/* expansion chars */
+	  if(!shell_cmd)
+	    goto def;
+ 	case '*': case '[': case '?': case ']':	/* globbing chars */
+	  if(!shell_cmd && !quote_glob)
+	    goto def;
 	case ' ': case '\t': case '\n':		/* IFS white space */
-// 	case '\'':
 	case '"': case '\\':		/* quoting chars */
 	case '|': case '&': case ';':		/* shell metacharacters */
-// 	case '(': case ')':
 	case '<': case '>':
-// 	case '!': case '{': case '}':		/* reserved words */
-// 	case '*': case '[': case '?': case ']':	/* globbing chars */
-// 	case '^':
-// 	case '$': case '`':			/* expansion chars */
 	  *r++ = '\\';
 	  *r++ = c;
 	  break;
-	case '#':				/* comment char */
-#if 0
 	case '~':				/* tilde expansion */
-#endif
+	  if(!shell_cmd)
+	    goto def;
+	case '#':				/* comment char */
 	  if (s == string)
 	    *r++ = '\\';
 	  /* FALLTHROUGH */
-	default:
+	default: def:
 	  *r++ = c;
 	  break;
 	}
@@ -413,7 +607,8 @@ backslash_quote (char *string)
   return (result);
 }
 
-int
+#if 0 // no need yet
+static int
 contains_shell_metas (char *string)
 {
   char *s;
@@ -442,6 +637,7 @@ contains_shell_metas (char *string)
 
   return (0);
 }
+#endif //0
 
 /* Filename quoting for completion. */
 /* A function to strip quotes that are not protected by backquotes.  It
@@ -599,7 +795,7 @@ bash_quote_filename (char *s, int rtype, char *qcp)
   return ret;
 }
 
-int skip_double_quoted(char *s, int i)
+static int skip_double_quoted(char *s, int i)
 {
    while(s[i] && s[i]!='"')
    {
@@ -641,20 +837,18 @@ int lftp_char_is_quoted(char *string,int eindex)
   return (0);
 }
 
-extern "C" {
 int   lftp_complete_remote(int count,int key)
 {
    extern Function *rl_last_func;
 
    if(rl_last_func == (Function*)lftp_complete_remote)
-      rl_last_func = rl_complete;
+      rl_last_func = (Function*)rl_complete;
 
    force_remote = true;
    int ret=rl_complete(count,key);
    force_remote = false;
    return ret;
 }
-}
 
 int   lftp_rl_getc(FILE *file)
 {
@@ -669,6 +863,7 @@ int   lftp_rl_getc(FILE *file)
 	 return EOF;
       if(res!=r.NOCHAR)
 	 return res;
+      lftp_rl_redisplay_maybe();
       SMTask::Block();
       if(SignalHook::GetCount(SIGINT)>0)
       {
@@ -682,24 +877,24 @@ int   lftp_rl_getc(FILE *file)
 /* Tell the GNU Readline library how to complete.  We want to try to complete
    on command names if this is the first word in the line, or on filenames
    if not. */
-void initialize_readline ()
+void lftp_readline_init ()
 {
    /* Allow conditional parsing of the ~/.inputrc file. */
-   rl_readline_name = "lftp";
+   rl_readline_name = (char*)"lftp";
 
    /* Tell the completer that we want a crack first. */
    rl_attempted_completion_function = (CPPFunction *)lftp_completion;
 
-   rl_getc_function = (int (*)(...))lftp_rl_getc;
+   rl_getc_function = (Function*)lftp_rl_getc;
 
-   rl_completer_quote_characters = "\"";
-   rl_completer_word_break_characters = " \t\n\"";
-   rl_filename_quote_characters = " \t\n\\\">;|&()";
-   rl_filename_quoting_function = (char* (*)(...))bash_quote_filename;
-   rl_filename_dequoting_function = (char* (*)(...))bash_dequote_filename;
-   rl_char_is_quoted_p = (int (*)(...))lftp_char_is_quoted;
+   rl_completer_quote_characters = (char*)"\"";
+   rl_completer_word_break_characters = (char*)" \t\n\"";
+   rl_filename_quote_characters = (char*)" \t\n\\\">;|&()*?[]";
+   rl_filename_quoting_function = (CPFunction*)bash_quote_filename;
+   rl_filename_dequoting_function = (CPFunction*)bash_dequote_filename;
+   rl_char_is_quoted_p = (Function*)lftp_char_is_quoted;
 
-   rl_add_defun("complete-remote",(Function*)lftp_complete_remote,-1);
+   rl_add_defun((char*)"complete-remote",(Function*)lftp_complete_remote,-1);
    static char line[]="Meta-Tab: complete-remote";
-   rl_parse_and_bind(line);
+   rl_parse_and_bind(line); /* this function writes to the string */
 }
