@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-1997 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-1999 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,17 +18,28 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: ftpget.cc,v 1.11 1999/10/06 17:20:13 lav Exp $ */
+
 #include <config.h>
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
+#include "xstring.h"
 #include <unistd.h>
 #include <errno.h>
 #include "getopt.h"
 #include "GetJob.h"
 #include "LsJob.h"
 #include "GetPass.h"
+#include "log.h"
+#include "FileAccess.h"
+#include "Resolver.h"
+#include "SignalHook.h"
+#include "ResMgr.h"
+#include "modconfig.h"
+#ifndef MODULE_PROTO_FTP
+# include "ftpclass.h"
+#endif
 #ifdef HAVE_LOCALE_H
 # include <locale.h>
 #endif
@@ -43,7 +54,7 @@ void  PrintUsage(int p)
 {
    if(p)
    {
-      printf(_("FtpGet | Version %s | Copyright (C) 1996-97 Alexander V. Lukyanov\n"),VERSION);
+      printf(_("FtpGet | Version %s | Copyright (C) 1996-1999 Alexander V. Lukyanov\n"),VERSION);
       // xgettext:c-format
       printf(_("This is free software with ABSOLUTELY NO WARRANTY. See COPYING for details.\n"));
    }
@@ -57,7 +68,6 @@ void  PrintUsage(int p)
 	  "-v  --verbose      verbose (lots of output)\n"
 	  "    --async-mode   use asynchronous mode (faster)\n"
 	  "    --sync-mode    use synchronous mode (compatible with bugs)\n"
-	  "    --norest-mode  don't use REST command\n"
 	  "\n"
 	  "-o  output to local file `local' (default - base name of filename)\n")
       );
@@ -66,7 +76,11 @@ void  PrintUsage(int p)
 
 int   main(int argc,char **argv)
 {
-   int   port=21;
+#ifdef SOCKS4
+   SOCKSinit(argv[0]);
+#endif
+
+   char  *port=0;
    int   c;
    char  *user=NULL;
    char  *pass=NULL;
@@ -77,7 +91,6 @@ int   main(int argc,char **argv)
    int   verbose=0;
    int   cont=0;
    char  *host_name;
-   int	 flags=0;
 
    enum
    {
@@ -92,12 +105,11 @@ int   main(int argc,char **argv)
    {
       {"help",no_argument,0,HELP_OPT},
       {"version",no_argument,0,VERSION_OPT},
-      {"norest-mode",no_argument,0,NOREST_MODE_OPT},
       {"sync-mode",no_argument,0,SYNC_MODE_OPT},
       {"async-mode",no_argument,0,ASYNC_MODE_OPT},
       {"continue",no_argument,0,'c'},
       {"user",required_argument,0,'u'},
-      {"post",required_argument,0,'p'},
+      {"port",required_argument,0,'p'},
       {"list",no_argument,0,'l'},
       {"quiet",no_argument,0,'q'},
       {"verbose",no_argument,0,'v'},
@@ -108,20 +120,24 @@ int   main(int argc,char **argv)
    bindtextdomain (PACKAGE, LOCALEDIR);
    textdomain (PACKAGE);
 
+   ResMgr::ClassInit(); // resources must be inited before other classes
+   SignalHook::ClassInit();
+   Resolver::ClassInit();
+
+#ifndef MODULE_PROTO_FTP
+   Ftp::ClassInit();
+#endif
+
    program=argv[0];
 
-   Ftp *f=new Ftp;
+   FileAccess *f=FileAccess::New("ftp");
 
    while((c=getopt_long(argc,argv,"+p:u:lqvc",ftpget_options,0))!=-1)
    {
       switch(c)
       {
       case('p'):
-         if(sscanf(optarg,"%d",&port)!=1)
-         {
-            fprintf(stderr,"%s: invalid port number\n",program);
-	    exit(1);
-         }
+	 port=optarg;
          break;
       case('u'):
          user=optarg;
@@ -151,13 +167,10 @@ int   main(int argc,char **argv)
          cont=1;
          break;
       case(SYNC_MODE_OPT):
-	 f->SetFlag(f->SYNC_MODE,1);
+	 ResMgr::Set("ftp:sync-mode","yes",0);
 	 break;
       case(ASYNC_MODE_OPT):
-	 f->SetFlag(f->SYNC_MODE,0);
-	 break;
-      case(NOREST_MODE_OPT):
-	 flags|=Ftp::NOREST_MODE;
+	 ResMgr::Set("ftp:sync-mode","no",0);
 	 break;
       case(HELP_OPT):
 	 PrintUsage(0);
@@ -174,9 +187,13 @@ int   main(int argc,char **argv)
    if(optind+2>argc)
       PrintUsage(1);
 
+   Log::global=new Log();
    if(verbose)
-     f->SetDebug(stderr,5);
-   f->SetFlag(flags,1);
+   {
+      Log::global->SetOutput(2,false);
+      Log::global->SetLevel(5);
+      Log::global->Enable();
+   }
 
    f->Connect(host_name=argv[optind++],port);
    if(user)
@@ -187,9 +204,9 @@ int   main(int argc,char **argv)
 
    if(list)
    {
-      ArgV *rest=new ArgV(argc-optind,argv+optind);
-      j=new LsJob(f,0,rest->Combine());
-      delete rest;
+      ArgV *rest=new ArgV(argc-optind+1,argv+optind-1);
+      rest->setarg(0,"ls");
+      j=new LsJob(f,0,rest);
    }
    else
    {
