@@ -18,6 +18,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: history.cc,v 1.3 1999/05/22 12:37:51 lav Exp $ */
+
 #include <config.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -35,7 +37,7 @@ History::History()
    stamp=0;
    fd=-1;
 
-   char *home=getenv("HOME");
+   const char *home=getenv("HOME");
    if(home==0)
       home=".";
    int home_len=strlen(home);
@@ -68,7 +70,7 @@ time_t History::extract_stamp(const char *res)
 
 const char *History::Lookup(FileAccess *s)
 {
-   const char *url=s->GetConnectURL(s->NO_CWD);
+   const char *url=s->GetConnectURL(s->NO_PATH);
    if(!url)
       return 0;
    const char *res=super::Lookup(url);
@@ -131,7 +133,7 @@ void History::Set(FileAccess *s,const char *cwd)
       return;
    char *res=(char*)alloca(32+strlen(cwd)+1);
    sprintf(res,"%lu:%s",(unsigned long)time(0),cwd);
-   super::Add(s->GetConnectURL(s->NO_CWD),res);
+   super::Add(s->GetConnectURL(s->NO_PATH),res);
 }
 
 void History::Save()
