@@ -18,6 +18,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: history.h,v 1.1 1998/10/03 15:27:37 lav Exp $ */
+
 #ifndef HISTORY_H
 #define HISTORY_H
 
@@ -45,6 +47,12 @@ public:
    void Save();
    History();
    ~History();
+
+   void Rewind()
+      {
+	 Refresh();
+	 KeyValueDB::Rewind();
+      }
 };
 
 #endif //HISTORY_H
