@@ -17,18 +17,20 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
+
+/* $Id: lftp.cc,v 1.28 1999/10/05 15:23:37 lav Exp $ */
+
 #include <config.h>
 
+#include "modconfig.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
 #include <unistd.h>
-#include <string.h>
-#include <signal.h>
 #include <sys/types.h>
-#include <sys/stat.h>
-#include <sys/wait.h>
-#include <ctype.h>
+#include <sys/stat.h> // for mkdir()
+#include "xstring.h"
 #include <fcntl.h>
 #ifdef HAVE_LOCALE_H
 # include <locale.h>
@@ -36,29 +38,37 @@
 
 #include "xalloca.h"
 #include "ftpclass.h"
-#include "LocalAccess.h"
-#include "lftp.h"
+#ifndef MODULE_PROTO_FILE
+# include "LocalAccess.h"
+#endif
+#ifndef MODULE_PROTO_HTTP
+# include "Http.h"
+#endif
 #include "xmalloc.h"
 #include "alias.h"
 #include "CmdExec.h"
 #include "SignalHook.h"
 #include "GetPass.h"
 #include "history.h"
+#include "log.h"
+#include "DummyProto.h"
+#include "Resolver.h"
+#include "ResMgr.h"
 
 #include "confpaths.h"
 
-extern "C" {
-#include "readline/readline.h"
-#include "readline/history.h"
-}
+#include "lftp_rl.h"
+#include "complete.h"
+
+
+CmdExec	 *top_exec;
 
-int   remote_completion=0;
 
 void  hook_signals()
 {
    SignalHook::DoCount(SIGHUP);
-   SignalHook::DoCount(SIGPIPE);
    SignalHook::Ignore(SIGTTOU);
+   ProcWait::Signal(true);
 }
 
 void WaitDone(CmdExec *exec)
@@ -75,35 +85,57 @@ void WaitDone(CmdExec *exec)
 class ReadlineFeeder : public CmdFeeder
 {
    bool tty:1;
+   bool ctty:1;
    bool add_newline:1;
+   char *to_free;
 public:
    ReadlineFeeder()
    {
       tty=isatty(0);
+      ctty=(tcgetpgrp(0)!=(pid_t)-1);
+      to_free=0;
+   }
+   virtual ~ReadlineFeeder()
+   {
+      xfree(to_free);
    }
 
-   char *NextCmd(class CmdExec *exec,const char *prompt)
+   const char *NextCmd(class CmdExec *exec,const char *prompt)
    {
+      xfree(to_free);
+      to_free=0;
+
       if(add_newline)
       {
 	 add_newline=false;
-	 return xstrdup("\n");
+	 return "\n";
       }
 
       ::completion_shell=exec;
       ::remote_completion=exec->remote_completion;
 
-      char *cmd_buf;
+      const char *cmd_buf;
       if(tty)
       {
+	 if(ctty) // controlling terminal
+	 {
+	    pid_t term_pg=tcgetpgrp(0);
+	    if(term_pg!=(pid_t)-1 && getpgrp()!=term_pg)
+	    {
+	       // looks like we are in background. Can't read from tty
+	       exec->block+=TimeOut(500);
+	       return "";
+	    }
+	 }
+
 	 SignalHook::ResetCount(SIGINT);
-	 cmd_buf=xstrdup(readline(prompt));
+	 cmd_buf=lftp_readline(prompt);
 	 if(cmd_buf && *cmd_buf)
 	 {
 	    if(exec->csh_history)
 	    {
 	       char *history_value;
-	       int expanded = history_expand (cmd_buf, &history_value);
+	       int expanded = lftp_history_expand (cmd_buf, &history_value);
 
 	       if (expanded)
 	       {
@@ -114,26 +146,24 @@ public:
 		  if (expanded < 0 || expanded == 2)	/* 2 == print only */
 		  {
 		     exec->block+=NoWait();  // and retry immediately
-		     free (cmd_buf);
-		     free (history_value);
-		     return xstrdup("");
+		     xfree(history_value);
+		     return "";
 		  }
 
-		  xfree(cmd_buf);
+		  to_free=history_value;
 		  cmd_buf=history_value;
 	       }
 	    }
-	    using_history();
-	    HIST_ENTRY *temp=previous_history();
-	    if(temp==0 || strcmp(temp->line,cmd_buf))
-	       add_history(cmd_buf);
-	    using_history();
+	    lftp_add_history_nodups(cmd_buf);
 	 }
 	 else if(cmd_buf==0 && exec->interactive)
 	    puts("exit");
       }
       else
-	 cmd_buf=xstrdup(readline_from_file(stdin));
+      {
+	 to_free=readline_from_file(stdin);
+	 cmd_buf=to_free;
+      }
 
       ::completion_shell=0;
 
@@ -145,8 +175,21 @@ public:
       }
       return cmd_buf;
    }
+   void clear()
+      {
+	 if(!tty)
+	    return;
+	 lftp_rl_clear();
+      }
 };
 
+static void sig_term(int sig)
+{
+   time_t t=time(0);
+   printf(_("[%lu] Terminated by signal %d. %s"),(unsigned long)getpid(),sig,ctime(&t));
+   exit(1);
+}
+
 static void move_to_background()
 {
    fflush(stdout);
@@ -161,7 +204,7 @@ static void move_to_background()
       SignalHook::Ignore(SIGHUP);
       SignalHook::Ignore(SIGTSTP);
 
-      char *home=getenv("HOME");
+      const char *home=getenv("HOME");
       if(!home) home=".";
 
       char *log=(char*)alloca(strlen(home)+1+9+1);
@@ -174,13 +217,21 @@ static void move_to_background()
       int fd=open(log,O_WRONLY|O_APPEND|O_CREAT,0600);
       if(fd>=0)
       {
-	 dup2(fd,1);
-	 dup2(fd,2);
+	 dup2(fd,1); // stdout
+	 dup2(fd,2); // stderr
 	 if(fd!=1 && fd!=2)
 	    close(fd);
       }
+      close(0);	  // close stdin.
+      open("/dev/null",O_RDONLY); // reopen it, just in case.
+
+#ifdef HAVE_SETSID
+      setsid();	  // start a new session.
+#endif
+
       pid=getpid();
       time_t t=time(0);
+      SignalHook::Handle(SIGTERM,sig_term);
       printf(_("[%lu] Started.  %s"),(unsigned long)pid,ctime(&t));
       for(;;)
       {
@@ -203,7 +254,7 @@ static void move_to_background()
    }
 }
 
-void  source_if_exist(CmdExec *exec,char *rc)
+void  source_if_exist(CmdExec *exec,const char *rc)
 {
    if(access(rc,R_OK)!=-1)
    {
@@ -213,23 +264,45 @@ void  source_if_exist(CmdExec *exec,char *rc)
    }
 }
 
+static void tty_clear()
+{
+   top_exec->pre_stdout();
+}
+
 int   main(int argc,char **argv)
 {
+#ifdef SOCKS4
+   SOCKSinit(argv[0]);
+#endif
+
    setlocale (LC_ALL, "");
    bindtextdomain (PACKAGE, LOCALEDIR);
    textdomain (PACKAGE);
 
+   ResMgr::ClassInit(); // resources must be inited before other classes
+   SignalHook::ClassInit();
+   Resolver::ClassInit();
+
+#ifndef MODULE_PROTO_FILE
    LocalAccess::ClassInit();
+#endif
+#ifndef MODULE_PROTO_HTTP
+   Http::ClassInit();
+#endif
+#ifndef MODULE_PROTO_FTP
    Ftp::ClassInit();
+#endif
 
-   char  *home=getenv("HOME")?:".";
+   const char *home=getenv("HOME");
+   if(!home) home=".";
 
-   CmdExec *top_exec=new CmdExec(new Ftp());
-   top_exec->session->SetDebug(NULL,0);
+   top_exec=new CmdExec(new DummyProto());
    top_exec->jobno=-1;
    top_exec->status_line=new StatusLine(1);
+   Log::global=new Log();
+   Log::global->SetCB(tty_clear);
 
-   initialize_readline();
+   lftp_readline_init();
 
    hook_signals();
 
@@ -248,22 +321,24 @@ int   main(int argc,char **argv)
 
    WaitDone(top_exec);
 
-   top_exec->SetCmdFeeder(new ReadlineFeeder);
-
-   top_exec->SetInteractive(isatty(0));
+   top_exec->Fg();
 
    ArgV *args=new ArgV(argc,argv);
    args->setarg(0,"lftp");
    if(args->count()>1)
    {
-      char *line=args->Combine();
-      add_history(line);
+      char *line=args->CombineQuoted();
+      lftp_add_history_nodups(line);
       free(line);
    }
+   lftp_feeder=new ReadlineFeeder;
    top_exec->ExecParsed(args);
 
    WaitDone(top_exec);
 
+   top_exec->AtExit();
+   WaitDone(top_exec);
+
    cwd_history.Save();
 
    if(Job::NumberOfJobs()>0)
