@@ -18,6 +18,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: mgetJob.cc,v 1.7 1999/05/13 19:19:37 lav Exp $ */
+
 #include <config.h>
 #include <fcntl.h>
 #include "mgetJob.h"
@@ -27,7 +29,7 @@ void mgetJob::ShowRunStatus(StatusLine *s)
 {
    if(rg)
    {
-      s->Show(_("Getting file list [%s]"),session->CurrentStatus());
+      s->Show(rg->Status());
       return;
    }
    GetJob::ShowRunStatus(s);
@@ -37,9 +39,7 @@ void mgetJob::PrintStatus(int v)
    if(rg)
    {
       SessionJob::PrintStatus(v);
-      putchar('\t');
-      printf(_("Getting file list [%s]"),session->CurrentStatus());
-      putchar('\n');
+      printf("\t%s\n",rg->Status());
       return;
    }
    GetJob::PrintStatus(v);
@@ -78,9 +78,10 @@ mgetJob::mgetJob(FileAccess *session,ArgV *args) : GetJob(session,new ArgV(args-
    if(!p)
       goto print_usage;
 
-   rg=new RemoteGlob(session,p,Ftp::LIST);
-   rg->RestrictPath();
-   rg->Do();
+   rg=session->MakeGlob(p);
+   if(!rg)
+      rg=new NoGlob(p);
+   while(rg->Do()==MOVED);
 }
 
 int mgetJob::Do()
@@ -100,14 +101,14 @@ int mgetJob::Do()
 
    if(rg->Error())
    {
-      fprintf(stderr,"rglob: %s - %s\n",rg->pattern,session->StrError(rg->ErrorCode()));
+      fprintf(stderr,"rglob: %s\n",rg->ErrorText());
       goto next;
    }
 
    files=rg->GetResult();
    if(!files)
    {
-      fprintf(stderr,_("%s: no files found\n"),rg->pattern);
+      fprintf(stderr,_("%s: no files found\n"),rg->GetPattern());
       goto next;
    }
    for(i=files; *i; i++)
@@ -143,11 +144,12 @@ next:
       return MOVED;
    }
 
-   rg=new RemoteGlob(session,p,Ftp::LIST);
-   rg->RestrictPath();
-   rg->Do();
+   rg=session->MakeGlob(p);
+   if(!rg)
+      rg=new NoGlob(p);
+   while(rg->Do()==MOVED);
 
-   return m;
+   return MOVED;
 }
 
 mgetJob::~mgetJob()
