@@ -18,15 +18,16 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: mgetJob.h,v 1.2 1999/03/17 13:27:14 lav Exp $ */
+
 #ifndef MGETJOB_H
 #define MGETJOB_H
 
 #include "GetJob.h"
-#include "rglob.h"
 
 class mgetJob : public GetJob
 {
-   RemoteGlob *rg;
+   Glob *rg;
    ArgV *args;
 
 public:
