@@ -18,9 +18,11 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: misc.cc,v 1.11 1999/10/09 13:58:57 lav Exp $ */
+
 #include <config.h>
 #include "xmalloc.h"
-#include <string.h>
+#include "xstring.h"
 #include <stdio.h>
 #include <pwd.h>
 #include <unistd.h>
@@ -33,11 +35,19 @@
 
 const char *dir_file(const char *dir,const char *file)
 {
-   if(dir==0 || dir[0]==0 || file[0]=='/')
+   if(dir==0 || dir[0]==0)
+      return file;
+   if(file==0 || file[0]==0)
+      return dir;
+   if(file[0]=='/')
       return file;
 
    static char *buf=0;
    static int buf_size=0;
+
+   if(buf && dir==buf) // it is possible to dir_file(dir_file(dir,dir),file)
+      dir=alloca_strdup(dir);
+
    int len=strlen(dir)+1+strlen(file)+1;
    if(buf_size<len)
       buf=(char*)xrealloc(buf,buf_size=len);
@@ -52,9 +62,9 @@ const char *dir_file(const char *dir,const char *file)
 }
 
 
-char *basename_ptr(char *s)
+const char *basename_ptr(const char *s)
 {
-   char *s1=s+strlen(s);
+   const char *s1=s+strlen(s);
    while(s1>s && s1[-1]=='/')
       s1--;
    while(s1>s && s1[-1]!='/')
@@ -62,12 +72,12 @@ char *basename_ptr(char *s)
    return s1;
 }
 
-char *expand_home_relative(char *s)
+const char *expand_home_relative(const char *s)
 {
    if(s[0]=='~')
    {
-      char *home=0;
-      char *sl;
+      const char *home=0;
+      const char *sl=strchr(s+1,'/');;
       static char *ret_path=0;
 
       if(s[1]==0 || s[1]=='/')
@@ -76,12 +86,11 @@ char *expand_home_relative(char *s)
       }
       else
       {
-	 char name[9];
-	 strncpy(name,s+1,8);
-	 name[8]=0;
-	 sl=strchr(name,'/');
-	 if(sl)
-	    *sl=0;
+	 // extract user name and find the home
+	 int name_len=(sl?sl-s-1:strlen(s+1));
+	 char *name=(char*)alloca(name_len+1);
+	 strncpy(name,s+1,name_len);
+	 name[name_len]=0;
 
 	 struct passwd *pw=getpwnam(name);
 	 if(pw)
@@ -90,7 +99,6 @@ char *expand_home_relative(char *s)
       if(home==0)
 	 return s;
 
-      sl=strchr(s,'/');
       if(sl)
       {
 	 ret_path=(char*)xrealloc(ret_path,strlen(sl)+strlen(home)+1);
@@ -123,7 +131,7 @@ int   create_directories(char *path)
 	 *sl=0;
       if(access(path,0)==-1)
       {
-	 res=mkdir(path,0755);
+	 res=mkdir(path,0777);
 	 if(res==-1)
 	 {
 	    if(errno!=EEXIST)
@@ -165,3 +173,17 @@ void  truncate_file_tree(const char *dir)
       (new ProcWait(pid))->Auto();  // don't wait for termination
    }
 }
+
+char *xgetcwd()
+{
+   int size=256;
+   for(;;)
+   {
+      char *cwd=getcwd(0,size);
+      if(cwd)
+	 return cwd;
+      if(errno!=ERANGE)
+	 return 0;
+      size*=2;
+   }
+}
