@@ -18,17 +18,20 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: misc.h,v 1.5 1999/09/20 09:29:01 lav Exp $ */
+
 #ifndef MISC_H
 #define MISC_H
 
 // expands tilde; returns pointer to static data
-char *expand_home_relative(char *);
+const char *expand_home_relative(const char *);
 
 // returns ptr to last path element
-char *basename_ptr(char *);
-inline const char *basename_ptr(const char *f)
+const char *basename_ptr(const char *);
+static inline
+char *basename_ptr(char *f)
 {
-   return (const char*)basename_ptr((char*)f);
+   return (char*)basename_ptr((const char *)f);
 }
 
 // clues file to dir; returns pointer to static storage
@@ -40,4 +43,12 @@ int   create_directories(char *);
 // rm -rf
 void  truncate_file_tree(const char *dir);
 
+// returns malloc'ed cwd no matter how long it is
+// returns 0 on error.
+char *xgetcwd();
+
+#define MINUTE (60)
+#define HOUR   (60*MINUTE)
+#define DAY    (24*HOUR)
+
 #endif // MISC_H
