@@ -22,6 +22,7 @@
 
 #include <errno.h>
 #include "mkdirJob.h"
+#include "plural.h"
 
 mkdirJob::mkdirJob(FileAccess *s,ArgV *a) : SessionJob(s)
 {
@@ -67,7 +68,7 @@ int mkdirJob::Do()
       session->Mkdir(curr,opt_p);
 
    int res=session->Done();
-   if(res==Ftp::DO_AGAIN || res==Ftp::IN_PROGRESS)
+   if(res==FA::DO_AGAIN || res==FA::IN_PROGRESS)
       return STALL;
    if(res<0)
    {
@@ -91,9 +92,7 @@ void  mkdirJob::PrintStatus(int v)
 
 void  mkdirJob::ShowRunStatus(StatusLine *s)
 {
-   if(Done())
-      s->Show("");
-   else
+   if(!Done())
       s->Show("%s `%s' [%s]",args->getarg(0),curr,session->CurrentStatus());
 }
 
@@ -108,5 +107,6 @@ void  mkdirJob::SayFinal()
    else if(failed)
       printf(_("%s failed for %d of %d directories\n"),op,failed,file_count);
    else
-      printf(_("%s ok, %d directories created\n"),op,file_count);
+      printf(plural("%s ok, %d director$y|ies$ created\n",file_count),
+	    op,file_count);
 }
