@@ -24,7 +24,7 @@
 #include "alias.h"
 #include "xmalloc.h"
 #include "xalloca.h"
-#include <string.h>
+#include "xstring.h"
 
 CmdExec::parse_result CmdExec::parse_one_cmd()
 {
@@ -219,14 +219,18 @@ CmdExec::parse_result CmdExec::parse_one_cmd()
       	 if(alias)
 	 {
 	    int alias_len=strlen(alias);
-	    if(alias_field<=(int)(line-next_cmd-alias_len))
+	    if(alias_field<(int)(line-next_cmd))
 	    {
+	       // the case of previous alias ending before end of new one
 	       free_used_aliases();
 	       old_next_cmd=next_cmd;
 	    }
 	    if(!TouchedAlias::IsTouched(alias,used_aliases))
 	    {
 	       alias_field-=next_cmd-old_next_cmd;
+	       alias_field-=line-next_cmd;
+	       if(alias_field<0)
+		  alias_field=0;
 	       used_aliases=new TouchedAlias(alias,used_aliases);
 	       if(line-cmd_buf < alias_len)
 	       {
