@@ -18,6 +18,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: pgetJob.cc,v 1.9 1999/08/04 15:01:45 lav Exp $ */
+
 #include <config.h>
 #include "pgetJob.h"
 
@@ -34,6 +36,8 @@ int pgetJob::Do()
    if(no_parallel || max_chunks<2)
       return GetJob::Do();
 
+   RateDrain();
+
    if(chunks==0 || offset<chunks[0]->start)
       m=GetJob::Do();
    else
@@ -84,11 +88,13 @@ int pgetJob::Do()
 	 break;
       }
       total_xferred+=MIN(chunks[i]->offset,chunks[i]->limit)-chunks[i]->start;
-      total_xfer_rate+=chunks[i]->minute_xfer_rate;
       if(last_bytes<chunks[i]->last_bytes)
 	 last_bytes=chunks[i]->last_bytes;
       if(!chunks[i]->Done())
+      {
+	 total_xfer_rate+=chunks[i]->minute_xfer_rate;
 	 chunks_done=false;
+      }
    }
 
    if(no_parallel)
@@ -101,6 +107,7 @@ int pgetJob::Do()
    {
       if(file_time>=0)
 	 local->setmtime(file_time);
+      RemoveBackupFile();
       NextFile();
       return MOVED;
    }
@@ -122,20 +129,16 @@ void pgetJob::ShowRunStatus(StatusLine *s)
    if(!curr || !session->IsOpen())
       return;
 
-   time_t now=time(0);
-   if(now==last_status_update)
-      return;
-   last_status_update=now;
-
-   int w=s->GetWidth()-30;
+   int w=s->GetWidthDelayed()-40;
    if(w<=0)
       return;
-   char *n=curr;
+   const char *n=curr;
    if((int)strlen(n)>w)
       n=n+strlen(n)-w;
    // xgettext:c-format
-   s->Show(_("`%s', got %lu of %lu (%d%%) %s"),n,total_xferred,size,
-	 percent(total_xferred,size),CurrRate(total_xfer_rate));
+   s->Show(_("`%s', got %lu of %lu (%d%%) %s%s"),n,total_xferred,size,
+	 percent(total_xferred,size),CurrRate(total_xfer_rate),
+	 CurrETA(total_xfer_rate,total_xferred));
 }
 
 // list subjobs (chunk xfers) only when verbose
@@ -158,8 +161,9 @@ void  pgetJob::PrintStatus(int verbose)
    if(curr && session->IsOpen())
    {
       putchar('\t');
-      printf(_("`%s', got %lu of %lu (%d%%) %s"),curr,total_xferred,size,
-	    percent(total_xferred,size),CurrRate(total_xfer_rate));
+      printf(_("`%s', got %lu of %lu (%d%%) %s%s"),curr,total_xferred,size,
+	    percent(total_xferred,size),CurrRate(total_xfer_rate),
+	    CurrETA(total_xfer_rate,total_xferred));
       putchar('\n');
       if(verbose>1)
       {
@@ -195,8 +199,8 @@ void pgetJob::NextFile()
    total_xferred=0;
 }
 
-pgetJob::ChunkXfer::ChunkXfer(FileAccess *session,char *remote,FDStream *local,
-		     long start,long limit)
+pgetJob::ChunkXfer::ChunkXfer(FileAccess *session,const char *remote,
+      FDStream *local,long start,long limit)
    : XferJob(session)
 {
    this->curr=remote;
@@ -240,10 +244,10 @@ int pgetJob::ChunkXfer::Do()
    {
       if(session->IsClosed())
       {
-	 session->Open(curr,Ftp::RETRIEVE,start);
+	 session->Open(curr,FA::RETRIEVE,start);
       }
       res=TryRead(session);
-      if(res<0 && res!=Ftp::DO_AGAIN)
+      if(res<0 && res!=FA::DO_AGAIN)
       {
 	 error=true;
 	 done=true;
