@@ -18,6 +18,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: pgetJob.h,v 1.2 1999/05/20 10:20:38 lav Exp $ */
+
 #ifndef PGETJOB_H
 #define PGETJOB_H
 
@@ -37,7 +39,7 @@ class pgetJob : public GetJob
       bool no_parallel:1;
       FDStream *local;
 
-      ChunkXfer(FileAccess *session,char *remote,FDStream *local,
+      ChunkXfer(FileAccess *session,const char *remote,FDStream *local,
 	        long start,long limit);
       ~ChunkXfer();
 
