@@ -20,7 +20,7 @@
 
 #include <config.h>
 #include <stdio.h>
-#include <string.h>
+#include "xstring.h"
 #include <ctype.h>
 #include "xalloca.h"
 #include "url.h"
@@ -30,7 +30,7 @@
    CONNECT -> [USER[:PASS]@]HOST[:PORT]
 */
 
-ParsedURL::ParsedURL(const char *url)
+ParsedURL::ParsedURL(const char *url,bool proto_required)
 {
    memory=(char*)xmalloc(strlen(url)*2+1);
    strcpy(memory,url);
@@ -61,9 +61,17 @@ ParsedURL::ParsedURL(const char *url)
       *scan=0;
       scan++;
       proto=base;
-      path=scan;
-      host="localhost";
-      return;
+      memmove(scan+10,scan,strlen(scan)+1);
+      host=scan;
+      strcpy(host,"localhost");
+      path=scan+10;
+      goto decode;
+   }
+   else if(proto_required)
+   {
+      // all the rest is path, if protocol is required.
+      path=base;
+      goto decode;
    }
 
    scan=base;
@@ -108,7 +116,77 @@ ParsedURL::ParsedURL(const char *url)
 
    if(*scan==':') // port found
    {
-      *scan++=0;
-      port=scan;
+      if(strchr(scan+1,':')==0)
+      {
+	 *scan++=0;
+	 port=scan;
+      }
+      else
+      {
+	 /* more than one colon - maybe it is ipv6 digital address */
+      }
+   }
+
+decode:
+   url::decode_string(user);
+   url::decode_string(pass);
+   url::decode_string(host);
+   url::decode_string(path);
+}
+
+void url::decode_string(char *p)
+{
+   if(!p)
+      return;
+   while(*p)
+   {
+      if(*p=='%' && p[1] && p[2])
+      {
+	 int n;
+	 if(sscanf(p+1,"%2x",&n)==1)
+	 {
+	    *p++=n;
+	    memmove(p,p+2,strlen(p+2)+1);
+	    continue;
+	 }
+      }
+      p++;
    }
 }
+
+/* encode_string was taken from wget-1.5.2 and slightly modified */
+
+/* Encodes the unsafe characters (listed in URL_UNSAFE) in a given
+   string, returning a malloc-ed %XX encoded string.  */
+#define need_quote(c) (!unsafe || iscntrl((unsigned char)c) || strchr(unsafe,c))
+char *url::encode_string (const char *s,char *res,const char *unsafe)
+{
+  char *p;
+
+#if 0 // this easily leads to memory leaks.
+  int i;
+  if (res==0)
+  {
+     const char *b = s;
+     for (i = 0; *s; s++, i++)
+       if (need_quote(*s))
+	 i += 2; /* Two more characters (hex digits) */
+     res = (char *)xmalloc (i + 1);
+     s = b;
+  }
+#endif
+  for (p = res; *s; s++)
+  {
+    if (need_quote(*s))
+      {
+	const unsigned char c = *s;
+	*p++ = '%';
+	sprintf(p,"%02X",c);
+	p+=2;
+      }
+    else
+      *p++ = *s;
+  }
+  *p = '\0';
+  return res;
+}
