@@ -34,11 +34,26 @@ public:
    char	 *port;
    char  *path;
 
-   ParsedURL(const char *url);
+   ParsedURL(const char *url,bool proto_required=false);
    ~ParsedURL()
    {
       xfree(memory);
    }
 };
 
+class url
+{
+public:
+# define URL_UNSAFE " <>\"%{}|\\^[]`"
+# define URL_PATH_UNSAFE URL_UNSAFE"#;?"
+# define URL_HOST_UNSAFE URL_UNSAFE":/"
+# define URL_PORT_UNSAFE URL_UNSAFE"/"
+# define URL_USER_UNSAFE URL_UNSAFE"/:@"
+# define URL_PASS_UNSAFE URL_UNSAFE"/:@"
+   // encode unsafe chars as %XY
+   static char *encode_string(const char *,char *buf=0,const char *u=URL_UNSAFE);
+   // reverse; done in-place.
+   static void decode_string(char *);
+};
+
 #endif//URL_H
