@@ -18,14 +18,17 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: xmalloc.cc,v 1.4 1999/05/22 12:41:27 lav Exp $ */
+
 #include <config.h>
 #include <stdio.h>
 #include "xmalloc.h"
 
-static void memory_error_and_abort(char *fname)
+static void memory_error_and_abort(const char *fname,size_t size)
 {
-  fprintf (stderr, "%s: out of virtual memory\n", fname);
-  exit (2);
+   fprintf(stderr,"%s: out of virtual memory when trying to get %lu bytes\n",
+	 fname,(long)size);
+   exit(2);
 }
 
 void *xmalloc (size_t bytes)
@@ -34,8 +37,11 @@ void *xmalloc (size_t bytes)
       return 0;
    void *temp=(void*)malloc(bytes);
    if(temp==0)
-      memory_error_and_abort("xmalloc");
-   return (temp);
+      memory_error_and_abort("xmalloc",bytes);
+#ifdef MEM_DEBUG
+   printf("xmalloc %p %lu\n",temp,(long)bytes);
+#endif
+   return(temp);
 }
 
 void *xrealloc(void *pointer,size_t bytes)
@@ -53,6 +59,20 @@ void *xrealloc(void *pointer,size_t bytes)
    else
       temp=(void*)realloc(pointer,bytes);
    if(temp==0)
-      memory_error_and_abort ("xrealloc");
+      memory_error_and_abort ("xrealloc",bytes);
+#ifdef MEM_DEBUG
+   printf("xrealloc %p %p %lu\n",pointer,temp,(long)bytes);
+#endif
    return(temp);
 }
+
+void xfree(void *p)
+{
+   if(p)
+   {
+#ifdef MEM_DEBUG
+      printf("xfree %p\n",p);
+#endif
+      free(p);
+   }
+}
