@@ -19,11 +19,13 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/* $Id: xmalloc.h,v 1.5 1999/05/22 12:41:27 lav Exp $ */
+
 #ifndef XMALLOC_H
 #define XMALLOC_H
 
-#include <string.h>
 #include <stdlib.h>
+#include "xstring.h"
 #include "xalloca.h"
 
 #ifdef DBMALLOC
@@ -42,11 +44,10 @@ static inline char *xstrdup(const char *s)
 static inline void *xmemdup(const void *m,int len)
 {
    if(!m) return 0;
-   return memcpy(xmalloc(len),m,len);
-}
-static inline void xfree(void *p)
-{
-   if(p) free(p);
+   void *buf=xmalloc(len);
+   memcpy(buf,m,len);
+   return buf;
 }
+void xfree(void *p);
 
 #endif /* XMALLOC_H */
